Package: fntl
Title: Numerical Tools for 'Rcpp' and Lambda Functions
Version: 0.1.2
Authors@R: 
    person("Andrew M.", "Raim", email = "andrew.raim@gmail.com",
        role = c("aut", "cre"), comment = c(ORCID = "0000-0002-4440-2330"))
Description: 
    Provides a 'C++' API for routinely used numerical tools such as integration,
    root-finding, and optimization, where function arguments are given as
    lambdas. This facilitates 'Rcpp' programming, enabling the development of
    'R'-like code in 'C++' where functions can be defined on the fly and use
    variables in the surrounding environment.
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.2
URL: https://github.com/andrewraim/fntl
Depends: R (>= 4.3)
Suggests: knitr, numDeriv, rmarkdown, quarto, tidyverse, tinytest
LinkingTo: Rcpp
Imports: Rcpp
VignetteBuilder: quarto
NeedsCompilation: yes
Packaged: 2024-11-12 17:56:15 UTC; araim
Author: Andrew M. Raim [aut, cre] (<https://orcid.org/0000-0002-4440-2330>)
Maintainer: Andrew M. Raim <andrew.raim@gmail.com>
Repository: CRAN
Date/Publication: 2024-11-12 19:00:02 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-26 18:22:49 UTC; unix
Archs: fntl.so.dSYM

Package: fntl
Title: Numerical Tools for 'Rcpp' and Lambda Functions
Version: 0.1.3.2
Authors@R: 
    person("Andrew M.", "Raim", email = "andrew.raim@gmail.com",
        role = c("aut", "cre"), comment = c(ORCID = "0000-0002-4440-2330"))
Description: 
    Provides a 'C++' API for routinely used numerical tools such as integration,
    root-finding, and optimization, where function arguments are given as
    lambdas. This facilitates 'Rcpp' programming, enabling the development of
    'R'-like code in 'C++' where functions can be defined on the fly and use
    variables in the surrounding environment.
License: MIT + file LICENSE
Encoding: UTF-8
URL: https://github.com/andrewraim/fntl
Depends: R (>= 4.3)
Suggests: knitr, Matrix, numDeriv, rmarkdown, quarto, RcppArmadillo,
        tidyverse, tinytest
LinkingTo: Rcpp
Imports: Rcpp
VignetteBuilder: quarto
Config/roxygen2/version: 8.1.0
NeedsCompilation: yes
Packaged: 2026-08-26 12:55:08 UTC; araim
Author: Andrew M. Raim [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-4440-2330>)
Maintainer: Andrew M. Raim <andrew.raim@gmail.com>
Repository: CRAN
Date/Publication: 2026-08-28 10:00:08 UTC

# Generated by roxygen2: do not edit by hand

export(bfgs1)
export(bfgs2)
export(bfgs_args)
export(cg1)
export(cg2)
export(cg_args)
export(col_apply)
export(d_trunc)
export(deriv1)
export(deriv2)
export(fd_deriv1)
export(fd_deriv2)
export(findroot_args)
export(findroot_bisect)
export(findroot_brent)
export(goldensection)
export(gradient0)
export(hessian0)
export(integrate0)
export(integrate_args)
export(jacobian0)
export(lbfgsb1)
export(lbfgsb2)
export(lbfgsb_args)
export(mat_apply)
export(neldermead)
export(neldermead_args)
export(nlm1)
export(nlm2)
export(nlm3)
export(nlm_args)
export(optimize_args)
export(optimize_brent)
export(outer1)
export(outer1_matvec)
export(outer1_triplet)
export(outer2)
export(outer2_matvec)
export(outer2_triplet)
export(p_trunc)
export(q_trunc)
export(r_trunc)
export(richardson_args)
export(row_apply)
export(solve_cg)
export(which0)
import(Rcpp)
importFrom(Rcpp,evalCpp)
useDynLib(fntl, .registration = TRUE)

# Introduction

`fntl` is an R package to facilitate Rcpp programming. It provides a C++ API
for routinely used numerical tools such as integration, root-finding, and
optimization, where function arguments are given as lambdas. This enables the
development of R-like code in C++ where functions can be defined on the fly
and use variables in the surrounding environment.

See the included vignette for a more in-depth discussion of the package and an
API guide.

# Installation

The `fntl` package may be installed directly from Github using a standard R
command like the following.

```r
devtools::install_github("andrewraim/fntl", ref = "v0.1.0")
```

Here, `v0.1.0` represents a tagged release; replace it with a later version if
one exists.


# Getting Started

The following example computes the integral

$$
B(a,b) = \int_0^1 x^{a-1} (1-x)^{b-1} dx.
$$

Create the file `example.cpp` with the following contents.

```cpp
// [[Rcpp::depends(fntl)]]
#include "fntl.h"

// [[Rcpp::export]]
Rcpp::List example(double a, double b)
{
    fntl::dfd f = [&](double x) {
        return std::pow(x, a - 1) * std::pow(1 - x, b - 1);
    };
    auto out = fntl::integrate(f, 0, 1);
    return Rcpp::wrap(out);
}
```

The `example` function may be called through R as follows.

```r
Rcpp::sourceCpp("example.cpp")
example(2, 3)
```

## -----------------------------------------------------------------------------
#| include: false
library(fntl)
library(tidyverse)

set.seed(1234)


## -----------------------------------------------------------------------------
mu_true = 0
x = rnorm(n = 200, mean = mu_true, sd = 1)
loglik = function(mu) { sum(dnorm(x, mu, 1, log = TRUE)) }
optimize(loglik, lower = -100, upper = 100, maximum = TRUE)


## auto f = [](double x, double y) -> double { return x*y; };

## auto f = [](double x, double y) { return x*y; };

## Rcpp::NumericVector x = Rcpp::rnorm(200);
## auto loglik = [&](double mu) {
##     double out = Rcpp::sum(Rcpp::dnorm(x, mu, 1, true));
## 	return out;
## };

## std::function<double(double)> loglik = [&](double mu) {
##     double out = Rcpp::sum(Rcpp::dnorm(x, mu, 1, true));
## 	return out;
## };

## -----------------------------------------------------------------------------
Rcpp::sourceCpp("examples/first.cpp")
out = first_ex(2, 3)
print(out$value)


## // Create args and export to a List
## fntl::integrate_args args0;
## Rcpp::List x = Rcpp::wrap(args0);
## 
## // Instantiate a second args struct from the list x
## x["stop_on_error"] = false;
## fntl::integrate_args args1 = Rcpp::as<fntl::integrate_args>(x);

## // This will cause an exception
## x["abcdefg"] = 0;
## fntl::integrate_args args1 = Rcpp::as<fntl::integrate_args>(x);

## fntl::integrate_result out = fntl::integrate(f, 0, 1);

## fntl::integrate_result out = fntl::integrate(f, 0, 1, args);
## Rcpp::List x = Rcpp::wrap(out);

## fntl::integrate_status status = fntl::integrate_status::OK;  // Define a status
## int err = to_underlying(status);                             // status to int
## status1 = fntl::integrate_status(err);                       // int to status

## typedef function<double(double)> dfd;
## typedef function<double(const NumericVector&)> dfv;
## typedef function<double(const NumericVector&, const NumericVector&)> dfvv;
## typedef function<NumericVector(const NumericVector&)> vfv;
## typedef function<NumericMatrix(const NumericVector&)> mfv;

## double mach_eps = std::numeric_limits<double>::epsilon();
## double mach_eps_2r = sqrt(mach_eps);
## double mach_eps_4r = std::pow(mach_eps, 0.25);
## unsigned int uint_max = std::numeric_limits<unsigned int>::max();

## enum class error_action : unsigned int {
## 	STOP = 3L,     // <1>
## 	WARNING = 2L,  // <2>
## 	MESSAGE = 1L,  // <3>
## 	NONE = 0L      // <4>
## };

## // [[Rcpp::depends(fntl)]]
## #include "fntl.h"
## 
## // [[Rcpp::export]]
## Rcpp::List crash_ex(Rcpp::NumericVector x0)
## {
## 	fntl::dfv f = [](Rcpp::NumericVector x) { return Rcpp::sum(x*x); };
## 	auto out = fntl::gradient(f, x0);
## 	return Rcpp::wrap(out);
## }

## fntl::dfv f = [](Rcpp::NumericVector x) -> double { return Rcpp::sum(x*x); };

## fntl::dfv f = [](Rcpp::NumericVector x) {
## 	double out = Rcpp::sum(x*x);
## 	return out;
## };

## -----------------------------------------------------------------------------
Rcpp::sourceCpp("examples/callr.cpp")
a = 2
b = 3
f = function(x) { x^(a - 1) * (1 - x)^(b - 1) }
out = callr_ex(f)
print(out$value)


## -----------------------------------------------------------------------------
args = integrate_args()
print(args)
a = 2; b = 3
f = function(x) { x^(a-1) * (1-x)^(b-1) }
out = integrate0(f, 0, 1, args)
print(out$value)


## -----------------------------------------------------------------------------
#| eval: false
# source("examples/timing1.R")
# Rcpp::sourceCpp("examples/timing2.cpp")
# Rcpp::sourceCpp("examples/timing3.cpp")
# Rcpp::sourceCpp("examples/timing4.cpp")
# n_levels = c(100, 200, 500, 1000, 10000)


## -----------------------------------------------------------------------------
#| eval: false
# set.seed(1234)
# start = Sys.time()
# timing1_ex(R = 200, n_levels)
# print(Sys.time() - start)


## -----------------------------------------------------------------------------
#| eval: false
# set.seed(1234)
# start = Sys.time()
# timing2_ex(R = 200, n_levels)
# print(Sys.time() - start)


## -----------------------------------------------------------------------------
#| eval: false
# set.seed(1234)
# start = Sys.time()
# timing3_ex(R = 200, n_levels)
# print(Sys.time() - start)


## -----------------------------------------------------------------------------
#| eval: false
# set.seed(1234)
# start = Sys.time()
# timing4_ex(R = 200, n_levels)
# print(Sys.time() - start)


## double sum1p(Rcpp::NumericMatrix x) { return Rcpp::sum(x + 1); }

## double sum1p_1(Rcpp::NumericMatrix& x) { return Rcpp::sum(x + 1); }

## double sum1p_2(const Rcpp::NumericMatrix& x) { return Rcpp::sum(x + 1); }

## integrate_result integrate(
## 	const dfd& f,        // <1>
## 	double lower,                // <2>
## 	double upper,                // <3>
##     const integrate_args& args   // <4>
## )
## 
## integrate_result integrate(
## 	const dfd& f,        // <1>
## 	double lower,                // <2>
## 	double upper                 // <3>
## )

## struct integrate_args {
## 	unsigned int subdivisions = 100L;  // <1>
## 	double rel_tol = mach_eps_4r;      // <2>
## 	double abs_tol = mach_eps_4r;      // <3>
## 	bool stop_on_error = true;         // <4>
## };

## struct integrate_result {
## 	double value;             // <1>
## 	double abs_error;         // <2>
## 	int subdivisions;         // <3>
## 	integrate_status status;  // <4>
## 	int n_eval;               // <5>
## 	std::string message;      // <6>
## 
## 	operator SEXP() const;    // <7>
## };

## enum class integrate_status : int {
## 	OK = 0L,                                  // <1>
## 	MAX_SUBDIVISIONS = 1L,                    // <2>
## 	ROUNDOFF_ERROR = 2L,                      // <3>
## 	BAD_INTEGRAND_BEHAVIOR = 3L,              // <4>
## 	ROUNDOFF_ERROR_EXTRAPOLATION_TABLE = 4L,  // <5>
## 	PROBABLY_DIVERGENT_INTEGRAL = 5L,         // <6>
## 	INVALID_INPUT = 6L                        // <7>
## };

## -----------------------------------------------------------------------------
Rcpp::sourceCpp("examples/integrate.cpp")
out = integrate_ex(0 , Inf)
print(out)


## double fd_deriv(
## 	const dfv& f,                                  // <1>
## 	const Rcpp::NumericVector& x,                  // <2>
## 	unsigned int i,                                // <3>
## 	double h,                                      // <5>
## 	const fd_types& fd_type = fd_types::SYMMETRIC  // <7>
## )
## 
## double fd_deriv2(
## 	const dfv& f,                                  // <1>
## 	const Rcpp::NumericVector& x,                  // <2>
## 	unsigned int i,                                // <3>
## 	unsigned int j,                                // <4>
## 	double h_i,                                    // <5>
## 	double h_j,                                    // <6>
## 	const fd_types& fd_type = fd_types::SYMMETRIC  // <7>
## )

## enum class fd_types : unsigned int {
## 	SYMMETRIC = 0L,
## 	FORWARD = 1L,
## 	BACKWARD = 2L
## };

## -----------------------------------------------------------------------------
n = 3
b = seq_len(n)
x0 = rep(0.5, n)
eps = 0.001  ## Fix a step size for finite differences
g = function(x) { b * cos(sum(b * x)) }
h = function(x) { -tcrossprod(b) * sin(sum(b * x)) }


## -----------------------------------------------------------------------------
Rcpp::sourceCpp("examples/fd-deriv.cpp")
out3 = out2 = out1 = numeric(n)
for (i in 1:n) {
	out1[i] = fd_deriv_ex(x0, i-1, eps, type = 0)  ## Symmetric
	out2[i] = fd_deriv_ex(x0, i-1, eps, type = 1)  ## Forward
	out3[i] = fd_deriv_ex(x0, i-1, eps, type = 2)  ## Backward
}
print(out1)
print(out2)
print(out3)
print(g(x0))


## -----------------------------------------------------------------------------
Rcpp::sourceCpp("examples/fd-deriv2.cpp")
out3 = out2 = out1 = matrix(NA, n, n)
for (i in 1:n) {
for (j in 1:n) {
	out1[i,j] = fd_deriv2_ex(x0, i-1, j-1, eps, eps, type = 0)  ## Symmetric
	out2[i,j] = fd_deriv2_ex(x0, i-1, j-1, eps, eps, type = 1)  ## Forward
	out3[i,j] = fd_deriv2_ex(x0, i-1, j-1, eps, eps, type = 2)  ## Backward
}
}
print(out1)
print(out2)
print(out3)
h(x0)


## richardson_result deriv(
## 	const dfv& f,                                  // <1>
## 	const Rcpp::NumericVector& x,                  // <2>
## 	unsigned int i,                                // <3>
## 	const richardson_args& args,                   // <5>
## 	const fd_types& fd_type = fd_types::SYMMETRIC  // <6>
## )
## 
## richardson_result deriv(
## 	const dfv& f,                                  // <1>
## 	const Rcpp::NumericVector& x,                  // <2>
## 	unsigned int i,                                // <3>
## 	const fd_types& fd_type = fd_types::SYMMETRIC  // <6>
## )
## 
## richardson_result deriv2(
## 	const dfv& f,                                  // <1>
## 	const Rcpp::NumericVector& x,                  // <2>
## 	unsigned int i,                                // <3>
## 	unsigned int j,                                // <4>
## 	const richardson_args& args,                   // <5>
## 	const fd_types& fd_type = fd_types::SYMMETRIC  // <6>
## )
## 
## richardson_result deriv2(
## 	const dfv& f,                                  // <1>
## 	const Rcpp::NumericVector& x,                  // <2>
## 	unsigned int i,                                // <3>
## 	unsigned int j,                                // <4>
## 	const fd_types& fd_type = fd_types::SYMMETRIC  // <6>
## )

## struct richardson_args
## {
## 	double delta = 0.5;                 // <1>
## 	unsigned int maxiter = 10;          // <2>
## 	double h = 1;                       // <3>
## 	double tol = mach_eps_4r;           // <4>
## 	double accuracy_factor = R_PosInf;  // <5>
## 
## 	richardson_args() { };              // <6>
## 	richardson_args(SEXP obj);          // <7>
## 	operator SEXP() const;              // <8>
## };
## 

## struct richardson_result
## {
## 	double value;              // <1>
## 	double err;                // <2>
## 	unsigned int iter;         // <3>
## 	richardson_status status;  // <4>
## 
## 	operator SEXP() const;     // <5>
## };

## enum class richardson_status : unsigned int {
## 	OK = 0L,                  // <1>
## 	NOT_CONVERGED = 1L,       // <2>
## 	NUMERICAL_PRECISION = 2L  // <3>
## };

## -----------------------------------------------------------------------------
n = 3
x0 = rep(0.5, n)


## -----------------------------------------------------------------------------
Rcpp::sourceCpp("examples/deriv.cpp")
out3 = out2 = out1 = numeric(n)
for (i in 1:n) {
	out1[i] = deriv_ex(x0, i-1, type = 0)$value  ## Symmetric
	out2[i] = deriv_ex(x0, i-1, type = 1)$value  ## Forward
	out3[i] = deriv_ex(x0, i-1, type = 2)$value  ## Backward
}
print(out1)
print(out2)
print(out3)


## -----------------------------------------------------------------------------
Rcpp::sourceCpp("examples/deriv2.cpp")
out3 = out2 = out1 = matrix(NA, n, n)
for (i in 1:n) {
for (j in 1:n) {
	out1[i,j] = deriv2_ex(x0, i-1, j-1, type = 0)$value  ## Symmetric
	out2[i,j] = deriv2_ex(x0, i-1, j-1, type = 1)$value  ## Forward
	out3[i,j] = deriv2_ex(x0, i-1, j-1, type = 2)$value  ## Backward
}
}
print(out1)
print(out2)
print(out3)


## gradient_result gradient(
## 	const dfv& f,                                  // <1>
## 	const Rcpp::NumericVector& x,                  // <2>
## 	const richardson_args& args,                   // <3>
## 	const fd_types& fd_type = fd_types::SYMMETRIC  // <4>
## )
## 
## gradient_result gradient(
## 	const dfv& f,                                  // <1>
## 	const Rcpp::NumericVector& x,                  // <2>
## 	const fd_types& fd_type = fd_types::SYMMETRIC  // <4>
## )

## struct gradient_result {
## 	std::vector<double> value;       // <1>
## 	std::vector<double> err;         // <2>
## 	std::vector<unsigned int> iter;  // <3>
## 
## 	operator SEXP() const;           // <4>
## };

## -----------------------------------------------------------------------------
Rcpp::sourceCpp("examples/gradient.cpp")
gradient_ex(x0 = 1:4)


## -----------------------------------------------------------------------------
f = function(x) { sum(x^2) }
numDeriv::grad(f, x = 1:4)


## jacobian_result jacobian(
## 	const vfv& f,                                 // <1>
## 	const Rcpp::NumericVector& x,                 // <2>
## 	const richardson_args& args                   // <3>
## 	const fd_types& fd_type = fd_types::SYMMETRIC // <4>
## )
## 
## jacobian_result jacobian(
## 	const vfv& f,                                 // <1>
## 	const Rcpp::NumericVector& x,                 // <2>
## 	const fd_types& fd_type = fd_types::SYMMETRIC // <4>
## )

## struct jacobian_result
## {
## 	std::vector<double> value;       // <1>
## 	std::vector<double> err;         // <2>
## 	std::vector<unsigned int> iter;  // <3>
## 	double rows;                     // <4>
## 	double cols;                     // <5>
## 
## 	operator SEXP() const;           // <6>
## };

## -----------------------------------------------------------------------------
Rcpp::sourceCpp("examples/jacobian.cpp")
out = jacobian_ex(x0 = 1:4)
names(out)
print(out$value)


## -----------------------------------------------------------------------------
f = function(x) { cumsum(sin(x)) }
numDeriv::jacobian(f, x = 1:4)


## hessian_result hessian(
## 	const dfv& f,                                  // <1>
## 	const Rcpp::NumericVector& x,                  // <2>
## 	const richardson_args& args,                   // <3>
## 	const fd_types& fd_type = fd_types::SYMMETRIC  // <4>
## )
## 
## hessian_result hessian(
## 	const dfv& f,                                  // <1>
## 	const Rcpp::NumericVector& x,                  // <2>
## 	const fd_types& fd_type = fd_types::SYMMETRIC  // <4>
## )

## struct hessian_result
## {
## 	std::vector<double> value;       // <1>
## 	std::vector<double> err;         // <2>
## 	std::vector<unsigned int> iter;  // <3>
## 	double dim;                      // <4>
## 
## 	operator SEXP() const;           // <5>
## };

## -----------------------------------------------------------------------------
Rcpp::sourceCpp("examples/hessian.cpp")
out = hessian_ex(x0 = c(1,2))
print(out)


## -----------------------------------------------------------------------------
f = function(x) { sum(sin(x)) }
numDeriv::hessian(f, x = c(1,2))


## struct findroot_args {
## 	double tol = mach_eps_4r;                      // <1>
## 	unsigned int maxiter = 1000;                   // <2>
## 	error_action action = error_action::STOP;      // <3>
## 	
## 	findroot_args() { };                           // <4>
## 	findroot_args(SEXP obj);                       // <5>
## 	operator SEXP() const;                         // <6>
## };

## struct findroot_result {
## 	double root;                 // <1>
## 	double f_root;               // <2>
## 	unsigned int iter;           // <3>
## 	double tol;                  // <4>
## 	findroot_status status;      // <5>
## 	std::string message;         // <6>
## 
## 	operator SEXP() const;       // <7>
## };

## enum class findroot_status : unsigned int {
## 	OK = 0L,                  // <1>
## 	NUMERICAL_OVERFLOW = 1L,  // <2>
## 	NOT_CONVERGED = 2L        // <3>
## };

## findroot_result findroot_bisect(
## 	const dfd& f,              // <1>
## 	double lower,              // <2>
## 	double upper,              // <3>
## 	const findroot_args& args  // <4>
## )
## 
## findroot_result findroot_bisect(
## 	const dfd& f,              // <1>
## 	double lower,              // <2>
## 	double upper               // <3>
## )

## -----------------------------------------------------------------------------
Rcpp::sourceCpp("examples/findroot-bisect.cpp")
out = findroot_bisect_ex(0, 10)
print(out)


## findroot_result findroot_brent(
## 	const dfd& f,              // <1>
## 	double lower,              // <2>
## 	double upper,              // <3>
## 	const findroot_args& args  // <4>
## )
## 
## findroot_result findroot_brent(
## 	const dfd& f,              // <1>
## 	double lower,              // <2>
## 	double upper               // <3>
## )

## -----------------------------------------------------------------------------
Rcpp::sourceCpp("examples/findroot-brent.cpp")
out = findroot_brent_ex(0, 10)
print(out)


## struct optimize_args
## {
## 	bool fnscale = 1;                          // <1>
## 	double tol = mach_eps_2r;                  // <2>
## 	unsigned int maxiter = 1000;               // <3>
## 	unsigned int report_period = uint_max;     // <4>
## 	error_action action = error_action::STOP;  // <5>
## 
## 	optimize_args() { };                       // <6>
## 	optimize_args(SEXP obj);                   // <7>
## 	operator SEXP() const;                     // <8>
## };
## 
## 

## struct optimize_result
## {
## 	double par;                   // <1>
## 	double value;                 // <2>
## 	unsigned int iter;            // <3>
## 	double tol;                   // <4>
## 	optimize_status status;       // <5>
## 	std::string message;          // <6>
## 
## 	operator SEXP() const;        // <7>
## };

## enum class optimize_status : unsigned int {
## 	OK = 0L,                  // <1>
## 	NUMERICAL_OVERFLOW = 1L,  // <2>
## 	NOT_CONVERGED = 2L        // <3>
## };

## optimize_result goldensection(
## 	const dfd& f,              // <1>
## 	double lower,              // <2>
## 	double upper,              // <3>
## 	const optimize_args& args  // <4>
## )
## 
## optimize_result goldensection(
## 	const dfd& f,              // <1>
## 	double lower,              // <2>
## 	double upper               // <3>
## )

## -----------------------------------------------------------------------------
Rcpp::sourceCpp("examples/goldensection.cpp")
out = goldensection_ex(0, 1)
print(out)


## optimize_result optimize_brent(
## 	const dfd& f,              // <1>
## 	double lower,              // <2>
## 	double upper,              // <3>
## 	const optimize_args& args  // <4>
## )
## 
## optimize_result optimize_brent(
## 	const dfd& f,              // <1>
## 	double lower,              // <2>
## 	double upper               // <3>
## )

## -----------------------------------------------------------------------------
Rcpp::sourceCpp("examples/optimize-brent.cpp")
out = optimize_brent_ex(0, 1)
print(out)


## neldermead_result neldermead(
## 	const Rcpp::NumericVector& init,  // <1>
## 	const dfv& f,                     // <2>
## 	const neldermead_args& args       // <3>
## )
## 
## neldermead_result neldermead(
## 	const Rcpp::NumericVector& init,  // <1>
## 	const dfv& f                      // <2>
## )

## struct neldermead_args
## {
## 	double alpha = 1.0;           // <1>
## 	double beta = 0.5;            // <2>
## 	double gamma = 2.0;           // <3>
## 	unsigned int trace = 0;       // <4>
## 	double abstol = R_NegInf;     // <5>
## 	double reltol = mach_eps_2r;  // <6>
## 	unsigned int maxit = 500;     // <7>
## 	double fnscale = 1.0;         // <8>
## 
## 	neldermead_args() { };        // <9>
## 	neldermead_args(SEXP obj);    // <10>
## 	operator SEXP() const;        // <11>
## };

## struct neldermead_result {
## 	std::vector<double> par;   // <1>
## 	double value;              // <2>
## 	neldermead_status status;  // <3>
## 	int fncount;               // <4>
## 
## 	operator SEXP() const;     // <5>
## };

## enum class neldermead_status : unsigned int {
## 	OK = 0L,                 // <1>
## 	NOT_CONVERGED = 1L,      // <2>
## 	SIMLEX_DEGENERACY = 10L  // <3>
## };

## -----------------------------------------------------------------------------
Rcpp::sourceCpp("examples/neldermead.cpp")
out = neldermead_ex(x0 = c(1, -1))
print(out)


## bfgs_result bfgs(
## 	const Rcpp::NumericVector& init,  // <1>
## 	const dfv& f,                     // <2>
## 	const vfv& g,                     // <3>
## 	const bfgs_args& args             // <4>
## )
## 
## bfgs_result bfgs(
## 	const Rcpp::NumericVector& init,  // <1>
## 	const dfv& f,                     // <2>
## 	const bfgs_args& args             // <4>
## )
## 
## bfgs_result bfgs(
## 	const Rcpp::NumericVector& init,  // <1>
## 	const dfv& f,                     // <2>
## 	const vfv& g                      // <3>
## )
## 
## bfgs_result bfgs(
## 	const Rcpp::NumericVector& init,  // <1>
## 	const dfv& f                      // <2>
## )

## struct bfgs_args {
## 	richardson_args deriv_args;    // <1>
## 	double parscale = 1;           // <2>
## 	int trace = 0;                 // <3>
## 	double fnscale = 1;            // <4>
## 	int maxit = 100;               // <5>
## 	int report = 10;               // <6>
## 	double abstol = R_NegInf;      // <7>
## 	double reltol = mach_eps_2r;   // <8>
## 
## 	bfgs_args() { };               // <9>
## 	bfgs_args(SEXP obj);           // <10>
## 	operator SEXP() const;         // <11>
## };

## struct bfgs_result {
## 	std::vector<double> par;  // <1>
## 	double value;             // <2>
## 	bfgs_status status;       // <3>
## 	int fncount;              // <4>
## 	int grcount;              // <5>
## 
## 	operator SEXP() const;    // <6>
## };

## enum class bfgs_status : unsigned int {
## 	OK = 0L,             // <1>
## 	NOT_CONVERGED = 1L   // <2>
## };

## -----------------------------------------------------------------------------
Rcpp::sourceCpp("examples/bfgs.cpp")
out = bfgs_ex(x0 = rep(1, 4))
print(out$numerical)
print(out$analytical)


## lbfgsb_result lbfgsb(
## 	const Rcpp::NumericVector& init,  // <1>
## 	const dfv& f,                     // <2>
## 	const vfv& g,                     // <3>
## 	const lbfgsb_args& args           // <4>
## )
## 
## lbfgsb_result lbfgsb(
## 	const Rcpp::NumericVector& init,  // <1>
## 	const dfv& f,                     // <2>
## 	const lbfgsb_args& args           // <4>
## )
## 
## lbfgsb_result lbfgsb(
## 	const Rcpp::NumericVector& init,  // <1>
## 	const dfv& f,                     // <2>
## 	const vfv& g                      // <3>
## )
## 
## lbfgsb_result lbfgsb(
## 	const Rcpp::NumericVector& init,  // <1>
## 	const dfv& f                      // <2>
## )

## struct lbfgsb_args {
## 	std::vector<double> lower;     // <1>
## 	std::vector<double> upper;     // <2>
## 	richardson_args deriv_args;    // <3>
## 	double parscale = 1;           // <4>
## 	int trace = 0;                 // <5>
## 	double fnscale = 1;            // <6>
## 	int lmm = 5;                   // <7>
## 	int maxit = 100;               // <8>
## 	int report = 10;               // <9>
## 	double factr = 1e7;            // <10>
## 	double pgtol = 0;              // <11>
## 
## 	lbfgsb_args() { };             // <12>
## 	lbfgsb_args(SEXP obj);         // <13>
## 	operator SEXP() const;         // <14>
## };

## struct lbfgsb_result {
## 	std::vector<double> par;  // <1>
## 	double value;             // <2>
## 	lbfgsb_status status;     // <3>
## 	int fncount;              // <4>
## 	int grcount;              // <5>
## 	std::string msg;          // <6>
## 
## 	operator SEXP() const;    // <7>
## };

## enum class lbfgsb_status : unsigned int {
## 	OK = 0L,             // <1>
## 	NOT_CONVERGED = 1L,  // <2>
## 	WARN = 51L,          // <3>
## 	ERROR = 52L,         // <4>
## };

## -----------------------------------------------------------------------------
Rcpp::sourceCpp("examples/lbfgsb.cpp")
out = lbfgsb_ex(x0 = rep(1, 4))
print(out$numerical)
print(out$analytical)


## cg_result bfgs(
## 	const Rcpp::NumericVector& init,  // <1>
## 	const dfv& f,                     // <2>
## 	const vfv& g,                     // <3>
## 	const cg_args& args               // <4>
## )
## 
## cg_result cg(
## 	const Rcpp::NumericVector& init,  // <1>
## 	const dfv& f,                     // <2>
## 	const cg_args& args               // <4>
## )
## 
## cg_result cg(
## 	const Rcpp::NumericVector& init,  // <1>
## 	const dfv& f,                     // <2>
## 	const vfv& g                      // <3>
## )
## 
## cg_result cg(
## 	const Rcpp::NumericVector& init,  // <1>
## 	const dfv& f                      // <2>
## )

## 
## struct cg_args
## {
## 	richardson_args deriv_args;    // <1>
## 	double parscale = 1;           // <2>
## 	double fnscale = 1;            // <3>
## 	double abstol = R_NegInf;      // <4>
## 	double reltol = mach_eps_2r;   // <5>
## 	int type = 1;                  // <6>
## 	int trace = 0;                 // <7>
## 	int maxit = 100;               // <8>
## 
## 	cg_args() { };                 // <9>
## 	cg_args(SEXP obj);             // <10>
## 	operator SEXP() const;         // <11>
## };

## struct cg_result {
## 	std::vector<double> par;  // <1>
## 	double value;             // <2>
## 	cg_status status;         // <3>
## 	int fncount;              // <4>
## 	int grcount;              // <5>
## 
## 	operator SEXP() const;    // <6>
## };

## enum class cg_status : unsigned int {
## 	OK = 0L,             // <1>
## 	NOT_CONVERGED = 1L   // <2>
## };

## -----------------------------------------------------------------------------
Rcpp::sourceCpp("examples/cg.cpp")
out = cg_ex(x0 = rep(1, 4))
print(out$numerical)
print(out$analytical)


## 
## nlm_result nlm(
## 	const Rcpp::NumericVector& init,  // <1>
## 	const dfv& f,                     // <2>
## 	const vfv& g,                     // <3>
## 	const mfv& h,                     // <4>
## 	const nlm_args& args              // <5>
## )
## 
## nlm_result nlm(
## 	const Rcpp::NumericVector& init,  // <1>
## 	const dfv& f,                     // <2>
## 	const vfv& g,                     // <3>
## 	const nlm_args& args              // <5>
## )
## 
## nlm_result nlm(
## 	const Rcpp::NumericVector& init,  // <1>
## 	const dfv& f,                     // <2>
## 	const nlm_args& args              // <5>
## )
## 
## nlm_result nlm(
## 	const Rcpp::NumericVector& init,  // <1>
## 	const dfv& f,                     // <2>
## 	const vfv& g,                     // <3>
## 	const mfv& h                      // <4>
## )
## 
## nlm_result nlm(
## 	const Rcpp::NumericVector& init,  // <1>
## 	const dfv& f,                     // <2>
## 	const vfv& g                      // <3>
## )
## 
## nlm_result nlm(
## 	const Rcpp::NumericVector& init,  // <1>
## 	const dfv& f                      // <2>
## )

## struct nlm_args
## {
## 	std::vector<double> typsize;   // <1>
## 	unsigned int print_level = 0;  // <2>
## 	double fscale = 1;             // <3>
## 	double fnscale = 1;            // <4>
## 	unsigned int ndigit = 12;      // <5>
## 	double gradtol = 1e-6;         // <6>
## 	double stepmax = R_PosInf;     // <7>
## 	double steptol = 1e-6;         // <8>
## 	int iterlim = 100;             // <9>
## 	unsigned int method = 1;       // <10>
## 	double trust_radius = 1.0;     // <11>
## 
## 	nlm_args() { };                // <12>
## 	nlm_args(SEXP obj);            // <13>
## 	operator SEXP() const;         // <14>
## };

## struct nlm_result
## {
## 	std::vector<double> par;      // <1>
## 	std::vector<double> grad;     // <2>
## 	double estimate;              // <3>
## 	int iterations;               // <4>
## 	nlm_status status;            // <5>
## 
## 	operator SEXP() const;        // <6>
## };

## enum class nlm_status : unsigned int {
## 	OK = 0L,                   // <1>
## 	GRADIENT_WITHIN_TOL = 1L,  // <2>
## 	ITERATES_WITH_TOL = 2L,    // <3>
## 	NO_LOWER_STEP = 3L,        // <4>
## 	ITERATION_MAX = 4L,        // <5>
## 	STEP_SIZE_EXCEEDED = 5L    // <6>
## };
## 

## -----------------------------------------------------------------------------
Rcpp::sourceCpp("examples/nlm.cpp")
out = nlm_ex(x0 = rep(1, 4))
nn = c("par", "grad")
print(out$res1[nn])
print(out$res2[nn])
print(out$res3[nn])


## -----------------------------------------------------------------------------
#| eval: false
# apply(X, c(1,2), f)
# apply(X, 1, f)
# apply(X, 2, f)


## template <typename T, int RTYPE>
## Rcpp::Vector<RTYPE> row_apply(
## 	const Rcpp::Matrix<RTYPE>& X,							// <1>
## 	const std::function<T(const Rcpp::Vector<RTYPE>&)>& f   // <2>
## )
## 
## template <typename T, int RTYPE>
## Rcpp::Vector<RTYPE> col_apply(
## 	const Rcpp::Matrix<RTYPE>& X,							// <1>
## 	const std::function<T(const Rcpp::Vector<RTYPE>&)>& f   // <2>
## )
## 
## template <typename T, int RTYPE>
## Rcpp::Matrix<RTYPE> mat_apply(
## 	const Rcpp::Matrix<RTYPE>& X,							// <1>
## 	const std::function<T(T)>& f                            // <2>
## )

## -----------------------------------------------------------------------------
Rcpp::sourceCpp("examples/apply.cpp")
X = matrix(1:12, 4, 3)
out = apply_ex(X)
print(out)


## Rcpp::NumericMatrix outer(
## 	const Rcpp::NumericMatrix& X,  // <1>
## 	const dfvv& f                  // <3>
## )
## 
## Rcpp::NumericMatrix outer(
## 	const Rcpp::NumericMatrix& X,  // <1>
## 	const Rcpp::NumericMatrix& Y,  // <2>
## 	const dfvv& f                  // <3>
## )
## 
## Rcpp::NumericVector outer_matvec(
## 	const Rcpp::NumericMatrix& X,  // <1>
## 	const dfvv& f,                 // <3>
## 	const Rcpp::NumericVector& a   // <4>
## )
## 
## Rcpp::NumericVector outer_matvec(
## 	const Rcpp::NumericMatrix& X,  // <1>
## 	const Rcpp::NumericMatrix& Y,  // <2>
## 	const dfvv& f,                 // <3>
## 	const Rcpp::NumericVector& a   // <4>
## )

## -----------------------------------------------------------------------------
Rcpp::sourceCpp("examples/outer.cpp")
m = 5; n = 3; d = 2
X = matrix(rnorm(10), m, d)
Y = matrix(rnorm(6), n, d)
a = rep(1, m)
b = rep(1, n)
out = outer_ex(X, Y, a, b)
print(out)


## cg_result solve_cg(
## 	const vfv& l,                     // <1>
## 	const Rcpp::NumericVector& b,     // <2>
## 	const Rcpp::NumericVector& init,  // <3>
## 	const cg_args& args               // <4>
## )
## 
## cg_result solve_cg(
## 	const vfv& l,                     // <1>
## 	const Rcpp::NumericVector& b,     // <2>
## 	const Rcpp::NumericVector& init   // <3>
## )
## 
## cg_result solve_cg(
## 	const vfv& l,                     // <1>
## 	const Rcpp::NumericVector& b      // <2>
## )

## -----------------------------------------------------------------------------
Rcpp::sourceCpp("examples/solve-cg.cpp")
b = rep(1, 10)
out = solve_cg_ex(b)
print(out)


## -----------------------------------------------------------------------------
A = matrix(0, 10, 10)
diag(A) = 2
A[cbind(1:9, 2:10)] = 1
A[cbind(2:10, 1:9)] = 1
solve(A, b)


## -----------------------------------------------------------------------------
#| eval: false
# which(f(X), arr.ind = TRUE)


## template <typename T, int RTYPE>
## Rcpp::IntegerMatrix which(
## 	const Rcpp::Matrix<RTYPE>& X,      // <1>
## 	const std::function<bool(T)>& f),  // <2>
## 	bool one_based = false             // <3>
## )

## -----------------------------------------------------------------------------
Rcpp::sourceCpp("examples/which.cpp")
x = runif(10, -1, 1)
X = matrix(x, 2, 5)
out = which_ex(X)
print(X)
print(out)


## -----------------------------------------------------------------------------
f = function(x) { x > 0 & x < 0.5 }
which(f(X), arr.ind = TRUE) - 1


## typedef std::function<double(double,bool)> density;       // <1>
## typedef std::function<double(double,bool,bool)> cdf;      // <2>
## typedef std::function<double(double,bool,bool)> quantile; // <3>

## double d_trunc(
## 	double x,                      // <1>
## 	double lo,                     // <2>
## 	double hi,                     // <3>
## 	const density& f,              // <4>
## 	const cdf& F,                  // <5>
## 	bool log = false               // <6>
## )
## 
## Rcpp::NumericVector d_trunc(
## 	const Rcpp::NumericVector& x,  // <1>
## 	const Rcpp::NumericVector& lo, // <2>
## 	const Rcpp::NumericVector& hi, // <3>
## 	const density& f,              // <4>
## 	const cdf& F,                  // <5>
## 	bool log = false               // <6>
## )

## -----------------------------------------------------------------------------
Rcpp::sourceCpp("examples/d_trunc.cpp")
x = seq(0, 1, length.out = 30)
d_trunc_ex(x, shape1 = 2, shape2 = 5, lo = 0.5, hi = 0.95)


## double p_trunc(
## 	double x,              // <1>
## 	double lo,             // <2>
## 	double hi,             // <3>
## 	const cdf& F,          // <4>
## 	bool lower = true,     // <5>
## 	bool log = false       // <6>
## )
## 
## Rcpp::NumericVector p_trunc(
## 	const Rcpp::NumericVector& x,  // <1>
## 	const Rcpp::NumericVector& lo, // <2>
## 	const Rcpp::NumericVector& hi, // <3>
## 	const cdf& F,                  // <4>
## 	bool lower = true,             // <5>
## 	bool log = false               // <6>
## )

## -----------------------------------------------------------------------------
Rcpp::sourceCpp("examples/p_trunc.cpp")
x = seq(0, 1, length.out = 30)
p_trunc_ex(x, shape1 = 2, shape2 = 5, lo = 0.5, hi = 0.95)


## double q_trunc(
## 	double p,              // <1>
## 	double lo,             // <2>
## 	double hi,             // <3>
## 	const cdf& F,          // <4>
## 	const quantile& Finv,  // <5>
## 	bool lower = true,     // <6>
## 	bool log = false       // <7>
## )
## 
## Rcpp::NumericVector q_trunc(
## 	const Rcpp::NumericVector& p,  // <1>
## 	const Rcpp::NumericVector& lo, // <2>
## 	const Rcpp::NumericVector& hi, // <3>
## 	const cdf& F,                  // <4>
## 	const quantile& Finv,          // <5>
## 	bool lower = true,             // <6>
## 	bool log = false               // <7>
## )

## -----------------------------------------------------------------------------
Rcpp::sourceCpp("examples/q_trunc.cpp")
p = seq(0, 1, length.out = 30)
q_trunc_ex(p, shape1 = 2, shape2 = 5, lo = 0.5, hi = 0.95)


## double r_trunc(
## 	double lo,             // <2>
## 	double hi,             // <3>
## 	const cdf& F,          // <4>
## 	const quantile& Finv   // <5>
## )
## 
## Rcpp::NumericVector r_trunc(
## 	unsigned int n,                // <1>
## 	const Rcpp::NumericVector& lo, // <2>
## 	const Rcpp::NumericVector& hi, // <3>
## 	const cdf& F,                  // <4>
## 	const quantile& Finv           // <5>
## )
## 

## -----------------------------------------------------------------------------
Rcpp::sourceCpp("examples/r_trunc.cpp")
r_trunc_ex(n = 20, shape1 = 2, shape2 = 5, lo = 0.5, hi = 0.95)

library(fntl)

Rcpp::sourceCpp("cpp/test-multivariate-optimization.cpp")
set.seed(1234)

# ----- Likelihood maximization with each method -----

# Maximize likelihood of a sample from N(mu, sigma2)
n = 200
mu = 3
sigma2 = 10
tol = 0.01

x = rnorm(n, mu, sqrt(sigma2))

loglik = function(par) {
	mu = par[1]
	sigma2 = exp(par[2])
	sum(dnorm(x, mu, sqrt(sigma2), TRUE))
}
numDeriv::grad(func = loglik, c(0,0))

mu_hat = mean(x)
sigma2_hat = (n-1) / n * var(x)
par_hat = c(mu_hat, log(sigma2_hat))
ll_hat = loglik(par_hat)

out = neldermead_mle(x)
expect_equal(out$par[1], mu_hat, tolerance = tol)
expect_equal(exp(out$par[2]), sigma2_hat, tolerance = tol)
expect_equal(out$value, ll_hat, tolerance = tol)

out = lbfgsb_mle(x)
expect_equal(out$par[1], mu_hat, tolerance = tol)
expect_equal(exp(out$par[2]), sigma2_hat, tolerance = tol)
expect_equal(out$value, ll_hat, tolerance = tol)

out = bfgs_mle(x)
expect_equal(out$par[1], mu_hat, tolerance = tol)
expect_equal(exp(out$par[2]), sigma2_hat, tolerance = tol)
expect_equal(out$value, ll_hat, tolerance = tol)

out = cg_mle(x)
expect_equal(out$par[1], mu_hat, tolerance = tol)
expect_equal(exp(out$par[2]), sigma2_hat, tolerance = tol)
expect_equal(out$value, ll_hat, tolerance = tol)

out = nlm_mle(x)
expect_equal(out$par[1], mu_hat, tolerance = tol)
expect_equal(exp(out$par[2]), sigma2_hat, tolerance = tol)
expect_equal(out$estimate, ll_hat, tolerance = tol)

# ----- Norm minimization with each method -----

# Minimize norm2(x - a) for a given a.
a = c(-1, 0, 1)

out = neldermead_norm2(a)
expect_equal(out$par, a, tolerance = tol)
expect_equal(out$value, 0, tolerance = tol)

out = lbfgsb_norm2(a)
expect_equal(out$par, a, tolerance = tol)
expect_equal(out$value, 0, tolerance = tol)

out = bfgs_norm2(a)
expect_equal(out$par, a, tolerance = tol)
expect_equal(out$value, 0, tolerance = tol)

out = cg_norm2(a)
expect_equal(out$par, a, tolerance = tol)
expect_equal(out$value, 0, tolerance = tol)

out = nlm_norm2(a)
expect_equal(out$par, a, tolerance = tol)
expect_equal(out$estimate, 0, tolerance = tol)

timing1_ex = function(R, n_levels)
{
	beta0_true = 0
	beta1_true = 1

	out = numeric(length(n_levels))
	for (k in seq_along(n_levels)) {
		n = n_levels[k]
		x = rnorm(n)
		pr_true = plogis(beta0_true + beta1_true * x)
		beta_hats = matrix(NA, R, 2)
		beta_trues = matrix(NA, R, 2)

		for (r in seq_len(R)) {
			# Generate a sample
			y = rbinom(n, size = 1, prob = pr_true)

			# A slow loglikelihood function
			loglik = function(par) {
				out = 0
				for (i in 1:n) {
					pr_i = plogis(par[1] + par[2] * x[i])
					out = out + y[i] * log(pr_i) + (1 - y[i]) * log1p(-pr_i)
				}
				return(out)
			}

			# Compute the MLE using L-BFGS-B
			init = c(0, 0)
			ctrl = list(fnscale = -1)
			optim_out = optim(init, loglik, method = "L-BFGS-B", control = ctrl)

			# Save estimates
			beta_hats[r,] = optim_out$par
			beta_trues[r,] = c(beta0_true, beta1_true)
		}

		out[k] = mean((beta_hats - beta_trues)^2)
	}

	return(out)
}

## -----------------------------------------------------------------------------
#| include: false
library(fntl)
library(tidyverse)

set.seed(1234)


## -----------------------------------------------------------------------------
mu_true = 0
x = rnorm(n = 200, mean = mu_true, sd = 1)
loglik = function(mu) { sum(dnorm(x, mu, 1, log = TRUE)) }
optimize(loglik, lower = -100, upper = 100, maximum = TRUE)


## auto f = [](double x, double y) -> double { return x*y; };

## auto f = [](double x, double y) { return x*y; };

## Rcpp::NumericVector x = Rcpp::rnorm(200);
## auto loglik = [&](double mu) {
##     double out = Rcpp::sum(Rcpp::dnorm(x, mu, 1, true));
## 	return out;
## };

## std::function<double(double)> loglik = [&](double mu) {
##     double out = Rcpp::sum(Rcpp::dnorm(x, mu, 1, true));
## 	return out;
## };

## -----------------------------------------------------------------------------
#| eval: false
#| prompt: true
# file.path(path.package("fntl"), "doc", "examples")


## -----------------------------------------------------------------------------
Rcpp::sourceCpp("examples/first.cpp")
out = first_ex(2, 3)
print(out$value)


## // Create args and export to a list
## fntl::integrate_args args0;
## Rcpp::List x = Rcpp::wrap(args0);
## 
## // Instantiate a second args struct from the list x
## x["stop_on_error"] = false;
## fntl::integrate_args args1 = Rcpp::as<fntl::integrate_args>(x);

## // This will cause an exception
## x["abcdefg"] = 0;
## fntl::integrate_args args1 = Rcpp::as<fntl::integrate_args>(x);

## fntl::integrate_result out = fntl::integrate(f, 0, 1);

## fntl::integrate_result out = fntl::integrate(f, 0, 1, args);
## Rcpp::List x = Rcpp::wrap(out);

## fntl::integrate_status status = fntl::integrate_status::OK;  // Define a status
## unsigned int err = static_cast<unsigned int>(status);        // status to int
## status1 = fntl::integrate_status(err);                       // int to status

## typedef function<double(double)> dfd;
## typedef function<double(const NumericVector&)> dfv;
## typedef function<double(const NumericVector&, const NumericVector&)> dfvv;
## typedef function<bool(const NumericVector&, const NumericVector&)> bfvv;
## typedef function<NumericVector(const NumericVector&)> vfv;
## typedef function<NumericMatrix(const NumericVector&)> mfv;

## double mach_eps = std::numeric_limits<double>::epsilon();
## double mach_eps_2r = sqrt(mach_eps);
## double mach_eps_4r = std::pow(mach_eps, 0.25);
## unsigned int uint_max = std::numeric_limits<unsigned int>::max();

## enum class error_action : unsigned int {
## 	STOP = 3L,     // <1>
## 	WARNING = 2L,  // <2>
## 	MESSAGE = 1L,  // <3>
## 	NONE = 0L      // <4>
## };

## // [[Rcpp::depends(fntl)]]
## #include "fntl.h"
## 
## // [[Rcpp::export]]
## Rcpp::List crash_ex(Rcpp::NumericVector x0)
## {
## 	fntl::dfv f = [](Rcpp::NumericVector x) { return Rcpp::sum(x*x); };
## 	auto out = fntl::gradient(f, x0);
## 	return Rcpp::wrap(out);
## }

## fntl::dfv f = [](Rcpp::NumericVector x) -> double { return Rcpp::sum(x*x); };

## fntl::dfv f = [](Rcpp::NumericVector x) {
## 	double out = Rcpp::sum(x*x);
## 	return out;
## };

## -----------------------------------------------------------------------------
Rcpp::sourceCpp("examples/callr.cpp")
a = 2
b = 3
f = function(x) { x^(a - 1) * (1 - x)^(b - 1) }
out = callr_ex(f)
print(out$value)


## -----------------------------------------------------------------------------
args = integrate_args()
print(args)
a = 2; b = 3
f = function(x) { x^(a-1) * (1-x)^(b-1) }
out = integrate0(f, 0, 1, args)
print(out$value)


## -----------------------------------------------------------------------------
#| eval: false
# source("examples/timing1.R")
# Rcpp::sourceCpp("examples/timing2.cpp")
# Rcpp::sourceCpp("examples/timing3.cpp")
# Rcpp::sourceCpp("examples/timing4.cpp")
# n_levels = c(100, 200, 500, 1000, 10000)


## -----------------------------------------------------------------------------
#| eval: false
# set.seed(1234)
# start = Sys.time()
# timing1_ex(R = 200, n_levels)
# print(Sys.time() - start)


## -----------------------------------------------------------------------------
#| eval: false
# set.seed(1234)
# start = Sys.time()
# timing2_ex(R = 200, n_levels)
# print(Sys.time() - start)


## -----------------------------------------------------------------------------
#| eval: false
# set.seed(1234)
# start = Sys.time()
# timing3_ex(R = 200, n_levels)
# print(Sys.time() - start)


## -----------------------------------------------------------------------------
#| eval: false
# set.seed(1234)
# start = Sys.time()
# timing4_ex(R = 200, n_levels)
# print(Sys.time() - start)


## double sum1p(Rcpp::NumericMatrix x) { return Rcpp::sum(x + 1); }

## double sum1p_1(Rcpp::NumericMatrix& x) { return Rcpp::sum(x + 1); }

## double sum1p_2(const Rcpp::NumericMatrix& x) { return Rcpp::sum(x + 1); }

## -----------------------------------------------------------------------------
# Generate data
n = 200
mu = numeric(n)
y = rnorm(n, mu, sqrt(2))

Rcpp::sourceCpp("examples/arma.cpp")
arma_example(y, mu)


## integrate_result integrate(
## 	const dfd& f,        // <1>
## 	double lower,                // <2>
## 	double upper,                // <3>
##     const integrate_args& args   // <4>
## )
## 
## integrate_result integrate(
## 	const dfd& f,        // <1>
## 	double lower,                // <2>
## 	double upper                 // <3>
## )

## struct integrate_args {
## 	unsigned int subdivisions = 100L;  // <1>
## 	double rel_tol = mach_eps_4r;      // <2>
## 	double abs_tol = mach_eps_4r;      // <3>
## 	bool stop_on_error = true;         // <4>
## };

## struct integrate_result {
## 	double value;             // <1>
## 	double abs_error;         // <2>
## 	int subdivisions;         // <3>
## 	integrate_status status;  // <4>
## 	int n_eval;               // <5>
## 	std::string message;      // <6>
## 
## 	operator SEXP() const;    // <7>
## };

## enum class integrate_status : int {
## 	OK = 0L,                                  // <1>
## 	MAX_SUBDIVISIONS = 1L,                    // <2>
## 	ROUNDOFF_ERROR = 2L,                      // <3>
## 	BAD_INTEGRAND_BEHAVIOR = 3L,              // <4>
## 	ROUNDOFF_ERROR_EXTRAPOLATION_TABLE = 4L,  // <5>
## 	PROBABLY_DIVERGENT_INTEGRAL = 5L,         // <6>
## 	INVALID_INPUT = 6L                        // <7>
## };

## -----------------------------------------------------------------------------
Rcpp::sourceCpp("examples/integrate.cpp")
out = integrate_ex(0 , Inf)
print(out)


## double fd_deriv(
## 	const dfv& f,                                  // <1>
## 	const Rcpp::NumericVector& x,                  // <2>
## 	unsigned int i,                                // <3>
## 	double h,                                      // <5>
## 	const fd_types& fd_type = fd_types::SYMMETRIC  // <7>
## )
## 
## double fd_deriv2(
## 	const dfv& f,                                  // <1>
## 	const Rcpp::NumericVector& x,                  // <2>
## 	unsigned int i,                                // <3>
## 	unsigned int j,                                // <4>
## 	double h_i,                                    // <5>
## 	double h_j,                                    // <6>
## 	const fd_types& fd_type = fd_types::SYMMETRIC  // <7>
## )

## enum class fd_types : unsigned int {
## 	SYMMETRIC = 0L,
## 	FORWARD = 1L,
## 	BACKWARD = 2L
## };

## -----------------------------------------------------------------------------
n = 3
b = seq_len(n)
x0 = rep(0.5, n)
eps = 0.001  ## Fix a step size for finite differences
g = function(x) { b * cos(sum(b * x)) }
h = function(x) { -tcrossprod(b) * sin(sum(b * x)) }


## -----------------------------------------------------------------------------
Rcpp::sourceCpp("examples/fd-deriv.cpp")
out3 = out2 = out1 = numeric(n)
for (i in 1:n) {
	out1[i] = fd_deriv_ex(x0, i-1, eps, type = 0)  ## Symmetric
	out2[i] = fd_deriv_ex(x0, i-1, eps, type = 1)  ## Forward
	out3[i] = fd_deriv_ex(x0, i-1, eps, type = 2)  ## Backward
}
print(out1)
print(out2)
print(out3)
print(g(x0))


## -----------------------------------------------------------------------------
Rcpp::sourceCpp("examples/fd-deriv2.cpp")
out3 = out2 = out1 = matrix(NA, n, n)
for (i in 1:n) {
for (j in 1:n) {
	out1[i,j] = fd_deriv2_ex(x0, i-1, j-1, eps, eps, type = 0)  ## Symmetric
	out2[i,j] = fd_deriv2_ex(x0, i-1, j-1, eps, eps, type = 1)  ## Forward
	out3[i,j] = fd_deriv2_ex(x0, i-1, j-1, eps, eps, type = 2)  ## Backward
}
}
print(out1)
print(out2)
print(out3)
h(x0)


## richardson_result deriv(
## 	const dfv& f,                                  // <1>
## 	const Rcpp::NumericVector& x,                  // <2>
## 	unsigned int i,                                // <3>
## 	const richardson_args& args,                   // <5>
## 	const fd_types& fd_type = fd_types::SYMMETRIC  // <6>
## )
## 
## richardson_result deriv(
## 	const dfv& f,                                  // <1>
## 	const Rcpp::NumericVector& x,                  // <2>
## 	unsigned int i,                                // <3>
## 	const fd_types& fd_type = fd_types::SYMMETRIC  // <6>
## )
## 
## richardson_result deriv2(
## 	const dfv& f,                                  // <1>
## 	const Rcpp::NumericVector& x,                  // <2>
## 	unsigned int i,                                // <3>
## 	unsigned int j,                                // <4>
## 	const richardson_args& args,                   // <5>
## 	const fd_types& fd_type = fd_types::SYMMETRIC  // <6>
## )
## 
## richardson_result deriv2(
## 	const dfv& f,                                  // <1>
## 	const Rcpp::NumericVector& x,                  // <2>
## 	unsigned int i,                                // <3>
## 	unsigned int j,                                // <4>
## 	const fd_types& fd_type = fd_types::SYMMETRIC  // <6>
## )

## struct richardson_args
## {
## 	double delta = 0.5;                 // <1>
## 	unsigned int maxiter = 10;          // <2>
## 	double h = 1;                       // <3>
## 	double tol = mach_eps_4r;           // <4>
## 	double accuracy_factor = R_PosInf;  // <5>
## 
## 	richardson_args() { };              // <6>
## 	richardson_args(SEXP obj);          // <7>
## 	operator SEXP() const;              // <8>
## };
## 

## struct richardson_result
## {
## 	double value;              // <1>
## 	double err;                // <2>
## 	unsigned int iter;         // <3>
## 	richardson_status status;  // <4>
## 
## 	operator SEXP() const;     // <5>
## };

## enum class richardson_status : unsigned int {
## 	OK = 0L,                  // <1>
## 	NOT_CONVERGED = 1L,       // <2>
## 	NUMERICAL_PRECISION = 2L  // <3>
## };

## -----------------------------------------------------------------------------
n = 3
x0 = rep(0.5, n)


## -----------------------------------------------------------------------------
Rcpp::sourceCpp("examples/deriv.cpp")
out3 = out2 = out1 = numeric(n)
for (i in 1:n) {
	out1[i] = deriv_ex(x0, i-1, type = 0)$value  ## Symmetric
	out2[i] = deriv_ex(x0, i-1, type = 1)$value  ## Forward
	out3[i] = deriv_ex(x0, i-1, type = 2)$value  ## Backward
}
print(out1)
print(out2)
print(out3)


## -----------------------------------------------------------------------------
Rcpp::sourceCpp("examples/deriv2.cpp")
out3 = out2 = out1 = matrix(NA, n, n)
for (i in 1:n) {
for (j in 1:n) {
	out1[i,j] = deriv2_ex(x0, i-1, j-1, type = 0)$value  ## Symmetric
	out2[i,j] = deriv2_ex(x0, i-1, j-1, type = 1)$value  ## Forward
	out3[i,j] = deriv2_ex(x0, i-1, j-1, type = 2)$value  ## Backward
}
}
print(out1)
print(out2)
print(out3)


## gradient_result gradient(
## 	const dfv& f,                                  // <1>
## 	const Rcpp::NumericVector& x,                  // <2>
## 	const richardson_args& args,                   // <3>
## 	const fd_types& fd_type = fd_types::SYMMETRIC  // <4>
## )
## 
## gradient_result gradient(
## 	const dfv& f,                                  // <1>
## 	const Rcpp::NumericVector& x,                  // <2>
## 	const fd_types& fd_type = fd_types::SYMMETRIC  // <4>
## )

## struct gradient_result {
## 	std::vector<double> value;       // <1>
## 	std::vector<double> err;         // <2>
## 	std::vector<unsigned int> iter;  // <3>
## 
## 	operator SEXP() const;           // <4>
## };

## -----------------------------------------------------------------------------
Rcpp::sourceCpp("examples/gradient.cpp")
gradient_ex(x0 = 1:4)


## -----------------------------------------------------------------------------
f = function(x) { sum(x^2) }
numDeriv::grad(f, x = 1:4)


## jacobian_result jacobian(
## 	const vfv& f,                                 // <1>
## 	const Rcpp::NumericVector& x,                 // <2>
## 	const richardson_args& args                   // <3>
## 	const fd_types& fd_type = fd_types::SYMMETRIC // <4>
## )
## 
## jacobian_result jacobian(
## 	const vfv& f,                                 // <1>
## 	const Rcpp::NumericVector& x,                 // <2>
## 	const fd_types& fd_type = fd_types::SYMMETRIC // <4>
## )

## struct jacobian_result
## {
## 	std::vector<double> value;       // <1>
## 	std::vector<double> err;         // <2>
## 	std::vector<unsigned int> iter;  // <3>
## 	double rows;                     // <4>
## 	double cols;                     // <5>
## 
## 	operator SEXP() const;           // <6>
## };

## -----------------------------------------------------------------------------
Rcpp::sourceCpp("examples/jacobian.cpp")
out = jacobian_ex(x0 = 1:4)
names(out)
print(out$value)


## -----------------------------------------------------------------------------
f = function(x) { cumsum(sin(x)) }
numDeriv::jacobian(f, x = 1:4)


## hessian_result hessian(
## 	const dfv& f,                                  // <1>
## 	const Rcpp::NumericVector& x,                  // <2>
## 	const richardson_args& args,                   // <3>
## 	const fd_types& fd_type = fd_types::SYMMETRIC  // <4>
## )
## 
## hessian_result hessian(
## 	const dfv& f,                                  // <1>
## 	const Rcpp::NumericVector& x,                  // <2>
## 	const fd_types& fd_type = fd_types::SYMMETRIC  // <4>
## )

## struct hessian_result
## {
## 	std::vector<double> value;       // <1>
## 	std::vector<double> err;         // <2>
## 	std::vector<unsigned int> iter;  // <3>
## 	double dim;                      // <4>
## 
## 	operator SEXP() const;           // <5>
## };

## -----------------------------------------------------------------------------
Rcpp::sourceCpp("examples/hessian.cpp")
out = hessian_ex(x0 = c(1,2))
print(out)


## -----------------------------------------------------------------------------
f = function(x) { sum(sin(x)) }
numDeriv::hessian(f, x = c(1,2))


## struct findroot_args {
## 	double tol = mach_eps_4r;                      // <1>
## 	unsigned int maxiter = 1000;                   // <2>
## 	error_action action = error_action::STOP;      // <3>
## 	
## 	findroot_args() { };                           // <4>
## 	findroot_args(SEXP obj);                       // <5>
## 	operator SEXP() const;                         // <6>
## };

## struct findroot_result {
## 	double root;                 // <1>
## 	double f_root;               // <2>
## 	unsigned int iter;           // <3>
## 	double tol;                  // <4>
## 	findroot_status status;      // <5>
## 	std::string message;         // <6>
## 
## 	operator SEXP() const;       // <7>
## };

## enum class findroot_status : unsigned int {
## 	OK = 0L,                  // <1>
## 	NUMERICAL_OVERFLOW = 1L,  // <2>
## 	NOT_CONVERGED = 2L        // <3>
## };

## findroot_result findroot_bisect(
## 	const dfd& f,              // <1>
## 	double lower,              // <2>
## 	double upper,              // <3>
## 	const findroot_args& args  // <4>
## )
## 
## findroot_result findroot_bisect(
## 	const dfd& f,              // <1>
## 	double lower,              // <2>
## 	double upper               // <3>
## )

## -----------------------------------------------------------------------------
Rcpp::sourceCpp("examples/findroot-bisect.cpp")
out = findroot_bisect_ex(0, 10)
print(out)


## findroot_result findroot_brent(
## 	const dfd& f,              // <1>
## 	double lower,              // <2>
## 	double upper,              // <3>
## 	const findroot_args& args  // <4>
## )
## 
## findroot_result findroot_brent(
## 	const dfd& f,              // <1>
## 	double lower,              // <2>
## 	double upper               // <3>
## )

## -----------------------------------------------------------------------------
Rcpp::sourceCpp("examples/findroot-brent.cpp")
out = findroot_brent_ex(0, 10)
print(out)


## struct optimize_args
## {
## 	bool fnscale = 1;                          // <1>
## 	double tol = mach_eps_2r;                  // <2>
## 	unsigned int maxiter = 1000;               // <3>
## 	unsigned int report_period = uint_max;     // <4>
## 	error_action action = error_action::STOP;  // <5>
## 
## 	optimize_args() { };                       // <6>
## 	optimize_args(SEXP obj);                   // <7>
## 	operator SEXP() const;                     // <8>
## };
## 
## 

## struct optimize_result
## {
## 	double par;                   // <1>
## 	double value;                 // <2>
## 	unsigned int iter;            // <3>
## 	double tol;                   // <4>
## 	optimize_status status;       // <5>
## 	std::string message;          // <6>
## 
## 	operator SEXP() const;        // <7>
## };

## enum class optimize_status : unsigned int {
## 	OK = 0L,                  // <1>
## 	NUMERICAL_OVERFLOW = 1L,  // <2>
## 	NOT_CONVERGED = 2L        // <3>
## };

## optimize_result goldensection(
## 	const dfd& f,              // <1>
## 	double lower,              // <2>
## 	double upper,              // <3>
## 	const optimize_args& args  // <4>
## )
## 
## optimize_result goldensection(
## 	const dfd& f,              // <1>
## 	double lower,              // <2>
## 	double upper               // <3>
## )

## -----------------------------------------------------------------------------
Rcpp::sourceCpp("examples/goldensection.cpp")
out = goldensection_ex(0, 1)
print(out)


## optimize_result optimize_brent(
## 	const dfd& f,              // <1>
## 	double lower,              // <2>
## 	double upper,              // <3>
## 	const optimize_args& args  // <4>
## )
## 
## optimize_result optimize_brent(
## 	const dfd& f,              // <1>
## 	double lower,              // <2>
## 	double upper               // <3>
## )

## -----------------------------------------------------------------------------
Rcpp::sourceCpp("examples/optimize-brent.cpp")
out = optimize_brent_ex(0, 1)
print(out)


## neldermead_result neldermead(
## 	const Rcpp::NumericVector& init,  // <1>
## 	const dfv& f,                     // <2>
## 	const neldermead_args& args       // <3>
## )
## 
## neldermead_result neldermead(
## 	const Rcpp::NumericVector& init,  // <1>
## 	const dfv& f                      // <2>
## )

## struct neldermead_args
## {
## 	double alpha = 1.0;           // <1>
## 	double beta = 0.5;            // <2>
## 	double gamma = 2.0;           // <3>
## 	unsigned int trace = 0;       // <4>
## 	double abstol = R_NegInf;     // <5>
## 	double reltol = mach_eps_2r;  // <6>
## 	unsigned int maxit = 500;     // <7>
## 	double fnscale = 1.0;         // <8>
## 
## 	neldermead_args() { };        // <9>
## 	neldermead_args(SEXP obj);    // <10>
## 	operator SEXP() const;        // <11>
## };

## struct neldermead_result {
## 	std::vector<double> par;   // <1>
## 	double value;              // <2>
## 	neldermead_status status;  // <3>
## 	int fncount;               // <4>
## 
## 	operator SEXP() const;     // <5>
## };

## enum class neldermead_status : unsigned int {
## 	OK = 0L,                 // <1>
## 	NOT_CONVERGED = 1L,      // <2>
## 	SIMLEX_DEGENERACY = 10L  // <3>
## };

## -----------------------------------------------------------------------------
Rcpp::sourceCpp("examples/neldermead.cpp")
out = neldermead_ex(x0 = c(1, -1))
print(out)


## bfgs_result bfgs(
## 	const Rcpp::NumericVector& init,  // <1>
## 	const dfv& f,                     // <2>
## 	const vfv& g,                     // <3>
## 	const bfgs_args& args             // <4>
## )
## 
## bfgs_result bfgs(
## 	const Rcpp::NumericVector& init,  // <1>
## 	const dfv& f,                     // <2>
## 	const bfgs_args& args             // <4>
## )
## 
## bfgs_result bfgs(
## 	const Rcpp::NumericVector& init,  // <1>
## 	const dfv& f,                     // <2>
## 	const vfv& g                      // <3>
## )
## 
## bfgs_result bfgs(
## 	const Rcpp::NumericVector& init,  // <1>
## 	const dfv& f                      // <2>
## )

## struct bfgs_args {
## 	richardson_args deriv_args;    // <1>
## 	double parscale = 1;           // <2>
## 	int trace = 0;                 // <3>
## 	double fnscale = 1;            // <4>
## 	int maxit = 100;               // <5>
## 	int report = 10;               // <6>
## 	double abstol = R_NegInf;      // <7>
## 	double reltol = mach_eps_2r;   // <8>
## 
## 	bfgs_args() { };               // <9>
## 	bfgs_args(SEXP obj);           // <10>
## 	operator SEXP() const;         // <11>
## };

## struct bfgs_result {
## 	std::vector<double> par;  // <1>
## 	double value;             // <2>
## 	bfgs_status status;       // <3>
## 	int fncount;              // <4>
## 	int grcount;              // <5>
## 
## 	operator SEXP() const;    // <6>
## };

## enum class bfgs_status : unsigned int {
## 	OK = 0L,             // <1>
## 	NOT_CONVERGED = 1L   // <2>
## };

## -----------------------------------------------------------------------------
Rcpp::sourceCpp("examples/bfgs.cpp")
out = bfgs_ex(x0 = rep(1, 4))
print(out$numerical)
print(out$analytical)


## lbfgsb_result lbfgsb(
## 	const Rcpp::NumericVector& init,  // <1>
## 	const dfv& f,                     // <2>
## 	const vfv& g,                     // <3>
## 	const lbfgsb_args& args           // <4>
## )
## 
## lbfgsb_result lbfgsb(
## 	const Rcpp::NumericVector& init,  // <1>
## 	const dfv& f,                     // <2>
## 	const lbfgsb_args& args           // <4>
## )
## 
## lbfgsb_result lbfgsb(
## 	const Rcpp::NumericVector& init,  // <1>
## 	const dfv& f,                     // <2>
## 	const vfv& g                      // <3>
## )
## 
## lbfgsb_result lbfgsb(
## 	const Rcpp::NumericVector& init,  // <1>
## 	const dfv& f                      // <2>
## )

## struct lbfgsb_args {
## 	std::vector<double> lower;     // <1>
## 	std::vector<double> upper;     // <2>
## 	richardson_args deriv_args;    // <3>
## 	double parscale = 1;           // <4>
## 	int trace = 0;                 // <5>
## 	double fnscale = 1;            // <6>
## 	int lmm = 5;                   // <7>
## 	int maxit = 100;               // <8>
## 	int report = 10;               // <9>
## 	double factr = 1e7;            // <10>
## 	double pgtol = 0;              // <11>
## 
## 	lbfgsb_args() { };             // <12>
## 	lbfgsb_args(SEXP obj);         // <13>
## 	operator SEXP() const;         // <14>
## };

## struct lbfgsb_result {
## 	std::vector<double> par;  // <1>
## 	double value;             // <2>
## 	lbfgsb_status status;     // <3>
## 	int fncount;              // <4>
## 	int grcount;              // <5>
## 	std::string msg;          // <6>
## 
## 	operator SEXP() const;    // <7>
## };

## enum class lbfgsb_status : unsigned int {
## 	OK = 0L,             // <1>
## 	NOT_CONVERGED = 1L,  // <2>
## 	WARN = 51L,          // <3>
## 	ERROR = 52L,         // <4>
## };

## -----------------------------------------------------------------------------
Rcpp::sourceCpp("examples/lbfgsb.cpp")
out = lbfgsb_ex(x0 = rep(1, 4))
print(out$numerical)
print(out$analytical)


## cg_result bfgs(
## 	const Rcpp::NumericVector& init,  // <1>
## 	const dfv& f,                     // <2>
## 	const vfv& g,                     // <3>
## 	const cg_args& args               // <4>
## )
## 
## cg_result cg(
## 	const Rcpp::NumericVector& init,  // <1>
## 	const dfv& f,                     // <2>
## 	const cg_args& args               // <4>
## )
## 
## cg_result cg(
## 	const Rcpp::NumericVector& init,  // <1>
## 	const dfv& f,                     // <2>
## 	const vfv& g                      // <3>
## )
## 
## cg_result cg(
## 	const Rcpp::NumericVector& init,  // <1>
## 	const dfv& f                      // <2>
## )

## 
## struct cg_args
## {
## 	richardson_args deriv_args;    // <1>
## 	double parscale = 1;           // <2>
## 	double fnscale = 1;            // <3>
## 	double abstol = R_NegInf;      // <4>
## 	double reltol = mach_eps_2r;   // <5>
## 	int type = 1;                  // <6>
## 	int trace = 0;                 // <7>
## 	int maxit = 100;               // <8>
## 
## 	cg_args() { };                 // <9>
## 	cg_args(SEXP obj);             // <10>
## 	operator SEXP() const;         // <11>
## };

## struct cg_result {
## 	std::vector<double> par;  // <1>
## 	double value;             // <2>
## 	cg_status status;         // <3>
## 	int fncount;              // <4>
## 	int grcount;              // <5>
## 
## 	operator SEXP() const;    // <6>
## };

## enum class cg_status : unsigned int {
## 	OK = 0L,             // <1>
## 	NOT_CONVERGED = 1L   // <2>
## };

## -----------------------------------------------------------------------------
Rcpp::sourceCpp("examples/cg.cpp")
out = cg_ex(x0 = rep(1, 4))
print(out$numerical)
print(out$analytical)


## 
## nlm_result nlm(
## 	const Rcpp::NumericVector& init,  // <1>
## 	const dfv& f,                     // <2>
## 	const vfv& g,                     // <3>
## 	const mfv& h,                     // <4>
## 	const nlm_args& args              // <5>
## )
## 
## nlm_result nlm(
## 	const Rcpp::NumericVector& init,  // <1>
## 	const dfv& f,                     // <2>
## 	const vfv& g,                     // <3>
## 	const nlm_args& args              // <5>
## )
## 
## nlm_result nlm(
## 	const Rcpp::NumericVector& init,  // <1>
## 	const dfv& f,                     // <2>
## 	const nlm_args& args              // <5>
## )
## 
## nlm_result nlm(
## 	const Rcpp::NumericVector& init,  // <1>
## 	const dfv& f,                     // <2>
## 	const vfv& g,                     // <3>
## 	const mfv& h                      // <4>
## )
## 
## nlm_result nlm(
## 	const Rcpp::NumericVector& init,  // <1>
## 	const dfv& f,                     // <2>
## 	const vfv& g                      // <3>
## )
## 
## nlm_result nlm(
## 	const Rcpp::NumericVector& init,  // <1>
## 	const dfv& f                      // <2>
## )

## struct nlm_args
## {
## 	std::vector<double> typsize;   // <1>
## 	unsigned int print_level = 0;  // <2>
## 	double fscale = 1;             // <3>
## 	double fnscale = 1;            // <4>
## 	unsigned int ndigit = 12;      // <5>
## 	double gradtol = 1e-6;         // <6>
## 	double stepmax = R_PosInf;     // <7>
## 	double steptol = 1e-6;         // <8>
## 	int iterlim = 100;             // <9>
## 	unsigned int method = 1;       // <10>
## 	double trust_radius = 1.0;     // <11>
## 
## 	nlm_args() { };                // <12>
## 	nlm_args(SEXP obj);            // <13>
## 	operator SEXP() const;         // <14>
## };

## struct nlm_result
## {
## 	std::vector<double> par;      // <1>
## 	std::vector<double> grad;     // <2>
## 	double estimate;              // <3>
## 	int iterations;               // <4>
## 	nlm_status status;            // <5>
## 
## 	operator SEXP() const;        // <6>
## };

## enum class nlm_status : unsigned int {
## 	OK = 0L,                   // <1>
## 	GRADIENT_WITHIN_TOL = 1L,  // <2>
## 	ITERATES_WITH_TOL = 2L,    // <3>
## 	NO_LOWER_STEP = 3L,        // <4>
## 	ITERATION_MAX = 4L,        // <5>
## 	STEP_SIZE_EXCEEDED = 5L    // <6>
## };
## 

## -----------------------------------------------------------------------------
Rcpp::sourceCpp("examples/nlm.cpp")
out = nlm_ex(x0 = rep(1, 4))
nn = c("par", "grad")
print(out$res1[nn])
print(out$res2[nn])
print(out$res3[nn])


## -----------------------------------------------------------------------------
#| eval: false
# apply(X, c(1,2), f)
# apply(X, 1, f)
# apply(X, 2, f)


## template <typename T, int RTYPE>
## Rcpp::Vector<RTYPE> row_apply(
## 	const Rcpp::Matrix<RTYPE>& X,							// <1>
## 	const std::function<T(const Rcpp::Vector<RTYPE>&)>& f   // <2>
## )
## 
## template <typename T, int RTYPE>
## Rcpp::Vector<RTYPE> col_apply(
## 	const Rcpp::Matrix<RTYPE>& X,							// <1>
## 	const std::function<T(const Rcpp::Vector<RTYPE>&)>& f   // <2>
## )
## 
## template <typename T, int RTYPE>
## Rcpp::Matrix<RTYPE> mat_apply(
## 	const Rcpp::Matrix<RTYPE>& X,							// <1>
## 	const std::function<T(T)>& f                            // <2>
## )

## -----------------------------------------------------------------------------
Rcpp::sourceCpp("examples/apply.cpp")
X = matrix(1:12, 4, 3)
out = apply_ex(X)
print(out)


## Rcpp::NumericMatrix outer(
## 	const Rcpp::NumericMatrix& X,  // <1>
## 	const dfvv& f                  // <3>
## )
## 
## Rcpp::NumericMatrix outer(
## 	const Rcpp::NumericMatrix& X,  // <1>
## 	const Rcpp::NumericMatrix& Y,  // <2>
## 	const dfvv& f                  // <3>
## )
## 
## Rcpp::NumericVector outer_matvec(
## 	const Rcpp::NumericMatrix& X,  // <1>
## 	const dfvv& f,                 // <3>
## 	const Rcpp::NumericVector& a   // <4>
## )
## 
## Rcpp::NumericVector outer_matvec(
## 	const Rcpp::NumericMatrix& X,  // <1>
## 	const Rcpp::NumericMatrix& Y,  // <2>
## 	const dfvv& f,                 // <3>
## 	const Rcpp::NumericVector& a   // <4>
## )
## 
## Rcpp::NumericMatrix outer(
## 	const Rcpp::NumericMatrix& X,  // <1>
## 	const Rcpp::NumericMatrix& Y,  // <2>
## 	const dfvv& f                  // <3>
## )
## 
## mat_triplet_result outer_triplet(
## 	const Rcpp::NumericMatrix& X,  // <1>
## 	const dfvv& f,                 // <3>
## 	const bfvv& g                  // <5>
## )
## 
## mat_triplet_result outer_triplet(
## 	const Rcpp::NumericMatrix& X,  // <1>
## 	const Rcpp::NumericMatrix& Y,  // <2>
## 	const dfvv& f,                 // <3>
## 	const bfvv& g                  // <5>
## )

## struct mat_triplet_result
## {
## 	std::vector<unsigned int> i;  // <1>
## 	std::vector<unsigned int> j;  // <2>
## 	std::vector<double> x;        // <3>
## 	unsigned int m;               // <4>
## 	unsigned int n;               // <5>
## 
## 	operator SEXP() const;        // <6>
## };

## -----------------------------------------------------------------------------
Rcpp::sourceCpp("examples/outer.cpp")
m = 5; n = 3; d = 2
X = matrix(rnorm(10), m, d)
Y = matrix(rnorm(6), n, d)
a = rep(1, m)
b = rep(1, n)
out = outer_ex(X, Y, a, b)
print(out[1:4])


## -----------------------------------------------------------------------------
#| message: false
library(Matrix)
sp1 = out[[5]]
sp2 = out[[6]]
sparseMatrix(sp1$i, sp1$j, x = sp1$x, dims = c(sp1$m, sp1$n), symmetric = T)
sparseMatrix(sp2$i, sp2$j, x = sp2$x, dims = c(sp2$m, sp2$n), symmetric = F)


## -----------------------------------------------------------------------------
Rcpp::sourceCpp("examples/outer-sp.cpp")
outer_sp_ex(X)


## cg_result solve_cg(
## 	const vfv& l,                     // <1>
## 	const Rcpp::NumericVector& b,     // <2>
## 	const Rcpp::NumericVector& init,  // <3>
## 	const cg_args& args               // <4>
## )
## 
## cg_result solve_cg(
## 	const vfv& l,                     // <1>
## 	const Rcpp::NumericVector& b,     // <2>
## 	const Rcpp::NumericVector& init   // <3>
## )
## 
## cg_result solve_cg(
## 	const vfv& l,                     // <1>
## 	const Rcpp::NumericVector& b      // <2>
## )

## -----------------------------------------------------------------------------
Rcpp::sourceCpp("examples/solve-cg.cpp")
b = rep(1, 10)
out = solve_cg_ex(b)
print(out)


## -----------------------------------------------------------------------------
A = matrix(0, 10, 10)
diag(A) = 2
A[cbind(1:9, 2:10)] = 1
A[cbind(2:10, 1:9)] = 1
solve(A, b)


## -----------------------------------------------------------------------------
#| eval: false
# which(f(X), arr.ind = TRUE)


## template <typename T, int RTYPE>
## Rcpp::IntegerMatrix which(
## 	const Rcpp::Matrix<RTYPE>& X,      // <1>
## 	const std::function<bool(T)>& f),  // <2>
## 	bool one_based = false             // <3>
## )

## -----------------------------------------------------------------------------
Rcpp::sourceCpp("examples/which.cpp")
x = runif(10, -1, 1)
X = matrix(x, 2, 5)
out = which_ex(X)
print(X)
print(out)


## -----------------------------------------------------------------------------
f = function(x) { x > 0 & x < 0.5 }
which(f(X), arr.ind = TRUE) - 1


## typedef std::function<double(double,bool)> density;       // <1>
## typedef std::function<double(double,bool,bool)> cdf;      // <2>
## typedef std::function<double(double,bool,bool)> quantile; // <3>

## double d_trunc(
## 	double x,                      // <1>
## 	double lo,                     // <2>
## 	double hi,                     // <3>
## 	const density& f,              // <4>
## 	const cdf& F,                  // <5>
## 	bool log = false               // <6>
## )
## 
## Rcpp::NumericVector d_trunc(
## 	const Rcpp::NumericVector& x,  // <1>
## 	const Rcpp::NumericVector& lo, // <2>
## 	const Rcpp::NumericVector& hi, // <3>
## 	const density& f,              // <4>
## 	const cdf& F,                  // <5>
## 	bool log = false               // <6>
## )

## -----------------------------------------------------------------------------
Rcpp::sourceCpp("examples/d_trunc.cpp")
x = seq(0, 1, length.out = 30)
d_trunc_ex(x, shape1 = 2, shape2 = 5, lo = 0.5, hi = 0.95)


## double p_trunc(
## 	double x,              // <1>
## 	double lo,             // <2>
## 	double hi,             // <3>
## 	const cdf& F,          // <4>
## 	bool lower = true,     // <5>
## 	bool log = false       // <6>
## )
## 
## Rcpp::NumericVector p_trunc(
## 	const Rcpp::NumericVector& x,  // <1>
## 	const Rcpp::NumericVector& lo, // <2>
## 	const Rcpp::NumericVector& hi, // <3>
## 	const cdf& F,                  // <4>
## 	bool lower = true,             // <5>
## 	bool log = false               // <6>
## )

## -----------------------------------------------------------------------------
Rcpp::sourceCpp("examples/p_trunc.cpp")
x = seq(0, 1, length.out = 30)
p_trunc_ex(x, shape1 = 2, shape2 = 5, lo = 0.5, hi = 0.95)


## double q_trunc(
## 	double p,              // <1>
## 	double lo,             // <2>
## 	double hi,             // <3>
## 	const cdf& F,          // <4>
## 	const quantile& Finv,  // <5>
## 	bool lower = true,     // <6>
## 	bool log = false       // <7>
## )
## 
## Rcpp::NumericVector q_trunc(
## 	const Rcpp::NumericVector& p,  // <1>
## 	const Rcpp::NumericVector& lo, // <2>
## 	const Rcpp::NumericVector& hi, // <3>
## 	const cdf& F,                  // <4>
## 	const quantile& Finv,          // <5>
## 	bool lower = true,             // <6>
## 	bool log = false               // <7>
## )

## -----------------------------------------------------------------------------
Rcpp::sourceCpp("examples/q_trunc.cpp")
p = seq(0, 1, length.out = 30)
q_trunc_ex(p, shape1 = 2, shape2 = 5, lo = 0.5, hi = 0.95)


## double r_trunc(
## 	double lo,             // <2>
## 	double hi,             // <3>
## 	const cdf& F,          // <4>
## 	const quantile& Finv   // <5>
## )
## 
## Rcpp::NumericVector r_trunc(
## 	unsigned int n,                // <1>
## 	const Rcpp::NumericVector& lo, // <2>
## 	const Rcpp::NumericVector& hi, // <3>
## 	const cdf& F,                  // <4>
## 	const quantile& Finv           // <5>
## )
## 

## -----------------------------------------------------------------------------
Rcpp::sourceCpp("examples/r_trunc.cpp")
r_trunc_ex(n = 20, shape1 = 2, shape2 = 5, lo = 0.5, hi = 0.95)


## void logger(const char* fmt, ...);

## -----------------------------------------------------------------------------
Rcpp::sourceCpp("examples/logger.cpp")
logger_ex()

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

#' Numerical Derivatives via Finite Differences
#'
#' @param f Function to differentiate.
#' @param x Scalar at which to evaluate the derivative.
#' @param i First coordinate to differentiate.
#' @param j Second coordinate to differentiate.
#' @param h Step size in the first coordinate.
#' @param h_i Step size in the first coordinate.
#' @param h_j Step size in the second coordinate.
#' @param fd_type Type of derivative: `0` for symmetric difference, `1` for
#' forward difference, and `2` for backward difference.
#' @param args List of additional arguments from the function \code{richardson_args}.
#'
#' @examples
#' args = richardson_args()
#'
#' f = sin   # Try 2nd derivatives of a univariate function
#' x0 = 0.5
#' print(-sin(x0))  ## Exact answer for f''(x0)
#'
#' fd_deriv2(f, x0, i = 0, j = 0, h_i = 0.001, h_j = 0.001, fd_type = 0)
#' fd_deriv2(f, x0, i = 0, j = 0, h_i = 0.001, h_j = 0.001, fd_type = 1)
#' fd_deriv2(f, x0, i = 0, j = 0, h_i = 0.001, h_j = 0.001, fd_type = 2)
#'
#' deriv2(f, x0, i = 0, j = 0, args, fd_type = 0)
#'
#' # Try 2nd derivatives of a bivariate function
#' f = function(x) { sin(x[1]) + cos(x[2]) }
#' x0 = c(0.5, 0.25)
#'
#' print(-sin(x0[1]))  ## Exact answer for f_xx(x0)
#' print(-cos(x0[2]))  ## Exact answer for f_yy(x0)
#' print(0)         ## Exact answer for f_xy(x0,y0)
#'
#' numDeriv::hessian(f, x0)
#'
#' fd_deriv2(f, x0, i = 0, j = 0, h_i = 0.001, h_j = 0.001, fd_type = 0)
#' fd_deriv2(f, x0, i = 0, j = 0, h_i = 0.001, h_j = 0.001, fd_type = 1)
#' fd_deriv2(f, x0, i = 0, j = 0, h_i = 0.001, h_j = 0.001, fd_type = 2)
#'
#' fd_deriv2(f, x0, i = 0, j = 1, h_i = 0.001, h_j = 0.001, fd_type = 0)
#' fd_deriv2(f, x0, i = 0, j = 1, h_i = 0.001, h_j = 0.001, fd_type = 1)
#' fd_deriv2(f, x0, i = 0, j = 1, h_i = 0.001, h_j = 0.001, fd_type = 2)
#'
#' fd_deriv2(f, x0, i = 1, j = 1, h_i = 0.001, h_j = 0.001, fd_type = 0)
#' fd_deriv2(f, x0, i = 1, j = 1, h_i = 0.001, h_j = 0.001, fd_type = 1)
#' fd_deriv2(f, x0, i = 1, j = 1, h_i = 0.001, h_j = 0.001, fd_type = 2)
#'
#' deriv2(f, x0, i = 1, j = 1, args, fd_type = 0)
#' deriv2(f, x0, i = 1, j = 1, args, fd_type = 1)
#' deriv2(f, x0, i = 1, j = 1, args, fd_type = 2)
#'
#' @return
#' `fd_deriv1` and `fd_deriv2` return a single numeric value corresponding to
#' the first and second derivative via finite differences. `deriv1` and
#' `deriv2` return a list with the form of a `richardson_result` described in
#' section "Richardson Extrapolated Finite Differences" of the package
#' vignette.
#'
#' @name deriv
#'
#' @export
fd_deriv1 <- function(f, x, i, h, fd_type) {
    .Call(`_fntl_fd_deriv_rcpp`, f, x, i, h, fd_type)
}

#' @name deriv
#' @export
fd_deriv2 <- function(f, x, i, j, h_i, h_j, fd_type) {
    .Call(`_fntl_fd_deriv2_rcpp`, f, x, i, j, h_i, h_j, fd_type)
}

#' @name deriv
#' @export
deriv1 <- function(f, x, i, args, fd_type) {
    .Call(`_fntl_deriv_rcpp`, f, x, i, args, fd_type)
}

#' @name deriv
#' @export
deriv2 <- function(f, x, i, j, args, fd_type) {
    .Call(`_fntl_deriv2_rcpp`, f, x, i, j, args, fd_type)
}

#' Numerical Gradient Vector
#'
#' @param f Function to differentiate.
#' @param x Vector at which to evaluate the gradient.
#' @param args List of additional arguments from the function \code{richardson_args}.
#'
#' @examples
#' f = function(x) { sum(sin(x)) }
#' args = richardson_args()
#' x0 = seq(0, 1, length.out = 5)
#' cos(x0)  ## Exact answer
#' gradient0(f, x0, args)
#' numDeriv::grad(f, x0)
#'
#' @return
#' A list with the form of a `gradient_result` described in section "Gradient"
#' of the package vignette.
#'
#' @export
gradient0 <- function(f, x, args) {
    .Call(`_fntl_gradient_rcpp`, f, x, args)
}

#' Numerical Jacobian Matrix
#'
#' @param f Function to differentiate.
#' @param x Vector at which to evaluate the Jacobian.
#' @param args List of additional arguments from the function \code{richardson_args}.
#'
#' @examples
#' f = function(x) { cumsum(sin(x)) }
#' x0 = seq(1, 10, length.out = 5)
#' args = richardson_args()
#' out = jacobian0(f, x0, args)
#' print(out$value)
#' numDeriv::jacobian(f, x0)
#'
#' @return
#' A list with the form of a `jacobian_result` described in section "Jacobian"
#' of the package vignette.
#'
#' @export
jacobian0 <- function(f, x, args) {
    .Call(`_fntl_jacobian_rcpp`, f, x, args)
}

#' Numerical Hessian
#'
#' @param f Function to differentiate.
#' @param x Vector at which to evaluate the Hessian.
#' @param args List of additional arguments from the function \code{richardson_args}.
#'
#' @examples
#' f = function(x) { sum(x^2) }
#' x0 = seq(1, 10, length.out = 5)
#' args = richardson_args()
#' hessian0(f, x0, args)
#' numDeriv::hessian(f, x0)
#'
#' @return
#' A list with the form of a `hessian_result` described in section "Hessian"
#' of the package vignette.
#'
#' @export
hessian0 <- function(f, x, args) {
    .Call(`_fntl_hessian_rcpp`, f, x, args)
}

#' Find Root
#'
#' @param f Function for which a root is desired.
#' @param lower Lower limit of search interval. Must be finite.
#' @param upper Upper limit of search interval. Must be finite.
#' @param args List of additional arguments from the function \code{findroot_args}.
#'
#' @examples
#' f = function(x) { x^2 - 1 }
#' args = findroot_args()
#' findroot_bisect(f, 0, 10, args)
#' findroot_brent(f, 0, 10, args)
#'
#' @return
#' A list with the form of a `findroot_result` described in section
#' "Root-Finding" of the package vignette.
#'
#' @name findroot
#'
#' @export
findroot_bisect <- function(f, lower, upper, args) {
    .Call(`_fntl_findroot_bisect_rcpp`, f, lower, upper, args)
}

#' @name findroot
#' @export
findroot_brent <- function(f, lower, upper, args) {
    .Call(`_fntl_findroot_brent_rcpp`, f, lower, upper, args)
}

#' Univariate Optimization
#'
#' @param f Function to optimize.
#' @param lower Lower limit of search interval. Must be finite.
#' @param upper Upper limit of search interval. Must be finite.
#' @param args List of additional arguments from the function \code{optimize_args}.
#'
#' @examples
#' f = function(x) { x^2 - 1 }
#' args = optimize_args()
#' goldensection(f, 0, 10, args)
#' optimize_brent(f, 0, 10, args)
#'
#' @return
#' A list with the form of a `optimize_result` described in section
#' "Univariate Optimization" of the package vignette.
#'
#' @name univariate-optimization
#'
#' @export
goldensection <- function(f, lower, upper, args) {
    .Call(`_fntl_goldensection_rcpp`, f, lower, upper, args)
}

#' @name univariate-optimization
#' @export
optimize_brent <- function(f, lower, upper, args) {
    .Call(`_fntl_optimize_brent_rcpp`, f, lower, upper, args)
}

#' Integration
#'
#' Compute the integral \eqn{\int_a^b f(x) dx}.
#'
#' @param f Function to integrate.
#' @param lower Lower limit of integral.
#' @param upper Upper limit of integral.
#' @param args List of additional arguments from the function \code{integrate_args}.
#'
#' @examples
#' f = function(x) { exp(-x^2 / 2) }
#' args = integrate_args()
#' integrate0(f, 0, 10, args)
#'
#' @return
#' A list with the form of a `integrate_result` described in section
#' "Integration" of the package vignette.
#'
#' @export
integrate0 <- function(f, lower, upper, args) {
    .Call(`_fntl_integrate_rcpp`, f, lower, upper, args)
}

#'  Multivariate Optimization
#'
#' @param init Initial value
#' @param f Function \eqn{f} to optimize
#' @param g Gradient function of \eqn{f}.
#' @param h Hessian function of \eqn{f}.
#' @param args List of additional arguments for optimization.
#'
#' @details
#' The argument `args` should be a list constructed from one of the following
#' functions:
#'
#' - `bfgs_args` for BFGS;
#' - `lbfgsb_args` for L-BFGS-B;
#' - `cg_args` for CG;
#' - `neldermead_args` for Nelder-Mead;
#' - `nlm_args` for the Newton-type algorithm used in `nlm`.
#'
#' When `g` or `h` are omitted, the gradient or Hessian will be respectively
#' be computed via finite differences.
#'
#' @examples
#' f = function(x) { sum(x^2) }
#' g = function(x) { 2*x }
#' h = function(x) { 2*diag(length(x)) }
#' x0 = c(1,1)
#'
#' args = cg_args()
#' cg1(x0, f, g, args)
#' cg2(x0, f, args)
#'
#' args = bfgs_args()
#' bfgs1(x0, f, g, args)
#' bfgs2(x0, f, args)
#'
#' args = lbfgsb_args()
#' lbfgsb1(x0, f, g, args)
#' lbfgsb2(x0, f, args)
#'
#' args = neldermead_args()
#' neldermead(x0, f, args)
#'
#' args = nlm_args()
#' nlm1(x0, f, g, h, args)
#' nlm2(x0, f, g, args)
#' nlm3(x0, f, args)
#'
#' @return
#' A list with results corresponding to the specified function. See the
#' package vignette for further details.
#'
#' - `cg1` and `cg2` return a `cg_result` which is documented in the section
#'   "Conjugate Gradient".
#' - `bfgs1` and `bfgs2` return a `bfgs_result` which is documented in the
#'   section "BFGS".
#' - `lbfgsb1` and `lbfgsb2` return a `lbfgsb_result` which is documented in
#'   the section "L-BFGS-B".
#' - `neldermead` returns a `neldermead_result` which is documented in
#'   the section "Nelder-Mead".
#' - `nlm1`, `nlm2`, and `nlm3` return a `nlm_result` which is documented in
#'   the section "Newton-Type Algorithm for Nonlinear Optimization".
#'
#' @name multivariate-optimization
#'
#' @export
cg1 <- function(init, f, g, args) {
    .Call(`_fntl_cg1_rcpp`, init, f, g, args)
}

#' @name multivariate-optimization
#' @export
cg2 <- function(init, f, args) {
    .Call(`_fntl_cg2_rcpp`, init, f, args)
}

#' @name multivariate-optimization
#' @export
bfgs1 <- function(init, f, g, args) {
    .Call(`_fntl_bfgs1_rcpp`, init, f, g, args)
}

#' @name multivariate-optimization
#' @export
bfgs2 <- function(init, f, args) {
    .Call(`_fntl_bfgs2_rcpp`, init, f, args)
}

#' @name multivariate-optimization
#' @export
lbfgsb1 <- function(init, f, g, args) {
    .Call(`_fntl_lbfgsb1_rcpp`, init, f, g, args)
}

#' @name multivariate-optimization
#' @export
lbfgsb2 <- function(init, f, args) {
    .Call(`_fntl_lbfgsb2_rcpp`, init, f, args)
}

#' @name multivariate-optimization
#' @export
neldermead <- function(init, f, args) {
    .Call(`_fntl_neldermead_rcpp`, init, f, args)
}

#' @name multivariate-optimization
#' @export
nlm1 <- function(init, f, g, h, args) {
    .Call(`_fntl_nlm1_rcpp`, init, f, g, h, args)
}

#' @name multivariate-optimization
#' @export
nlm2 <- function(init, f, g, args) {
    .Call(`_fntl_nlm2_rcpp`, init, f, g, args)
}

#' @name multivariate-optimization
#' @export
nlm3 <- function(init, f, args) {
    .Call(`_fntl_nlm3_rcpp`, init, f, args)
}

#' Matrix Apply Functions
#'
#' @param X A matrix
#' @param f The function to apply.
#'
#' @details
#' The `mat_apply`, `row_apply`, and `col_apply` C++ functions are intended to
#' operate like the following calls in R, respectively.
#' ```
#' apply(x, c(1,2), f)
#' apply(x, 1, f)
#' apply(x, 2, f)
#' ```
#'
#' The R functions exposed here are specific to numeric-valued matrices, but
#' the underlying C++ functions are intended to work with any type of Rcpp
#' Matrix.
#'
#' @examples
#' X = matrix(1:12, nrow = 4, ncol = 3)
#' mat_apply(X, f = function(x) { x^(1/3) })
#' row_apply(X, f = function(x) { sum(x^2) })
#' col_apply(X, f = function(x) { sum(x^2) })
#'
#' @return
#' `mat_apply` returns a matrix. `row_apply` and `col_apply` return a vector.
#' See section "Apply" of the package vignette for details.
#'
#' @name matrix_apply
#'
#' @export
mat_apply <- function(X, f) {
    .Call(`_fntl_mat_apply_rcpp`, X, f)
}

#' @name matrix_apply
#' @export
row_apply <- function(X, f) {
    .Call(`_fntl_row_apply_rcpp`, X, f)
}

#' @name matrix_apply
#' @export
col_apply <- function(X, f) {
    .Call(`_fntl_col_apply_rcpp`, X, f)
}

#' Matrix Which Function
#'
#' @param X A matrix
#' @param f A predicate to apply to each element of \eqn{X}.
#'
#' @details
#' The `which` C++ functions are intended to operate like the following call
#' in R.
#' ```
#' which(f(X), arr.ind = TRUE) - 1
#' ```
#'
#' The R functions exposed here are specific to numeric-valued matrices, but
#' the underlying C++ functions are intended to work with any type of Rcpp
#' Matrix.
#'
#' @examples
#' X = matrix(1:12 / 6, nrow = 4, ncol = 3)
#' f = function(x) { x < 1 }
#' which0(X, f)
#'
#' @return
#' A matrix with two columns. Each row contains a row and column index
#' corresponding to an element of \eqn{X} that matches the criteria of \eqn{f}.
#' See section "Which" of the package vignette for details.
#'
#' @export
which0 <- function(X, f) {
    .Call(`_fntl_which_rcpp`, X, f)
}

#' Outer Matrix
#'
#' Compute "outer" matrices and matrix-vector products based on a function
#' that operators on pairs of rows. See details.
#'
#' @param X A numerical matrix.
#' @param Y A numerical matrix.
#' @param f Function \eqn{f(x, y)} that operates on a pair of rows and
#' returns an element for the result.
#' @param g Function \eqn{g(x, y)} that operates on a pair of rows and returns
#' a logical value; if `TRUE`, the corresponding output of `f` is placed into
#' the result. Otherwise, the corresponding output of `f` is omitted from the
#' result.
#' @param a A scalar vector.
#'
#' @details
#' Functions \eqn{f(x,y)} and \eqn{g(x,y)} operate on pairs of rows. In the
#' functions with prefix `outer1`, \eqn{x} and \eqn{y} are pairs of rows of
#' \eqn{X}. In the functions with prefix `outer2`, \eqn{x} is a row from
#' \eqn{X} and \eqn{y} is a row from from \eqn{Y}.
#'
#' The `outer1` function computes the \eqn{n \times n} symmetric matrix
#'
#' \deqn{
#' \text{\texttt outer1}(X, f) =
#' \begin{bmatrix}
#' f(x_1, x_1) & \cdots & f(x_1, x_n) \cr
#' \vdots & \ddots & \vdots \cr
#' f(x_n, x_1) & \cdots & f(x_n, x_n) \cr
#' \end{bmatrix}
#' }
#'
#' and the `outer1_matvec` operation computes the \eqn{n}-dimensional vector
#'
#' \deqn{
#' \text{\texttt outer1\_matvec}(X, f, a) =
#' \begin{bmatrix}
#' f(x_1, x_1) & \cdots & f(x_1, x_n) \cr
#' \vdots & \ddots & \vdots \cr
#' f(x_n, x_1) & \cdots & f(x_n, x_n) \cr
#' \end{bmatrix}
#' \begin{bmatrix}
#' a_1 \cr
#' \vdots \cr
#' a_n \cr
#' \end{bmatrix}.
#' }
#'
#' The `outer2` operation computes the \eqn{m \times n} matrix
#'
#' \deqn{
#' \text{\texttt outer2}(X, Y, f) =
#' \begin{bmatrix}
#' f(x_1, y_1) & \cdots & f(x_1, y_n) \cr
#' \vdots & \ddots & \vdots \cr
#' f(x_m, y_1) & \cdots & f(x_m, y_n) \cr
#' \end{bmatrix}
#' }
#'
#' and the `outer2_matvec` operation computes the \eqn{m}-dimensional vector
#'
#' \deqn{
#' \text{\texttt outer2\_matvec}(X, Y, f, a) =
#' \begin{bmatrix}
#' f(x_1, y_1) & \cdots & f(x_1, y_n) \cr
#' \vdots & \ddots & \vdots \cr
#' f(x_m, y_1) & \cdots & f(x_m, y_n) \cr
#' \end{bmatrix}
#' \begin{bmatrix}
#' a_1 \cr
#' \vdots \cr
#' a_n \cr
#' \end{bmatrix}.
#' }
#'
#' The `outer_triplet1` and `outer_triplet2` operations are analogous to
#' `outer1` and `outer2`, respectively, except return sparse matrices in
#' triplet form. The triplet variants take an additional argument `g`, a
#' function $g(x_i, y_j)$ that returns `TRUE` if the result is to be stored
#' in the result and `FALSE` otherwise. The argument `g` can be used for
#' sparseness.
#'
#' @examples
#' set.seed(1234)
#' f = function(x,y) { sum( (x - y)^2 ) }
#' X = matrix(rnorm(12), 6, 2)
#' Y = matrix(rnorm(10), 5, 2)
#' outer1(X, f)
#' outer2(X, Y, f)
#'
#' a = rep(1, 6)
#' b = rep(1, 5)
#' outer1_matvec(X, f, a)
#' outer2_matvec(X, Y, f, b)
#'
#' @return
#' `outer1` and `outer2` return a matrix. `outer1_matvec` and `outer2_matvec`
#' return a vector. `outer1_triplet` and `outer2_triplet` return a list with
#' a sparse matrix (triplet) representation. See section "Outer" of the
#' package vignette for details.
#'
#' @name outer
#'
#' @export
outer1 <- function(X, f) {
    .Call(`_fntl_outer1_rcpp`, X, f)
}

#' @name outer
#' @export
outer2 <- function(X, Y, f) {
    .Call(`_fntl_outer2_rcpp`, X, Y, f)
}

#' @name outer
#' @export
outer1_matvec <- function(X, f, a) {
    .Call(`_fntl_outer1_matvec_rcpp`, X, f, a)
}

#' @name outer
#' @export
outer2_matvec <- function(X, Y, f, a) {
    .Call(`_fntl_outer2_matvec_rcpp`, X, Y, f, a)
}

#' @name outer
#' @export
outer1_triplet <- function(X, f, g) {
    .Call(`_fntl_outer1_triplet_rcpp`, X, f, g)
}

#' @name outer
#' @export
outer2_triplet <- function(X, Y, f, g) {
    .Call(`_fntl_outer2_triplet_rcpp`, X, Y, f, g)
}

#' Iteratively Solve a Linear System with Conjugate Gradient
#'
#' Solve the system \eqn{l(x) = b} where \eqn{l(x)} is a matrix-free
#' representation of the linear operation \eqn{Ax}.
#'
#' @param l A linear transformation of \eqn{x}.
#' @param b A vector.
#' @param init Initial value of solution.
#' @param args List of additional arguments from `cg_args`.
#'
#' @examples
#' set.seed(1234)
#'
#' n = 8
#' idx_diag = cbind(1:n, 1:n)
#' idx_ldiag = cbind(2:n, 1:(n-1))
#' idx_udiag = cbind(1:(n-1), 2:n)
#' b = rep(1, n)
#'
#' ## Solution by explicit computation of solve(A, b)
#' A = matrix(0, n, n)
#' A[idx_diag] = 2
#' A[idx_ldiag] = 1
#' A[idx_udiag] = 1
#' solve(A, b)
#'
#' ## Solve iteratively with solve_cg
#' f = function(x) { A %*% x }
#' args = cg_args()
#' init = rep(0, n)
#' solve_cg(f, b, init, args)
#'
#' @return
#' A list with the form of a `solve_cg_result` described in section "Conjugate
#' Gradient" of the package vignette.
#'
#' @export
solve_cg <- function(l, b, init, args) {
    .Call(`_fntl_solve_cg_rcpp`, l, b, init, args)
}

#' Functions for Truncated Distributions
#'
#' Density, CDF, quantile, and drawing functions for a univariate distribution
#' with density \eqn{f}, cdf, \eqn{F}, and quantile function \eqn{F^{-}}
#' truncated to the interval \eqn{[a ,b]}.
#'
#' @param n Number of draws.
#' @param x Vector of quantiles.
#' @param p Vector of probabilities.
#' @param lo Vector of lower limits.
#' @param hi Vector of upper limits.
#' @param f Density function with form `f(x, log)`.
#' @param F CDF function with signature `F(x, lower, log)`, where `x` is
#' numeric and `lower` and `log` are logical.
#' @param Finv Quantile function with signature `Finv(x, lower, log)`, where
#' `x` is numeric and `lower` and `log` are logical.
#' @param lower logical; if `TRUE`, probabilities are \eqn{P(X \leq x)};
#' otherwise, \eqn{P(X > x)}.
#' @param log logical; if `TRUE`, probabilities are given on log-scale.
#'
#' @return Vector with results.
#' `d_trunc` computes the density, `r_trunc` generates random deviates,
#' `p_trunc` computes the CDF, and `q_trunc` computes quantiles.
#'
#' @examples
#' library(tidyverse)
#'
#' m = 100  ## Length of sequence for density, CDF, etc
#' shape1 = 5
#' shape2 = 2
#' lo = 0.5
#' hi = 0.7
#'
#' # Density, CDF, and quantile function for untruncated distribution
#' ff = function(x, log) { dbeta(x, shape1, shape2, log = log) }
#' FF = function(x, lower, log) { pbeta(x, shape1, shape2, lower.tail = lower, log = log) }
#' FFinv = function(x, lower, log) { qbeta(x, shape1, shape2, lower.tail = lower, log = log) }
#'
#' # Compare truncated and untruncated densities
#' xseq = seq(0, 1, length.out = m)
#' lo_vec = rep(lo, m)
#' hi_vec = rep(hi, m)
#' f0seq = ff(xseq, log = FALSE)
#' fseq = d_trunc(xseq, lo_vec, hi_vec, ff, FF)
#' data.frame(x = xseq, f = fseq, f0 = f0seq) %>%
#'     ggplot() +
#'	    geom_line(aes(xseq, fseq)) +
#'	    geom_line(aes(xseq, f0seq), lty = 2) +
#'	    xlab("x") +
#'	    ylab("Density") +
#'     theme_minimal()
#'
#' # Compare truncated densities and empirical density of generated draws
#' n = 100000
#' lo_vec = rep(lo, n)
#' hi_vec = rep(hi, n)
#' x = r_trunc(n = n, lo_vec, hi_vec, FF, FFinv)
#' hist(x, probability = TRUE, breaks = 15)
#' points(xseq, fseq)
#'
#' # Compare empirical CDF of draws with CDF function
#' Femp = ecdf(x)
#' lo_vec = rep(lo, m)
#' hi_vec = rep(hi, m)
#' Fseq = p_trunc(xseq, lo_vec, hi_vec, FF)
#' data.frame(x = xseq, FF = Fseq) %>%
#'     mutate(F0 = Femp(x)) %>%
#'     ggplot() +
#'     geom_line(aes(xseq, FF), lwd = 1.2) +
#'     geom_line(aes(xseq, F0), col = "orange") +
#'     xlab("x") +
#'     ylab("Probability") +
#'     theme_minimal()
#'
#' # Compare empirical quantiles of draws with quantile function
#' pseq = seq(0, 1, length.out = m)
#' lo_vec = rep(lo, m)
#' hi_vec = rep(hi, m)
#' Finvseq = q_trunc(pseq, lo_vec, hi_vec, FF, FFinv)
#' Finvemp = quantile(x, prob = pseq)
#' data.frame(p = pseq, Finv = Finvseq, Finvemp = Finvemp) %>%
#'	    ggplot() +
#'	    geom_line(aes(pseq, Finv), lwd = 1.2) +
#'	    geom_line(aes(pseq, Finvemp), col = "orange") +
#'	    xlab("p") +
#'	    ylab("Quantile") +
#'	    theme_minimal()
#'
#' @name trunc
#' @export
d_trunc <- function(x, lo, hi, f, F, log = FALSE) {
    .Call(`_fntl_d_trunc_rcpp`, x, lo, hi, f, F, log)
}

#' @name trunc
#' @export
p_trunc <- function(x, lo, hi, F, lower = TRUE, log = FALSE) {
    .Call(`_fntl_p_trunc_rcpp`, x, lo, hi, F, lower, log)
}

#' @name trunc
#' @export
q_trunc <- function(p, lo, hi, F, Finv, lower = TRUE, log = FALSE) {
    .Call(`_fntl_q_trunc_rcpp`, p, lo, hi, F, Finv, lower, log)
}

#' @name trunc
#' @export
r_trunc <- function(n, lo, hi, F, Finv) {
    .Call(`_fntl_r_trunc_rcpp`, n, lo, hi, F, Finv)
}

#' Arguments
#'
#' Get an arguments list for internal methods with the default settings. This
#' object can be adjusted and passed to the respective function.
#'
#' @return
#' An argument list corresponding to the specified function. The elements of
#' the list are named and supplied with default values. See the package
#' vignette for further details.
#'
#' - `findroot_args` is documented in the section "Root-Finding".
#' - `optimize_args` is documented in the section "Univariate Optimization".
#' - `integrate_args` is documented in the section "Integration".
#' - `cg_args` is documented in the section "Conjugate Gradient".
#' - `bfgs_args` is documented in the section "BFGS".
#' - `lbfgsb_args` is documented in the section "L-BFGS-B".
#' - `neldermead_args`is documented in the section "Nelder-Mead".
#' - `nlm_args` is documented in the section "Newton-Type Algorithm for
#'   Nonlinear Optimization".
#' - `richardson_args` is documented in the section "Richardson
#'   Extrapolated Finite Differences".
#'
#' @name args
#' @export
findroot_args <- function() {
    .Call(`_fntl_findroot_args_rcpp`)
}

#' @name args
#' @export
optimize_args <- function() {
    .Call(`_fntl_optimize_args_rcpp`)
}

#' @name args
#' @export
integrate_args <- function() {
    .Call(`_fntl_integrate_args_rcpp`)
}

#' @name args
#' @export
cg_args <- function() {
    .Call(`_fntl_cg_args_rcpp`)
}

#' @name args
#' @export
bfgs_args <- function() {
    .Call(`_fntl_bfgs_args_rcpp`)
}

#' @name args
#' @export
lbfgsb_args <- function() {
    .Call(`_fntl_lbfgsb_args_rcpp`)
}

#' @name args
#' @export
neldermead_args <- function() {
    .Call(`_fntl_neldermead_args_rcpp`)
}

#' @name args
#' @export
nlm_args <- function() {
    .Call(`_fntl_nlm_args_rcpp`)
}

#' @name args
#' @export
richardson_args <- function() {
    .Call(`_fntl_richardson_args_rcpp`)
}

library(tidyverse)
library(Matrix)

set.seed(1234)

m = 10
n = 8
d = 5

f = \(x,y) { sqrt(sum((x - y)^2)) }
g = \(x,y) { f(x, y) <= 3 }

# ----- Test outer1, outer1_triplet, and outer1_matvec -----
x = rnorm(n*d)
X = matrix(x, n, d)

# Expected result
E = matrix(0, n, n)
for (i in 1:n) {
	for (j in 1:n) {
		E[i,j] = f(X[i,], X[j,])
	}
}
a = rnorm(n)

expect_error( outer1(x, f) )

A = outer1(X, f)
expect_equal(dim(A), c(n,n))
expect_equal(A, E)

sp = outer1_triplet(X, f, g = \(x,y) { TRUE })
B = sparseMatrix(sp$i, sp$j, x = sp$x, dims = c(sp$m, sp$n), symmetric = T)
expect_true(all(sp$i >= 1))
expect_true(all(sp$i <= m))
expect_true(all(sp$j >= 1))
expect_true(all(sp$j <= n))
expect_equal(as.matrix(B), E)

sp = outer1_triplet(X, f, g)
B = sparseMatrix(sp$i, sp$j, x = sp$x, dims = c(sp$m, sp$n), symmetric = T)
idx0 = which(E > 3, arr.ind = T)
idx1 = which(E <= 3, arr.ind = T)
expect_equal(B[idx1], E[idx1])
expect_true(all(B[idx0] == 0))

expect_true(all.equal(outer1_matvec(X, f, a), E %*% a,
	check.attributes = FALSE, check.class = FALSE))

# ----- Test outer2, outer2_triplet, and outer2_matvec -----
x = rnorm(m*d)
y = rnorm(n*d)
X = matrix(x, m, d)
Y = matrix(y, n, d)

# Expected result
E = matrix(0, m, n)
for (i in 1:m) {
	for (j in 1:n) {
		E[i,j] = f(X[i,], Y[j,])
	}
}
a = rnorm(n)

expect_error( outer2(x, y, f) )
expect_error( outer2(X, y, f) )
expect_error( outer2(x, Y, f) )

A = outer2(X, Y, f)
expect_equal(dim(A), c(m,n))
expect_equal(A, E)

sp = outer2_triplet(X, Y, f, g = \(x,y) { TRUE })
B = sparseMatrix(sp$i, sp$j, x = sp$x, dims = c(sp$m, sp$n), symmetric = F)
expect_true(all(sp$i >= 1))
expect_true(all(sp$i <= m))
expect_true(all(sp$j >= 1))
expect_true(all(sp$j <= n))
expect_equal(as.matrix(B), E)

sp = outer2_triplet(X, Y, f, g)
B = sparseMatrix(sp$i, sp$j, x = sp$x, dims = c(sp$m, sp$n), symmetric = F)
idx0 = which(E > 3, arr.ind = T)
idx1 = which(E <= 3, arr.ind = T)
expect_equal(B[idx1], E[idx1])
expect_true(all(B[idx0] == 0))

expect_true(all.equal(outer2_matvec(X, Y, f, a), E %*% a,
	check.attributes = FALSE, check.class = FALSE))

library(fntl)
library(numDeriv)

Rcpp::sourceCpp("cpp/test-deriv.cpp")

x0 = 0.5
tol = 0.001

f = function(x) { sin(x) }
out0 = grad(f, x0)

out1 = sine_deriv(x0, 0L)
expect_equal(out1$value, out0, tol)
expect_equal(out1$status, 0)
expect_equal(out1$err, 0, tol)

Rcpp::sourceCpp("cpp/test-univariate-optimization.cpp")

x_min = -base::pi / 2
x_max = +base::pi / 2

out = sine_goldensection(-pi, pi, maximize = FALSE)
expect_equal(out$par, x_min)
expect_equal(out$value, sin(x_min))
expect_equal(out$tol, 0, 1e-5)

out = sine_brent(-pi, pi, maximize = FALSE)
expect_equal(out$par, x_min)
expect_equal(out$value, sin(x_min))
expect_equal(out$tol, 0, 1e-5)

out = sine_goldensection(-pi, pi, maximize = TRUE)
expect_equal(out$par, x_max)
expect_equal(out$value, sin(x_max))
expect_equal(out$tol, 0, 1e-5)

out = sine_brent(-pi, pi, maximize = TRUE)
expect_equal(out$par, x_max)
expect_equal(out$value, sin(x_max))
expect_equal(out$tol, 0, 1e-5)

library(fntl)
library(numDeriv)

Rcpp::sourceCpp("cpp/test-gradient.cpp")

x0 = rep(0.25, 10)

f = function(x) { sum(x^2) }
out0 = grad(f, x0)
expect_equal(out0, 2*x0)

out1 = quadratic_gradient(x0)
expect_equal(out1$value, out0)

library(fntl)

Rcpp::sourceCpp("cpp/test-findroot.cpp")

a = 1
b = 0
c = -1
tol = 0.001

f = function(x) { a*x^2 + b*x + c }
curve(f, xlim = c(-2, 2))

out0 = (-b + c(-1,1) * sqrt(b - 4*a*c)) / (2*a)

out1 = uniroot(f, lower = 0, upper = 10)
expect_equal(out1$root, out0[2], tol)

out2 = poly_root_bisect(a, b, c, lower = 0, upper = 10)
expect_equal(out2$root, out0[2], tol)
expect_equal(out2$f_root, out1$f.root, tol)

out3 = poly_root_brent(a, b, c, lower = 0, upper = 10)
expect_equal(out3$root, out0[2], tol)
expect_equal(out3$f_root, out1$f.root, tol)

library(tidyverse)

set.seed(1234)
Rcpp::sourceCpp("cpp/test-trunc.cpp")

shape1 = 5
shape2 = 2
lo = 0.5
hi = 0.6

# Check density
expect_equal(d_beta_trunc(0, shape1, shape2, lo, hi), 0)
expect_equal(d_beta_trunc(1, shape1, shape2, lo, hi), 0)
expect_true(d_beta_trunc(0.51, shape1, shape2, lo, hi) > 0)
expect_true(d_beta_trunc(0.59, shape1, shape2, lo, hi) > 0)
expect_equal(d_beta_trunc(0, shape1, shape2, lo, hi, log = T), -Inf)
expect_equal(d_beta_trunc(1, shape1, shape2, lo, hi, log = T), -Inf)
lp = d_beta_trunc(0.59, shape1, shape2, lo, hi, log = T)
expect_true(lp > -Inf)

# Check CDF
expect_equal(p_beta_trunc(0, shape1, shape2, lo, hi), 0)
expect_equal(p_beta_trunc(1, shape1, shape2, lo, hi), 1)
pr = p_beta_trunc(0.51, shape1, shape2, lo, hi)
cpr = p_beta_trunc(0.51, shape1, shape2, lo, hi, lower = F)
expect_true(0 < pr && pr < 1)
expect_equal(cpr, 1 - pr)
lpr = p_beta_trunc(0.51, shape1, shape2, lo, hi, log = T)
clpr = p_beta_trunc(0.51, shape1, shape2, lo, hi, lower = F, log = T)
expect_equal(lpr, log(pr))
expect_equal(clpr, log(cpr))

# Check quantile
expect_equal(q_beta_trunc(0, shape1, shape2, lo, hi), lo)
expect_equal(q_beta_trunc(1, shape1, shape2, lo, hi), hi)
qq = q_beta_trunc(0.5, shape1, shape2, lo, hi)
expect_true(lo < qq && qq < hi)

qq = q_beta_trunc(0.75, shape1, shape2, lo, hi)
cqq = q_beta_trunc(0.75, shape1, shape2, lo, hi, lower = F)
expect_equal(p_beta_trunc(qq, shape1, shape2, lo, hi), 0.75)
expect_equal(p_beta_trunc(cqq, shape1, shape2, lo, hi), 0.25)

# Try some cases with distribution truncated to the lower part of the support
lo = 0
hi = 0.001

expect_equal(d_beta_trunc(0, shape1, shape2, lo, hi), 0)
expect_equal(d_beta_trunc(1, shape1, shape2, lo, hi), 0)
expect_true(d_beta_trunc(0.0005, shape1, shape2, lo, hi) > 0)

expect_equal(p_beta_trunc(0, shape1, shape2, lo, hi), 0)
expect_equal(p_beta_trunc(hi, shape1, shape2, lo, hi), 1)
expect_equal(p_beta_trunc(1, shape1, shape2, lo, hi), 1)
pr1 = p_beta_trunc(hi - hi*1e-2, shape1, shape2, lo, hi)
pr2 = p_beta_trunc(hi - hi*1e-6, shape1, shape2, lo, hi)
pr3 = p_beta_trunc(hi, shape1, shape2, lo, hi)
cpr1 = p_beta_trunc(hi - hi*1e-2, shape1, shape2, lo, hi, lower = F)
cpr2 = p_beta_trunc(hi - hi*1e-6, shape1, shape2, lo, hi, lower = F)
cpr3 = p_beta_trunc(hi, shape1, shape2, lo, hi, lower = F)
expect_true(pr1 > 0)
expect_true(pr1 < pr2)
expect_true(pr2 <= pr3)
# expect_equal(cpr1, 1 - pr1, tolerance = 0.02) ## Precision for this one is off...
expect_equal(cpr2, 1 - pr2, tolerance = 0.02)
expect_equal(cpr3, 1 - pr3)

expect_equal(q_beta_trunc(0, shape1, shape2, lo, hi), lo)
expect_equal(q_beta_trunc(1, shape1, shape2, lo, hi), hi)
qq = q_beta_trunc(0.5, shape1, shape2, lo, hi)
expect_true(lo < qq &&  qq < hi)

# Try some cases with distribution truncated to the upper part of the support
lo = 0.999
hi = 1

expect_equal(d_beta_trunc(0, shape1, shape2, lo, hi), 0)
expect_equal(d_beta_trunc(1, shape1, shape2, lo, hi), 0)
expect_true(d_beta_trunc(0.9995, shape1, shape2, lo, hi) > 0)

expect_equal(p_beta_trunc(0, shape1, shape2, lo, hi), 0)
expect_equal(p_beta_trunc(hi, shape1, shape2, lo, hi), 1)
expect_equal(p_beta_trunc(1, shape1, shape2, lo, hi), 1)
pr1 = p_beta_trunc(lo + (hi-lo)*1e-6, shape1, shape2, lo, hi)
pr2 = p_beta_trunc(lo + (hi-lo)*1e-2, shape1, shape2, lo, hi)
pr3 = p_beta_trunc(hi, shape1, shape2, lo, hi)
cpr1 = p_beta_trunc(lo + (hi-lo)*1e-6, shape1, shape2, lo, hi, lower = F)
cpr2 = p_beta_trunc(lo + (hi-lo)*1e-2, shape1, shape2, lo, hi, lower = F)
cpr3 = p_beta_trunc(hi, shape1, shape2, lo, hi, lower = F)
expect_true(pr1 > 0)
expect_true(pr1 < pr2)
expect_true(pr2 <= pr3)
# expect_equal(cpr1, 1 - pr1, tolerance = 0.02) ## Precision for this one is off...
expect_equal(cpr2, 1 - pr2, tolerance = 0.02)
expect_equal(cpr3, 1 - pr3)

expect_equal(q_beta_trunc(0, shape1, shape2, lo, hi), lo)
expect_equal(q_beta_trunc(1, shape1, shape2, lo, hi), hi)
qq = q_beta_trunc(0.5, shape1, shape2, lo, hi)
expect_true(lo < qq &&  qq < hi)

run_beta_checks = function(m, shape1, shape2, lo, hi) {
	# Compare truncated density and CDF with empirical density of generated draws
	x = r_beta_trunc(n = 500000, shape1, shape2, lo, hi)

	prob_emp = numeric(m)
	prob_cdf = numeric(m)
	prob_pdf = numeric(m)
	for (i in 1:m) {
		a = (i - 1) / m
		b = i / m
		prob_emp[i] = mean(a < x & x <= b)
		prob_cdf[i] = p_beta_trunc(b, shape1, shape2, lo, hi) -
			p_beta_trunc(a, shape1, shape2, lo, hi)
		out = integrate(lower = a, upper = b, f = function(x) {
			d_beta_trunc(x, shape1, shape2, lo, hi)
		})
		prob_pdf[i] = out$value
	}
	prob_pdf = as.numeric(prob_pdf)
	prob_cdf = as.numeric(prob_cdf)
	prob_emp = as.numeric(prob_emp)
	expect_equal(prob_cdf, prob_pdf)
	expect_equal(prob_emp, prob_cdf, tolerance = 1e-2)

	# Compare empirical quantiles of draws with quantile function
	pseq = seq(0, 1, length.out = m)
	qq_emp = quantile(x, prob = pseq)
	qq_fun = q_beta_trunc(pseq, shape1, shape2, lo, hi)
	qq_fun = as.numeric(qq_fun)
	qq_emp = as.numeric(qq_emp)
	expect_equal(qq_fun, qq_emp, tolerance = 1e-2)
}

run_beta_checks(m = 20, shape1 = 5, shape2 = 2, lo = 0, hi = 1)
run_beta_checks(m = 20, shape1 = 5, shape2 = 2, lo = 0, hi = 0.5)
run_beta_checks(m = 20, shape1 = 5, shape2 = 2, lo = 0.5, hi = 1)
run_beta_checks(m = 20, shape1 = 5, shape2 = 2, lo = 0, hi = 0.05)
run_beta_checks(m = 20, shape1 = 5, shape2 = 2, lo = 0.95, hi = 1)

# Run some test with Poisson to make sure a discrete distribution also works


run_pois_checks = function(m, lambda, lo, hi) {
	# Compare truncated density and CDF with empirical density of generated draws
	x = r_pois_trunc(n = 500000, lambda, lo, hi)

	xseq = seq(ceiling(lo), floor(hi))
	prob_emp = numeric(length(xseq))
	prob_cdf = numeric(length(xseq))
	prob_pdf = numeric(length(xseq))
	for (i in seq_along(xseq)) {
		v = xseq[i]
		prob_emp[i] = mean(x == v)
		prob_cdf[i] = p_pois_trunc(v, lambda, lo, hi) -
			p_pois_trunc(v-1, lambda, lo, hi)
		prob_pdf[i] = d_pois_trunc(v, lambda, lo, hi)
	}
	expect_equal(prob_cdf, prob_pdf)
	expect_equal(prob_emp, prob_cdf, tolerance = 1e-2)

	# Compare empirical quantiles of draws with quantile function
	m = floor(hi) - ceiling(lo) + 1
	pseq = seq(0, 1, length.out = m)
	qq_emp = quantile(x, prob = pseq)
	qq_fun = q_pois_trunc(pseq, lambda, lo, hi)
	qq_fun = as.numeric(qq_fun)
	qq_emp = as.numeric(qq_emp)
	expect_equal(qq_fun, qq_emp, tolerance = 1e-2)
}

# Note that if we want to include an integer `a` at the lower bound, we need to
# truncate at a value smaller than `a`. Therefore we subtract a small number
# `eps`.
eps = 1e-6
run_pois_checks(lambda = 5, lo = 0 - eps, hi = 10)
run_pois_checks(lambda = 5, lo = 0 - eps, hi = 5)
run_pois_checks(lambda = 5, lo = 5 - eps, hi = 10)
run_pois_checks(lambda = 5, lo = 0 - eps, hi = 0)
run_pois_checks(lambda = 5, lo = 14 - eps, hi = 14)

library(fntl)
library(numDeriv)

Rcpp::sourceCpp("cpp/test-jacobian.cpp")

x0 = c(0.5, 0.25)
f = function(x) { c(sum(2 * x^2), sum(4 * x^2)) }

out0 = jacobian(f, x0)
out1 = quadratic_jacobian(x0)
expect_equal(out1$value, out0)

library(fntl)
library(numDeriv)

Rcpp::sourceCpp("cpp/test-hessian.cpp")

x0 = c(0.5, 0.5)

out0 = diag(x = 2, nrow = 2, ncol = 2)

f = function(x) { sum(x^2) }
out1 = hessian(f, x0)
expect_equal(out1, out0)

out2 = quadratic_hessian(x0)
expect_equal(out2$value, out0)

Rcpp::sourceCpp("cpp/test-integral.cpp")

mu = 0
sigma2 = 1

a = 0.5
b = 1
expected = integrate(f = function(x) { exp(-(x - mu)^2 / (2 * sigma2)) }, lower = a, upper = b)
computed = normal_integral(mu, sigma2, a, b)
expect_equal(computed$value, expected$value)
expect_equal(computed$subdivisions, expected$subdivisions)
expect_equal(computed$abs_error, expected$abs.error)
expect_equal(computed$message, expected$message)

a = 0
b = Inf
expected = 1/2 * sqrt(sigma2 * 2 * base::pi)
computed = normal_integral(mu, sigma2, a, b)
expect_equal(computed$value, expected)

a = -Inf
b = 0
expected = 1/2 * sqrt(sigma2 * 2 * base::pi)
computed = normal_integral(mu, sigma2, a, b)
expect_equal(computed$value, expected)

a = -Inf
b = Inf
expected = sqrt(sigma2 * 2 * base::pi)
computed = normal_integral(mu, sigma2, a, b)
expect_equal(computed$value, expected)

library(fntl)

Rcpp::sourceCpp("cpp/test-apply.cpp")

X = matrix(1:12 + 0.5, 4, 3)

out0 = sapply(X, function(x) { x^(1/3) })
out1 = vec_pow(X, 1/3)
expect_equal(out0, out1)

out0 = apply(X, c(1,2), function(x) { x^(1/3) })
out1 = mat_pow(X, 1/3)
expect_equal(out0, out1)

out0 = apply(X, 1, function(x) { sum(x) })
out1 = row_sum(X)
expect_equal(out0, out1)

out0 = apply(X, 2, function(x) { sum(x) })
out1 = col_sum(X)
expect_equal(out0, out1)

out0 = apply(X, c(1,2), function(x) { floor(x)^2 })
out1 = imat_square(X)
expect_equal(out0, out1)

---
title: "`fntl`: Numerical Tools for Rcpp and Lambda Functions"
author:
  - name: Andrew M. Raim
    email: andrew.raim@gmail.com
affiliations:
  - name: U.S. Census Bureau
    department: Center for Statistical Research and Methodology
    address: 4600 Silver Hill Road
    city: Washington DC
    country: U.S.A.
format:
  pdf:
    fontsize: 12pt
    indent: false
    toc: true
    number-sections: true
    colorlinks: true
    link-citations: true
    prompt: false
    include-in-header:
      text: |
        \usepackage{common}
vignette: >
  %\VignetteIndexEntry{fntl}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{quarto::pdf}
bibliography: references.bib
editor_options: 
  chunk_output_type: console
abstract: |
  R programmers can combine R and C++ to effectively navigate
  a variety of computing tasks: R excels as a language for interactive tasks
  such as data wrangling, analysis, and plotting; on the other hand, C++ can be
  used to efficiently carry out intensive computations. Rcpp and related
  tools have greatly simplified interoperability between the two languages.
  However, numerical computing tasks that involve functions as arguments, such
  as integration, root-finding, and optimization, which are routinely carried
  out in R, are not as straightforward in C++ within the Rcpp framework.
  The `fntl` package seeks to improve this by providing a straightforward
  API for numerical tools where functional arguments are specified as C++
  lambda functions. Like functions in R, lambda functions can be defined in
  the course of a C++ program, "capturing" variables in the surrounding
  environment if desired, and be passed as arguments to other functions. This
  enables the development of R-like programs in C++, which may be appealing to
  Rcpp users compared to existing alternatives in the extended Rcpp family of
  packages. Because the overhead to evaluate a lambda function is low compared
  to that of evaluating an R function from C++, good performance is also
  possible in this paradigm.
thanks: |
  Document was compiled `{r} format(Sys.time(), "%Y-%m-%d %H:%M:%S %Z")` and
  corresponds to `fntl` version `{r} packageVersion("fntl")`.
  **Contact**`:` <andrew.raim@gmail.com>, Center for Statistical Research &
  Methodology, U.S. Census Bureau, Washington, DC, 20233, U.S.A.
geometry:
  - left=0.75in
  - right=0.75in
  - top=0.75in
  - bottom=1.00in
code-block-bg: "#FAFAFA"
code-block-border-left: "#008CFF"
callout-icon: false
filters:
  - include-code-files
execute:
  eval: true
---

```{r}
#| include: false
library(fntl)
library(tidyverse)

set.seed(1234)
```

::: {.callout-important title="$\dagger$Important"}
Users should read @sec-inferring-return carefully. It describes a
seemingly innocuous way of coding lambdas that can lead to crashing R. This can
be difficult to troubleshoot but is easily avoided once we are aware.
:::

# Disclaimer and Acknowledgments {-}

This document is released to inform interested parties of ongoing research and
to encourage discussion of work in progress. Any views expressed are those of
the author and not those of the U.S. Census Bureau.

Thanks to Drs. Joseph Kang and Tommy Wright (U.S. Census Bureau) for reviewing
a draft of this document.

Although there are no guarantees of correctness of the `fntl` package,
reasonable efforts will be made to address shortcomings. Comments, questions,
corrections, and possible improvements can be communicated through the
project's Github repository (<https://github.com/andrewraim/fntl>).

# Introduction {#sec-intro}

Users of R [@Rcore2024] can implement intensive computations in compiled C++
and engage in interactive computation through the interpreted R language. The
Rcpp package [@Rcpp2024] simplifies the process by automating interoperation
between the two languages and providing an intuitive API in C++. However,
numerical tools such as integration, root-finding, and optimization, which are
routinely used in R, are not as readily available or easy to use when carrying
out lower-level programming in C++. Numerical tools in R may be invoked
from C++, but this incurs an overhead which can offset gains in performance
which motivate the use of C++. The purpose of the `fntl` package is to
facilitate access to such numerical tools in C++ using lambda functions. Here,
R-like code can be achieved in C++ where functions are defined on the fly and
passed as arguments to other routines. The name "fntl" is a portmanteau of
"**f**unctions", "**n**umerical **t**ools", and "**l**ambdas".

@IhakaGentleman1996 describe programming with functions as one of the main
motivations in developing R. Namely, functions may be defined dynamically in
the course of a program and can make use of variables from the surrounding
environment. As an example, consider the following R snippet to compute the
maximum likelihood estimate (MLE) of $X_1, \ldots, X_n \sim \text{N}(\mu, 1)$.

```{r}
mu_true = 0
x = rnorm(n = 200, mean = mu_true, sd = 1)
loglik = function(mu) { sum(dnorm(x, mu, 1, log = TRUE)) }
optimize(loglik, lower = -100, upper = 100, maximum = TRUE)
```

Here the generated sample `x` is "baked in" to the definition of `loglik` so
that `loglik` can be regarded as a univariate function of `mu` to be used with
`optimize`. The ability to construct functions such as `loglik` in the course
of a program and pass them to other functions, as any other variable, can be
tremendously convenient. @Wickham2019 discusses some design patterns which are
possible using functions in R. However, such conveniences come at a cost, as
performance can be inefficient in both speed and memory management.

Performance limitations in R can sometimes be worked around; for example, loops
and apply statements are typically slow because they are executed by an interpreter,
but matrix operations are typically fast because they make use of calls
to BLAS, LAPACK, and other compiled matrix libraries. Refactoring loops into
matrix operations can yield a significant performance improvement when it is
feasible. Compiled languages such as FORTRAN and C/C++ are often used for high
performance implementation of numerical methods; however, they are not
well-suited for interactive usages such as data wrangling, data analysis,
and graphics like R, MATLAB, or Python. Julia [@BezansonEtAl2017] has emerged
over the past decade as a language for scientific computing which is both
suitable to be used interactively and compiles into efficient executable code.

R supports integration with FORTRAN and C/C++ so that high performance code
can be used interactively from the R interpreter; therefore, it remains a
viable alternative to languages like Julia. The `Rcpp` package largely
automates integration with C/C++ code so that users can focus their efforts on
solving the problem at hand. For a function call from R to C++, this automation
includes unpacking the arguments from `SEXP` objects into C++ objects and later
packing the result into an `SEXP` object to be returned. Overhead from repeated
packing and unpacking can hinder performance; on the other hand,
performance while submerged in C++ is free of this overhead. The Rcpp API
provides general C++ classes for vectors, matrices, lists, and other routinely
used objects from R. Additional Rcpp extension packages have been developed to
provide APIs with more application-specific classes; for example, RcppArmadillo
[@EddelbuettelSanderson2014] exposes the API from Armadillo for numerical
matrix operations.

A performance penalty is also suffered when defining a function in R and using
it from C++ with Rcpp. Calling the function from C++ requires going
back through the R runtime environment in addition to packing arguments into
`SEXP` objects and unpacking return values from `SEXP` objects. Doing so
repeatedly accumulates the penalty, and can negate performance benefits of
using C++. Instead, we will consider utilizing lambda functions in C++, which
were introduced in the C++11 specification. There are alternative methods of
defining and passing functions as objects - such as through function pointers
and functor classes - but lambda functions seem best aligned with the R style
discussed by @IhakaGentleman1996. Traditional functions and classes used in the
function pointer and functor approaches, respectively, are declared in their
own blocks prior to use. Another major difference lies in auxiliary data - such
as sample data in a loglikelihood function - which are not considered to be
primary arguments. With function pointers, auxiliary data are passed as
additional arguments that each caller must furnish. Functors are classes which
expose the function of interest and encapsulate auxiliary data using member
variables.

To demonstrate lambdas, consider the following C++ code snippet.

```{cpp}
auto f = [](double x, double y) -> double { return x*y; };
```

The function `f` may be invoked as usual with an expression such as `f(x, y)`.
Here, `f` is a lambda with `double` arguments `x` and `y` which returns the
product `x*y` as another `double`. The `auto` keyword instructs the compiler to
infer the type of `f` from the  return type of the right-hand side of the
equality operator. In the previous example, the return type of the lambda can
also be inferred, and we may rewrite it as follows.

```{cpp}
auto f = [](double x, double y) { return x*y; };
```

Like an R function, variables in scope of the lambda may be "baked in" to it.
Here is an example from our MLE setting.

```{cpp}
Rcpp::NumericVector x = Rcpp::rnorm(200);
auto loglik = [&](double mu) {
    double out = Rcpp::sum(Rcpp::dnorm(x, mu, 1, true));
	return out;
};
```

The function `loglik` takes a `double` argument `mu` and returns a `double`.
The randomly generated data `x` is included in the function definition; C++
documentation refers to `x` as a *capture* of the lambda `loglik`. The
bracketed expression `[&]` specifies that captures of `loglik` should be by
reference; alternatively, `[=]` would specify that captures should be done by
value so that copies of the original variable are used. It is also possible to
specify by-reference or by-value for each captured variable.

To be able to pass a lambda with captures as an argument to other
functions, we wrap it in a [Standard Template Library][stl-url] (STL) function
as follows.

```{cpp}
std::function<double(double)> loglik = [&](double mu) {
    double out = Rcpp::sum(Rcpp::dnorm(x, mu, 1, true));
	return out;
};
```

Note that the template argument of `std::function<double(double)>` describes
the domain and range of the function: the `double` in parentheses is the
argument and the `double` outside indicates the return type.

Lambda functions can be used directly with the Rcpp API in some cases. For
example, an Rcpp Gallery [post][RcppLambdaPost] demonstrates the use
of the STL `transform` function to apply a lambda to each element of a vector.
The `fntl` package avoids duplicating this existing functionality.

Numerical tools implemented in `fntl` are covered in other Rcpp-related
packages to some extent. The `RcppNumerical` package [@RcppNumerical2024]
implements numerical integration - both univariate and multivariate - and
optimization using a limited memory Broyden, Fletcher, Goldfarb and Shanno
method. This is accomplished by providing an Rcpp interface to  several open
source libraries. The `roptim` package [@Roptim2022] provides an Rcpp interface
to the R API to call individual optimization methods underlying the `optim`
function. Function arguments in both `RcppNumerical` and `roptim` are specified
via C++ functors. This vignette of `roptim` provides a discussion on calling
the R API which was helpful in the development of `fntl`. A number of numerical
utilities in the GSL [@GalassiEtAl2009] and [Boost](https://www.boost.org)
libraries are provided by the `RcppGSL` [@RcppGSL2024] and `BH` [@BH2024]
packages, respectively. The `RcppGSL` package requires installation of the
underlying GSL library to build the source package. The author of the present
document finds the interfaces of both GSL and Boost to be somewhat daunting,
which has motivated a search for alternatives.

The `fntl` package is guided by several design principles. The interface is
intended to be simple and familiar to R users. External dependencies beyond
the R platform itself and the Rcpp package are avoided; this is to support use
in locked-down computing environments where adding or upgrading system libraries
may be nontrivial. For numerical methods, we first prefer to make use of
functions exposed as entry points in the R API [@WritingRExtensions, Section 6].
In cases where a desired R method is not exposed in the API, we roll our own
implementation. Such methods in `fntl` may not be exactly the same as those in
R, but are intended to be comparable. 

@sec-overview presents an overview of the `fntl` API. Subsequent sections
present the API in detail by topic: numerical integration in @sec-integrate,
numerical differentiation in @sec-diff, root-finding in @sec-findroot,
univariate optimization in @sec-optimize, multivariate optimization in
@sec-optim, and matrix operations in @sec-matrix. The C++ examples in each
section can be obtained as standalone files in the
[vignettes/examples][examples-url] folder of the `fntl` source repository.

[stl-url]: https://www.cppreference.com/Cpp_STL_ReferenceManual.pdf
[RcppLambdaPost]: https://gallery.rcpp.org/articles/simple-lambda-func-c++11
[examples-url]: https://github.com/andrewraim/fntl/tree/main/vignettes/examples


# Overview {#sec-overview}

## A First Example {#sec-first-example}

We will first consider a brief example to illustrate use of the `fntl` package.
If attempting to follow along, ensure that you have successfully installed the
`fntl` package on your system. The following C++ code, given in the file
`examples/first.cpp`, computes the integral
$B(a,b) = \int_0^1 x^{a-1} (1-x)^{b-1} dx$.

```{.cpp include="examples/first.cpp"}
```

There are a number of points to note in this example.

1. The first two lines - the `depends` attribute and the include of `fntl.h` -
   are needed to access the `fntl` library from C++.

2. Functions and other objects in `fntl` are accessed through the `fntl`
   namespace (e.g., `fntl::integrate`).

3. The call to `integrate` follows a similar pattern as many functions in
   `fntl`. Primary arguments include the function `f` and the bounds of the
   integral, while optional arguments such as the number of subdivisions are
   passed in a struct of type `integrate_args`. The result of `integrate` is
   an `integrate_result` struct containing the integral approximation `value`,
   a return code `status`, and several other outputs from the operation.

4. The `integrate_args` struct uses default values for any unspecified
   arguments. Users often will not want to specify values such as tolerances
   or verbosities. Similar argument names and default values as the
   corresponding R function are used when possible.

5. The integrand `f` is defined as a `dfd`; this is a typedef in `fntl`
   which is a shorthand for `std::function<double(double)>`.

6. The last line shows the `integrate_result` being wrapped into an Rcpp `List`
   so that it becomes an R list when returned to R.

7. The `status` code is a value from the `integrate_status` enum class. Here it
   is converted to an integer using the function `to_underlying`.
   
8. An R function `first_ex` is generated by specifying an `Rcpp::export`
   annotation. This is done for most functions in this document to facilitate
   demonstrations.

Let us invoke the `first_ex` function through R. 

```{r}
Rcpp::sourceCpp("examples/first.cpp")
out = first_ex(2, 3)
print(out$value)
```

## Arguments

There are a number of `args` structs which represent optional arguments to
functions. `integrate_args` is one example; other `args` types have a prefix
matching their corresponding function. Each of these may
be instantiated from an Rcpp `List` or exported to an Rcpp `List` using the
`as` and `wrap` mechanisms, respectively.^[Details on implementing these
mechanisms may be found in the [Rcpp Extending][rcpp-extend] vignette.]

[rcpp-extend]: https://cran.r-project.org/web/packages/Rcpp/vignettes/Rcpp-extending.pdf

```{cpp}
// Create args and export to a List
fntl::integrate_args args0;
Rcpp::List x = Rcpp::wrap(args0);

// Instantiate a second args struct from the list x
x["stop_on_error"] = false;
fntl::integrate_args args1 = Rcpp::as<fntl::integrate_args>(x);
```

When instantiating an `args` struct from a `List`, we throw an error if the
list contains any elements with names which are not expected by the struct;
this is to protect against mistakes which could occur when a field is named
incorrectly where the effects may be subtle and difficult to track down.

```{cpp}
// This will cause an exception
x["abcdefg"] = 0;
fntl::integrate_args args1 = Rcpp::as<fntl::integrate_args>(x);
```

In most cases, a function that takes optional `args` has an alternative form
where the `args` may be omitted; this form assumes all default values for the
`args`. For example, our call to `integrate` in @sec-first-example could omit
the `args` as follows.

```{cpp}
fntl::integrate_result out = fntl::integrate(f, 0, 1);
```

## Results

Similar to `args`, there are a number of `result` structs that represent the
output of a function. `integrate_result` is an example; other `result` types
have a prefix matching their corresponding function. A `result` may be
exported to an Rcpp `List` using the `wrap` mechanism. This was seen in
@sec-first-example.

```{cpp}
fntl::integrate_result out = fntl::integrate(f, 0, 1, args);
Rcpp::List x = Rcpp::wrap(out);
```

## Status Codes {#sec-status}

Error conditions may result in an exception so that no return value is
produced. Some conditions - such as failure to converge within a given number
of iterations - may not result in an exception. Here, a `status` code is
returned with the result to indicate a possible issue that may warrant further
investigation. The example in @sec-first-example illustrates `integrate_status`
returned by `integrate`; other `status` types have a prefix matching their
corresponding function. Each `status` type is an enum class derived from either
`int` or `unsigned int`: an enumeration that can be constructed from an integer
or converted to an integer using the included function `to_underlying`.^[This
function is based on a post from [StackOverflow][to-underlying-url].]

```{cpp}
fntl::integrate_status status = fntl::integrate_status::OK;  // Define a status
int err = to_underlying(status);                             // status to int
status1 = fntl::integrate_status(err);                       // int to status
```

[to-underlying-url]: https://stackoverflow.com/questions/8357240/how-to-automatically-convert-strongly-typed-enum-into-int

## Function Typedefs

`fntl` defines shorthands for several commonly used function types, given as
follows, which are used throughout the present document.

```{cpp}
typedef function<double(double)> dfd;
typedef function<double(const NumericVector&)> dfv;
typedef function<double(const NumericVector&, const NumericVector&)> dfvv;
typedef function<NumericVector(const NumericVector&)> vfv;
typedef function<NumericMatrix(const NumericVector&)> mfv;
```

Type names are intended to convey argument and return types as briefly as
possible. Symbols to the right of `f` represent arguments while those to the
left represent the return type; in particular, `d` is **d**ouble, `v` is
numeric **v**ector, and `m` is numeric **m**atrix. For example, a `dfvv`
takes two vectors and returns a double.

The namespace `std` for `function` and `Rcpp` for `NumericVector` and
`NumericMatrix` have been omitted in the display so that statements each fit on
a single line.

## Constants

The following constants are defined in `fntl` and utilized in the API.

```{cpp}
double mach_eps = std::numeric_limits<double>::epsilon();
double mach_eps_2r = sqrt(mach_eps);
double mach_eps_4r = std::pow(mach_eps, 0.25);
unsigned int uint_max = std::numeric_limits<unsigned int>::max();
```

These correspond to machine epsilon $\epsilon$, $\epsilon^{1/2}$,
$\epsilon^{1/4}$, and the maximum value of an unsigned integer.

## Error Actions

Several functions in `fntl` take an `error_action` as an input to determine how
to react in an error state. Here is the definition of `error_action`.

```{cpp}
enum class error_action : unsigned int {
	STOP = 3L,     // <1>
	WARNING = 2L,  // <2>
	MESSAGE = 1L,  // <3>
	NONE = 0L      // <4>
};
```
1. Throw an exception.
2. Emit a warning and proceed.
3. Print a message and proceed.
4. Do not take any of the above actions and proceed.

Functions making use of an `error_action` typically also return a status code
as in @sec-status, which can be inspected by the caller if an exception is not
thrown.

## Inferring Return Types$^\dagger$ {#sec-inferring-return}

@sec-intro mentioned that the return type of a lambda does not necessarily need
to be specified in its interface. However, we must be vigilant when allowing
the type to be inferred, especially when working with `Rcpp` objects which are
converted seamlessly behind the scenes.

The following example appears harmless but is likely to crash R.

```{cpp}
// [[Rcpp::depends(fntl)]]
#include "fntl.h"

// [[Rcpp::export]]
Rcpp::List crash_ex(Rcpp::NumericVector x0)
{
	fntl::dfv f = [](Rcpp::NumericVector x) { return Rcpp::sum(x*x); };
	auto out = fntl::gradient(f, x0);
	return Rcpp::wrap(out);
}
```

The issue appears to be that `Rcpp::sum` does not necessarily return a
`double` - the expected result of a `fntl::dfv` - but something else that
normally produces a `double` behind the scenes when needed. Some contextual
information may be missing so that the conversion does not occur.

One way to address this is to specify that `double` is the return type of the
lambda.

```{cpp}
fntl::dfv f = [](Rcpp::NumericVector x) -> double { return Rcpp::sum(x*x); };
```

A second way to address the issue is to explicitly convert the result to a
`double` before returning.

```{cpp}
fntl::dfv f = [](Rcpp::NumericVector x) {
	double out = Rcpp::sum(x*x);
	return out;
};
```

## R Functions as Lambdas

Although not a primary intended use of the `fntl` package, it is possible to
use R functions as inputs. This is accomplished by wrapping an `Rcpp::Function`
in a lambda. This will incur the usual overhead of calling R code from C++,
but may be useful for testing or in situations where the overhead is a small
proportion of the run time.

Here is an example based on the first example in @sec-first-example.

```{.cpp include="examples/callr.cpp"}
```

We create function `f` in R and pass it as an argument to `callr_ex`. The
lambda `ff` calls the function `f`, implicitly converting the input `x` into an
`Rcpp::NumericVector`, and converts the output from an `Rcpp::NumericVector` to
a `double`. Let us demonstrate a call to the function `callr_ex` from R.

```{r}
Rcpp::sourceCpp("examples/callr.cpp")
a = 2
b = 3
f = function(x) { x^(a - 1) * (1 - x)^(b - 1) }
out = callr_ex(f)
print(out$value)
```

## R Interface

The `fntl` package includes an R interface which may be used to invoke much of
the underlying C++ API. This is intended for demonstration and testing purposes
only; performance will generally suffer here because of the overhead in moving
between C++ and R. In real applications, R code should make use of mainstream R
functions rather than this R interface.

For example, the `integrate0` R function is included to call the underlying
`integrate` C++ function shown in @sec-first-example. (The suffix "0" is added
to avoid a naming clash with R's `integrate` function). The R function
`integrate_args` can be used to construct a list which is suitable to pass to
`integrate0`.

```{r}
args = integrate_args()
print(args)
a = 2; b = 3
f = function(x) { x^(a-1) * (1-x)^(b-1) }
out = integrate0(f, 0, 1, args)
print(out$value)
```

## Performance Illustration

To illustrate performance characteristics, let us consider a brief simulation
of the relationship between mean-squared error (MSE) and sample size
in a logistic regression, suppose $Y_i \sim \text{Ber}(p_i)$ are independent
with $p_i = \text{logit}^{-1}(\beta_0 + \beta_1 x_i)$. Data-generating values of
the coefficients are taken to be $\beta_0 = 0$ and $\beta_1 = 1$, respectively.
We take $R$ draws of $(y_1, \ldots, y_n)$ and compute the MLE
$\hat{\beta}^{(r)} = (\hat{\beta}_0^{(r)}, \hat{\beta}_1^{(r)})$
for the $r$th draw using the L-BFGS-B optimization method. The MSE associated
with sample size $n$ is computed as
$\text{MSE}_n = \frac{1}{R} \sum_{r=1}^R \lVert \hat{\beta}^{(r)} - \beta \rVert^2$;
this is computed for sample sizes $n \in \{ 100, 200, 500, 1000, 10000 \}$.
We describe four versions of the code; to see the implementations, refer to the
corresponding source files in the `examples` folder.

```{r}
#| eval: false
source("examples/timing1.R")
Rcpp::sourceCpp("examples/timing2.cpp")
Rcpp::sourceCpp("examples/timing3.cpp")
Rcpp::sourceCpp("examples/timing4.cpp")
n_levels = c(100, 200, 500, 1000, 10000)
```

The first version of the program `timing1_ex` is written in pure R. This
version uses a naively coded version of the loglikelihood based on a loop over
$y_1, \ldots, y_n$. Experienced R users will recognize that vectorization will
dramatically improve the performance. However, we will proceed with the slow
loglikelihood for illustration. A particular run of this code took 1.31 minutes.

```{r}
#| eval: false
set.seed(1234)
start = Sys.time()
timing1_ex(R = 200, n_levels)
print(Sys.time() - start)
```

The second version `timing2_ex` ports `timing1_ex` from R to C++, where loops
generally need not be avoided to achieve good performance. Here we define the
loglikelihood as a lambda function and use the `lbfgsb` function described in
@sec-lbfgs to carry out optimization. A run of this code took 22.17 seconds.

```{r}
#| eval: false
set.seed(1234)
start = Sys.time()
timing2_ex(R = 200, n_levels)
print(Sys.time() - start)
```

A third version of the code `timing3_ex` demonstrates that overhead of calling
an R function from C++ does not necessarily result in poor performance. Here we
make a vectorized call to `dbinom` for each evaluation of the loglikelihood.
In this case, the overhead does not contribute significantly to the run time:
a run took 25.94 seconds.

```{r}
#| eval: false
set.seed(1234)
start = Sys.time()
timing3_ex(R = 200, n_levels)
print(Sys.time() - start)
```

Finally, a fourth version of the code `timing4_ex` demonstrates a case where
the overhead of repeatedly calling an R function from C++ results in abysmal
performance. Here we revert back to the loop in `timing2_ex`, but call the
`plogis` function in R rather than use one provided in Rcpp. A run of this code
took 9.89 minutes.

```{r}
#| eval: false
set.seed(1234)
start = Sys.time()
timing4_ex(R = 200, n_levels)
print(Sys.time() - start)
```

## Pass by Value and Reference

References and const references can be used in C++ code to avoid making
unnecessary copies of variables which waste time and memory. This additional
level of control (and responsibility) is typically not considered in R
programming, where pass-by-value is the standard. Consider the following
function, which adds one to each element of a matrix and returns the sum of
the result.

```{cpp}
double sum1p(Rcpp::NumericMatrix x) { return Rcpp::sum(x + 1); }
```

Here a copy of the matrix `x` is passed to `sum1p`. This pass-by-value can be
changed to pass-by-reference which avoids copying `x`.

```{cpp}
double sum1p_1(Rcpp::NumericMatrix& x) { return Rcpp::sum(x + 1); }
```

The body of `sum1p_1` does not alter `x`, but there are no safeguards in place to
enforce that it does not. This might raise anxiety for a user of `sum1p_1`
about whether their `x` has been modified. To alleviate their anxiety, we can
provide a safeguard using a const reference.

```{cpp}
double sum1p_2(const Rcpp::NumericMatrix& x) { return Rcpp::sum(x + 1); }
```

The compiler will emit an error if `sum1p_2` attempts to modify `x`.

Examples given in this document tend to use pass-by-value to avoid extra visual
clutter of const references. However, the `fntl` package makes frequent use of
const references in both the API and internal implementation. It is recommended
that users consider consider which of the three - by-value, by-reference, or
by-const-reference - is most appropriate for their application in actual usage.

# Integration {#sec-integrate}

Compute the integral
<!-- -->
$$
\int_a^b f(x) dx,
$$
<!-- -->
where limit $a$ may be finite or $-\infty$ and limit $b$ may be finite or
$\infty$. Directly uses the C functions [Rdqags][Rdqags-src] and
[Rdqagi][Rdqagi-src] underlying the R function [integrate][Rintegrate-src].
These functions are based on two respective QUADPACK routines:
[dqags][dqags-src] for the case when both limits are finite and
[dqagi][dqagi-src] for the case when one or both limits are infinite
[@PiessensEtAl1983]. 

[Rintegrate-src]: https://github.com/r-devel/r-svn/blob/main/src/library/stats/R/integrate.R
[Rdqags-src]: https://github.com/r-devel/r-svn/blob/b3133c253e72c7ae069e0d46964c9458a7219eaa/src/include/R_ext/Applic.h#L52
[Rdqagi-src]: https://github.com/r-devel/r-svn/blob/b3133c253e72c7ae069e0d46964c9458a7219eaa/src/include/R_ext/Applic.h#L57
[dqags-src]: https://www.netlib.org/quadpack/dqags.f
[dqagi-src]: https://www.netlib.org/quadpack/dqagi.f

**Function**

Source code is in the file [inst/include/integrate.h][integrate-h].

[integrate-h]: https://github.com/andrewraim/fntl/blob/main/inst/include/integrate.h

```{cpp}
integrate_result integrate(
	const dfd& f,        // <1>
	double lower,                // <2>
	double upper,                // <3>
    const integrate_args& args   // <4>
)

integrate_result integrate(
	const dfd& f,        // <1>
	double lower,                // <2>
	double upper                 // <3>
)
```
1. Function to use as the integrand.
2. Lower limit $a$ of integral; may be `R_NegInf`.
3. Upper limit $b$ of integral; may be `R_PosInf`.
4. Additional arguments.

**Optional Arguments**

```{cpp}
struct integrate_args {
	unsigned int subdivisions = 100L;  // <1>
	double rel_tol = mach_eps_4r;      // <2>
	double abs_tol = mach_eps_4r;      // <3>
	bool stop_on_error = true;         // <4>
};
```
1. The maximum number of subintervals.
2. Relative accuracy requested.
3. Absolute accuracy requested.
4. If `true`, errors in `integrate` raise exceptions.

**Result**

```{cpp}
struct integrate_result {
	double value;             // <1>
	double abs_error;         // <2>
	int subdivisions;         // <3>
	integrate_status status;  // <4>
	int n_eval;               // <5>
	std::string message;      // <6>

	operator SEXP() const;    // <7>
};
```
1. The final approximation of the integral.
2. Estimate of the modulus of the absolute error.
3. The number of subintervals produced in the subdivision process.
4. A code describing the status of the operation.
5. Number of function evaluations.
6. A message describing the status of the operation.
7. Conversion operator to `Rcpp::List`.

The `SEXP` conversion operator produces the following representation of
`integrate_result` as an `Rcpp::List`. The fields here directly correspond to
those in `integrate_result`.

|Name            | Type                  | Description |
|:---------------|:----------------------|:------------|
| `value`        | `Rcpp::NumericVector` | Length 1    |
| `abs_error`    | `Rcpp::NumericVector` | Length 1    |
| `subdivisions` | `Rcpp::IntegerVector` | Length 1    |
| `status`       | `Rcpp::IntegerVector` | Length 1    |
| `n_eval`       | `Rcpp::IntegerVector` | Length 1    |
| `message`      | `Rcpp::StringVector`  | Length 1    |

: {tbl-colwidths="[15,30,55]"}


**Status Codes**

```{cpp}
enum class integrate_status : int {
	OK = 0L,                                  // <1>
	MAX_SUBDIVISIONS = 1L,                    // <2>
	ROUNDOFF_ERROR = 2L,                      // <3>
	BAD_INTEGRAND_BEHAVIOR = 3L,              // <4>
	ROUNDOFF_ERROR_EXTRAPOLATION_TABLE = 4L,  // <5>
	PROBABLY_DIVERGENT_INTEGRAL = 5L,         // <6>
	INVALID_INPUT = 6L                        // <7>
};
```
1. OK.
2. maximum number of subdivisions reached.
3. roundoff error was detected.
4. extremely bad integrand behaviour.
5. roundoff error is detected in the extrapolation table.
6. the integral is probably divergent.
7. the input is invalid.

**Example**

Compute the integral $\int_0^\infty e^{-x^2/2} dx$. A C++ function with Rcpp
interface is defined in the file `examples/integrate.cpp`.

```{.cpp include="examples/integrate.cpp"}
```

Call the function from R.

```{r}
Rcpp::sourceCpp("examples/integrate.cpp")
out = integrate_ex(0 , Inf)
print(out)
```

# Differentiation {#sec-diff}

This section presents methods for numerical differentiation. First we present
simple finite differences, then Richardson extrapolation to automatically
select a step size, then the functions to compute the gradient, Jacobian, and
Hessian. The latter three make use of Richardson extrapolation.

## Finite Differences {#sec-fd-deriv}

Compute the first and second derivatives of
$f : \mathbb{R}^n \rightarrow \mathbb{R}$ numerically at point $x$.
Denote $e_i$ as the $i$th column of an $n \times n$ identity matrix.
First derivatives in the $i$th coordinate are computed as
<!-- -->
\begin{align*}
&\frac{\partial f(x)}{\partial x_i} \approx \frac{
f(x + h e_i) - f(x - h e_i)
}{2h}, \\
%
&\frac{\partial f(x)}{\partial x_i} \approx 
\frac{ f(x + h e_i) - f(x) }{h}, \\
%
&\frac{\partial f(x)}{\partial x_i} \approx 
\frac{ f(x) - f(x - h e_i) }{h},
\end{align*}
<!-- -->
for given $h > 0$ using symmetric, forward, or backward differences
respectively. Second derivatives in the $i$th and $j$th coordinate, with
$i, j \in \{ 1 \ldots, n\}$, are computed as
<!-- -->
\begin{align*}
&\frac{\partial^2 f(x)}{\partial x_i x_j} \approx
\frac{
f(x + h_i e_i + h_j e_j) - f(x + h_i e_i - h_j e_j) - f(x - h_i e_i + h_j e_j) +
f(x - h_i e_i - h_j e_j)
}{4 h_i h_j}, \\
%
&\frac{\partial^2 f(x)}{\partial x_i x_j} \approx
\frac{
f(x + h_i e_i + 2 h_j e_j) - f(x + h e_i) - f(x + h_j e_j) + f(x)
}{ h_i h_j}, \\
%
&\frac{\partial^2 f(x)}{\partial x_i x_j} \approx
\frac{
f(x) - f(x - h_i e_i) - f(x - h_j e_j) + f(x - h_i e_i - h_j e_j)
}{h_i h_j},
\end{align*}
<!-- -->
for given $h_1 > 0$ and $h_2 > 0$
using symmetric, forward, or backward differences respectively.
The accuracy of these derivatives depends on the nature of the function $f$ at
$x$ and the choice of $h$. See Section 5.7 of @PressEtAl2007 for discussion.

**Function**

Primary location of source code is the file [inst/include/fd-deriv.h][fd-deriv-h].

[fd-deriv-h]: https://github.com/andrewraim/fntl/blob/main/inst/include/fd-deriv.h

```{cpp}
double fd_deriv(
	const dfv& f,                                  // <1>
	const Rcpp::NumericVector& x,                  // <2>
	unsigned int i,                                // <3>
	double h,                                      // <5>
	const fd_types& fd_type = fd_types::SYMMETRIC  // <7>
)

double fd_deriv2(
	const dfv& f,                                  // <1>
	const Rcpp::NumericVector& x,                  // <2>
	unsigned int i,                                // <3>
	unsigned int j,                                // <4>
	double h_i,                                    // <5>
	double h_j,                                    // <6>
	const fd_types& fd_type = fd_types::SYMMETRIC  // <7>
)
```
1. Function $d$ to differentiate.
2. Point $x$ at which derivative is taken.
3. First coordinate to differentiate.
4. Second coordinate to differentiate.
5. Step size in the first coordinate.
6. Step size in the second coordinate
7. Type of finite difference.

The functions `fd_deriv` and `fd_deriv2` compute first and second derivatives,
respectively. The options for `fd_type` are given in the following enumeration
class.

```{cpp}
enum class fd_types : unsigned int {
	SYMMETRIC = 0L,
	FORWARD = 1L,
	BACKWARD = 2L
};
```

**Example**

Compute the first and second derivatives of $f(x) = \sin(b^\top x)$ at
$x_0 = (1/2, \ldots, 1/2)$ where $b = (1, \ldots, n)$. The gradient and Hessian
are given in closed-form by
<!-- -->
\begin{align*}
&\frac{\partial f(x)}{\partial x} = b \cos(b^\top x), \\
&\frac{\partial^2 f(x)}{\partial x \partial x^\top} = -b b^\top \sin(b^\top x).
\end{align*}
<!-- -->
Let us first prepare the problem in R.

```{r}
n = 3
b = seq_len(n)
x0 = rep(0.5, n)
eps = 0.001  ## Fix a step size for finite differences
g = function(x) { b * cos(sum(b * x)) }
h = function(x) { -tcrossprod(b) * sin(sum(b * x)) }
```

To demonstrate the numerical first derivative, a C++ function with Rcpp
interface is defined in the file `examples/fd-deriv.cpp`.

```{.cpp include="examples/fd-deriv.cpp"}
```

Call the function from R.

```{r}
Rcpp::sourceCpp("examples/fd-deriv.cpp")
out3 = out2 = out1 = numeric(n)
for (i in 1:n) {
	out1[i] = fd_deriv_ex(x0, i-1, eps, type = 0)  ## Symmetric
	out2[i] = fd_deriv_ex(x0, i-1, eps, type = 1)  ## Forward
	out3[i] = fd_deriv_ex(x0, i-1, eps, type = 2)  ## Backward
}
print(out1)
print(out2)
print(out3)
print(g(x0))
```

To demonstrate the numerical second derivative, a C++ function with Rcpp
interface is defined in the file `examples/fd-deriv2.cpp`.

```{.cpp include="examples/fd-deriv2.cpp"}
```

Call the function from R.

```{r}
Rcpp::sourceCpp("examples/fd-deriv2.cpp")
out3 = out2 = out1 = matrix(NA, n, n)
for (i in 1:n) {
for (j in 1:n) {
	out1[i,j] = fd_deriv2_ex(x0, i-1, j-1, eps, eps, type = 0)  ## Symmetric
	out2[i,j] = fd_deriv2_ex(x0, i-1, j-1, eps, eps, type = 1)  ## Forward
	out3[i,j] = fd_deriv2_ex(x0, i-1, j-1, eps, eps, type = 2)  ## Backward
}
}
print(out1)
print(out2)
print(out3)
h(x0)
```

## Richardson Extrapolated Finite Differences {#sec-deriv}

Compute the first and second derivatives of
$f : \mathbb{R}^n \rightarrow \mathbb{R}$ numerically at point $x$ using
Richardson extrapolation. This is typically more accurate than the simple
finite differences in @sec-fd-deriv and does not require an informed choice of
the step size $h$; however, it requires more evaluations of $f$. A general
discussion of Richardson extrapolation is given in Section 9.6 of
@QuarteroniSaccoSaleri2007.

Taking the function $g(h)$ to be one of finite difference approximations 
in \eqref{eqn:finite-differences1}--\eqref{eqn:finite-differences3}, $h > 0$ to
be an initial step size, and $\delta \in (0,1)$ to be a reduction factor, the
method computes a table of values
<!-- -->
\begin{align*}
A_{mq} = 
\begin{cases}
g(\delta^m h), & \text{for $m = 0, \ldots, n$ and $q = 0$}, \\
\displaystyle \frac{A_{m,q-1} - \delta^q A_{m-1,q-1}}{1 - \delta^q}, & 
\text{for $m = 0, \ldots, n$ and $q = 1, \ldots, m$},
\end{cases}
\end{align*}
<!-- -->
given a predetermined $n$. Note that $n+1$ evaluations of $f$ are required. The
result is taken to be the $A_{mq}$ such that
<!-- -->
$$
e_{mq} = \max\{ |A_{mq} - A_{m,q-1}|, |A_{mq} - A_{m-1,q-1}| \}
$$
<!-- -->
is the smallest, with the corresponding $e = e_{mq}$ reported as the achieved
tolerance. Furthermore, because numerical error may begin to worsen as the
table is iteratively computed, the method halts if it encounters the condition
<!-- -->
$$
|A_{mq} - A_{m-1,q-1}| > \tau e,
$$
<!-- -->
with $e$ as the smallest $e_{mq}$ computed so far and $\tau > 1$ is a given
multiplier. These criteria for convergence and stopping early due to reduced
numerical precision are suggested in Section 5.7 of @PressEtAl2007; similar
considerations for halting criteria are considered in the Julia package
[Richardson.jl][richardson-jl].

Taking $n = 0$ produces finite differences described in @sec-fd-deriv using the
initial step size $h$ as the perturbation. Here the achieved tolerance is
reported as infinite.

[richardson-jl]: https://github.com/JuliaMath/Richardson.jl

**Function**

Primary location of source code are the files
[inst/include/deriv.h][deriv-h],
[inst/include/deriv2.h][deriv2-h], and
[inst/include/richardson.h][richardson-h].

[deriv-h]: https://github.com/andrewraim/fntl/blob/main/inst/include/deriv.h
[deriv2-h]: https://github.com/andrewraim/fntl/blob/main/inst/include/deriv2.h
[richardson-h]: https://github.com/andrewraim/fntl/blob/main/inst/include/richardson.h


```{cpp}
richardson_result deriv(
	const dfv& f,                                  // <1>
	const Rcpp::NumericVector& x,                  // <2>
	unsigned int i,                                // <3>
	const richardson_args& args,                   // <5>
	const fd_types& fd_type = fd_types::SYMMETRIC  // <6>
)

richardson_result deriv(
	const dfv& f,                                  // <1>
	const Rcpp::NumericVector& x,                  // <2>
	unsigned int i,                                // <3>
	const fd_types& fd_type = fd_types::SYMMETRIC  // <6>
)

richardson_result deriv2(
	const dfv& f,                                  // <1>
	const Rcpp::NumericVector& x,                  // <2>
	unsigned int i,                                // <3>
	unsigned int j,                                // <4>
	const richardson_args& args,                   // <5>
	const fd_types& fd_type = fd_types::SYMMETRIC  // <6>
)

richardson_result deriv2(
	const dfv& f,                                  // <1>
	const Rcpp::NumericVector& x,                  // <2>
	unsigned int i,                                // <3>
	unsigned int j,                                // <4>
	const fd_types& fd_type = fd_types::SYMMETRIC  // <6>
)
```
1. Function $d$ to differentiate.
2. Point $x$ at which derivative is taken.
3. Optional arguments.
4. First coordinate to differentiate.
5. Second coordinate to differentiate.
6. Type of finite difference.

The functions `deriv` and `deriv2` compute first and second derivatives,
respectively. The enum class `fd_type` is described in @sec-fd-deriv.


**Optional Arguments**

```{cpp}
struct richardson_args
{
	double delta = 0.5;                 // <1>
	unsigned int maxiter = 10;          // <2>
	double h = 1;                       // <3>
	double tol = mach_eps_4r;           // <4>
	double accuracy_factor = R_PosInf;  // <5>

	richardson_args() { };              // <6>
	richardson_args(SEXP obj);          // <7>
	operator SEXP() const;              // <8>
};

```
1. The factor $\delta$ used to reduce $h$.
2. Maximum number of iterations.
3. The initial value of $h$.
4. Tolerance for convergence.
5. The factor $\tau$ used to check for loss of precision. The infinite default
   value disables the check.
6. Default constructor.
7. Constructor from an `Rcpp::List`.
8. Conversion operator to `Rcpp::List`.

**Result**

```{cpp}
struct richardson_result
{
	double value;              // <1>
	double err;                // <2>
	unsigned int iter;         // <3>
	richardson_status status;  // <4>

	operator SEXP() const;     // <5>
};
```
1. The final approximation of the derivative.
2. An estimate of the error in approximation.
3. Number of iterations $m$ used to produce the approximation.
4. A code describing the status of the operation. 
5. Conversion operator to `Rcpp::List`.

The `SEXP` conversion operator produces the following representation of
`fd_deriv_result` as an `Rcpp::List`.

|Name      | Type                  | Description                       |
|:---------|:----------------------|:----------------------------------|
| `value`  | `Rcpp::NumericVector` | A scalar based on field `value`.  |
| `err`    | `Rcpp::NumericVector` | A scalar based on field `err`.    |
| `iter`   | `Rcpp::IntegerVector` | A scalar based on field `iter`.   |
| `status` | `Rcpp::IntegerVector` | A scalar based on field `status`.   |

**Status Codes**

```{cpp}
enum class richardson_status : unsigned int {
	OK = 0L,                  // <1>
	NOT_CONVERGED = 1L,       // <2>
	NUMERICAL_PRECISION = 2L  // <3>
};
```
1. OK.
2. Not converged within `maxiter` iterations.
3. Early termination due to numerical precision.

**Example**

Compute first and second derivatives of $f(x) = \sin(b^\top x)$ at
$x_0 = (1/2, \ldots, 1/2)$ with $b = (1, \ldots, n)$. This is a continuation of
the example in @sec-fd-deriv. Let us again define the point $x_0$.

```{r}
n = 3
x0 = rep(0.5, n)
```

To demonstrate the numerical first derivative, a C++ function with Rcpp
interface is defined in the file `examples/deriv.cpp`.

```{.cpp include="examples/deriv.cpp"}
```

Call the function from R.

```{r}
Rcpp::sourceCpp("examples/deriv.cpp")
out3 = out2 = out1 = numeric(n)
for (i in 1:n) {
	out1[i] = deriv_ex(x0, i-1, type = 0)$value  ## Symmetric
	out2[i] = deriv_ex(x0, i-1, type = 1)$value  ## Forward
	out3[i] = deriv_ex(x0, i-1, type = 2)$value  ## Backward
}
print(out1)
print(out2)
print(out3)
```

To demonstrate the numerical second derivative, a C++ function with Rcpp
interface is defined in the file `examples/deriv2.cpp`.

```{.cpp include="examples/deriv2.cpp"}
```

Call the function from R.

```{r}
Rcpp::sourceCpp("examples/deriv2.cpp")
out3 = out2 = out1 = matrix(NA, n, n)
for (i in 1:n) {
for (j in 1:n) {
	out1[i,j] = deriv2_ex(x0, i-1, j-1, type = 0)$value  ## Symmetric
	out2[i,j] = deriv2_ex(x0, i-1, j-1, type = 1)$value  ## Forward
	out3[i,j] = deriv2_ex(x0, i-1, j-1, type = 2)$value  ## Backward
}
}
print(out1)
print(out2)
print(out3)
```

## Gradient {#sec-gradient}

The gradient of $f : \mathbb{R}^n \rightarrow \mathbb{R}$:
<!-- -->
$$
\frac{\partial f(x)}{\partial x}
=
\left[
\frac{\partial f(x)}{\partial x_1},
\ldots,
\frac{\partial f(x)}{\partial x_n}
\right].
$$
<!-- -->
Each coordinate is computed via `deriv` in @sec-deriv.

**Function**

Primary location of source code is the file [inst/include/gradient.h][gradient-h].

[gradient-h]: https://github.com/andrewraim/fntl/blob/main/inst/include/gradient.h

```{cpp}
gradient_result gradient(
	const dfv& f,                                  // <1>
	const Rcpp::NumericVector& x,                  // <2>
	const richardson_args& args,                   // <3>
	const fd_types& fd_type = fd_types::SYMMETRIC  // <4>
)

gradient_result gradient(
	const dfv& f,                                  // <1>
	const Rcpp::NumericVector& x,                  // <2>
	const fd_types& fd_type = fd_types::SYMMETRIC  // <4>
)
```
1. Function to take the gradient of.
2. Point at which to compute the gradient.
3. Optional arguments.
4. Type of finite difference to use. See the definition of `fd_types` in
   @sec-fd-deriv.

The arguments in `args` are applied to each coordinate of the gradient.

**Result**

```{cpp}
struct gradient_result {
	std::vector<double> value;       // <1>
	std::vector<double> err;         // <2>
	std::vector<unsigned int> iter;  // <3>

	operator SEXP() const;           // <4>
};
```
1. The final approximation of the gradient.
2. The respective approximation errors from `deriv` for each coordinate.
2. The respective iterations taken in `deriv` for each coordinate.
4. Conversion operator to `Rcpp::List`.

The `SEXP` conversion operator produces the following representation of
`gradient_result` as an `Rcpp::List`. The fields here directly correspond to
those in `gradient_result`.

|Name     | Type                  | Description |
|:--------|:----------------------|:------------|
| `value` | `Rcpp::NumericVector` | Length $n$  |
| `err`   | `Rcpp::NumericVector` | Length $n$  |
| `iter`  | `Rcpp::IntegerVector` | Length $n$  |

: {tbl-colwidths="[15,30,55]"}

**Example**

Compute the gradient of $f(x) = x^\top x$. A C++ function with Rcpp interface
is defined in the file `examples/gradient.cpp`.

```{.cpp include="examples/gradient.cpp"}
```

Call the function from R.

```{r}
Rcpp::sourceCpp("examples/gradient.cpp")
gradient_ex(x0 = 1:4)
```

Compare to `grad` from the `numDeriv` package [@numDeriv].

```{r}
f = function(x) { sum(x^2) }
numDeriv::grad(f, x = 1:4)
```

## Jacobian

The Jacobian of $f : \mathbb{R}^n \rightarrow \mathbb{R}^m$:
<!-- -->
$$
\frac{\partial f(x)}{\partial x}
=
\begin{bmatrix}
\frac{\partial f_1(x)}{\partial x_1} & \cdots & \frac{\partial f_1(x)}{\partial x_n} \\
\vdots & \ddots & \vdots \\
\frac{\partial f_m(x)}{\partial x_1} & \cdots & \frac{\partial f_m(x)}{\partial x_n}
\end{bmatrix}.
$$
<!-- -->
Each coordinate is computed via `deriv` in @sec-deriv.

**Function**

Source code is in the file [inst/include/jacobian.h][jacobian-h].

[jacobian-h]: https://github.com/andrewraim/fntl/blob/main/inst/include/jacobian.h

```{cpp}
jacobian_result jacobian(
	const vfv& f,                                 // <1>
	const Rcpp::NumericVector& x,                 // <2>
	const richardson_args& args                   // <3>
	const fd_types& fd_type = fd_types::SYMMETRIC // <4>
)

jacobian_result jacobian(
	const vfv& f,                                 // <1>
	const Rcpp::NumericVector& x,                 // <2>
	const fd_types& fd_type = fd_types::SYMMETRIC // <4>
)
```
1. Function to obtain the Jacobian of
2. Point at which to take the Jacobian.
3. Optional arguments.
4. Type of finite difference to use. See the definition of `fd_types` in
   @sec-fd-deriv.

**Result**

```{cpp}
struct jacobian_result
{
	std::vector<double> value;       // <1>
	std::vector<double> err;         // <2>
	std::vector<unsigned int> iter;  // <3>
	double rows;                     // <4>
	double cols;                     // <5>

	operator SEXP() const;           // <6>
};
```
1. The final approximation of the Jacobian stored as a vector in row-major
   format.
2. The respective approximation errors from `deriv` for each coordinate of
   `value`.
3. The respective iterations taken in `deriv` for each coordinate of `value`.
4. The row dimension $m$ of the Jacobian.
5. The column dimension $n$ of the Jacobian.
6. Conversion operator to `Rcpp::List`.

The `SEXP` conversion operator produces the following representation of
`jacobian_result` as an `Rcpp::List`.

|Name     | Type                  | Description                                    |
|:--------|:----------------------|:-----------------------------------------------|
| `value` | `Rcpp::NumericMatrix` | An $m \times n$ matrix based on field `value`. |
| `err`   | `Rcpp::NumericMatrix` | An $m \times n$ matrix based on field `err`.   |
| `iter`  | `Rcpp::IntegerMatrix` | An $m \times n$ matrix based on field `iter`.  |

: {tbl-colwidths="[15,30,55]"}

**Example**

Compute the Jacobian of $f(x) = [f_1(x), \ldots, f_5(x)]$,
$f_i(x) = \sum_{j=1}^i \sin(x_j)$, at the point $x = (1,2,3,4,5)$. To obtain
the resulting value as an $m \times n$ matrix, we apply the List conversion
operator to the result. A C++ function with Rcpp interface is defined in the
file `examples/jacobian.cpp`.

```{.cpp include="examples/jacobian.cpp"}
```

Call the function from R.

```{r}
Rcpp::sourceCpp("examples/jacobian.cpp")
out = jacobian_ex(x0 = 1:4)
names(out)
print(out$value)
```

Compare to `jacobian` from the `numDeriv` package [@numDeriv].

```{r}
f = function(x) { cumsum(sin(x)) }
numDeriv::jacobian(f, x = 1:4)
```

## Hessian {#sec-hessian}

Compute the Hessian of $f : \mathbb{R}^n \rightarrow \mathbb{R}$ numerically
at point $x$:
<!-- -->
$$
\frac{\partial^2 f(x)}{\partial x \partial x^\top} =
\begin{bmatrix}
\frac{\partial^2 f(x)}{\partial x_1 \partial x_1} &
\cdots &
\frac{\partial^2 f(x)}{\partial x_1 \partial x_n} \\
\vdots & \ddots & \vdots \\
\frac{\partial^2 f(x)}{\partial x_n \partial x_1} &
\cdots &
\frac{\partial^2 f(x)}{\partial x_n \partial x_n}
\end{bmatrix}.
$$
<!-- -->

Each coordinate is computed via `deriv2` in @sec-deriv.

Note that there is a C function [fdhess][fdhess-src] within R which is based on
Algorithm A5.6.2 of @DennisSchnabel1983. However, it is not included in the API
for external use [@WritingRExtensions, Section 6].

[fdhess-src]: https://github.com/r-devel/r-svn/blob/5c0e367225ec0ed9e1a95864302a6311708533ee/src/include/R_ext/Applic.h#L131


**Function**

Primary location of source code is the file [inst/include/hessian.h][hessian-h].

[hessian-h]: https://github.com/andrewraim/fntl/blob/main/inst/include/hessian.h

```{cpp}
hessian_result hessian(
	const dfv& f,                                  // <1>
	const Rcpp::NumericVector& x,                  // <2>
	const richardson_args& args,                   // <3>
	const fd_types& fd_type = fd_types::SYMMETRIC  // <4>
)

hessian_result hessian(
	const dfv& f,                                  // <1>
	const Rcpp::NumericVector& x,                  // <2>
	const fd_types& fd_type = fd_types::SYMMETRIC  // <4>
)
```
1. Function $f$ for which Hessian is to be computed.
2. Point $x$ at which Hessian is taken.
3. Optional arguments.
4. Type of finite difference to use. See the definition of `fd_types` in
   @sec-fd-deriv.

**Result**

The result is an $n \times n$ Hessian matrix.

```{cpp}
struct hessian_result
{
	std::vector<double> value;       // <1>
	std::vector<double> err;         // <2>
	std::vector<unsigned int> iter;  // <3>
	double dim;                      // <4>

	operator SEXP() const;           // <5>
};
```
1. The value of the Hessian: a vector containing the lower-triangular in
   column-major order.
2. The respective approximation errors from `deriv2` for each coordinate.
3. The respective iterations taken in `deriv2` for each coordinate.
4. The row and column dimension.
5. Conversion operator to `Rcpp::List`.

**Example**

Compute the Hessian of $f(x) = \sum_{i=1}^n \sin(x_i)$ at $x = (1, 2)$. To
obtain the resulting value as an $n \times n$ matrix, we apply the List
conversion operator to the result. A C++ function with Rcpp interface is
defined in the file `examples/hessian.cpp`.

```{.cpp include="examples/hessian.cpp"}
```

Call the function from R.

```{r}
Rcpp::sourceCpp("examples/hessian.cpp")
out = hessian_ex(x0 = c(1,2))
print(out)
```

Compare to `hessian` from the `numDeriv` package [@numDeriv].

```{r}
f = function(x) { sum(sin(x)) }
numDeriv::hessian(f, x = c(1,2))
```

# Root-Finding {#sec-findroot}

Find a root of $f : \mathbb{R} \rightarrow \mathbb{R}$, if present, on the
interval $[a,b]$; i.e., find $x \in [a,b]$ such that $f(x) \approx 0$.
The R function [uniroot][Runiroot-src] has its implementation in an underlying
C function [zeroin2][zeroin2-src] which appears not to be readily exported
for external use. We therefore provide several alternatives in C++.

[Runiroot-src]: https://github.com/r-devel/r-svn/blob/685baa84532ee09727f04725d5128f39d564bea5/src/library/stats/R/nlm.R#L55
[zeroin2-src]: https://github.com/r-devel/r-svn/blob/685baa84532ee09727f04725d5128f39d564bea5/src/library/stats/src/stats.h#L61C8-L61C17

There are currently two implementations. The bisection method
[e.g., @PressEtAl2007, Section 9.1] is simpler while Brent's method is
faster and is used in `uniroot`. This implementation of Brent's method is based
on the ALGOL code in Section 4.6 of @Brent1973. The two functions use a common
arguments struct, results struct, and status codes, which are given below.

**Optional Arguments**

```{cpp}
struct findroot_args {
	double tol = mach_eps_4r;                      // <1>
	unsigned int maxiter = 1000;                   // <2>
	error_action action = error_action::STOP;      // <3>
	
	findroot_args() { };                           // <4>
	findroot_args(SEXP obj);                       // <5>
	operator SEXP() const;                         // <6>
};
```
1. Tolerance for convergence.
2. Maximum number of iterations.
3. Action to take if operation returns a status code other than `OK`.
4. Default constructor.
5. Constructor from an `Rcpp::List`.
6. Conversion operator to `Rcpp::List`.

**Result**

```{cpp}
struct findroot_result {
	double root;                 // <1>
	double f_root;               // <2>
	unsigned int iter;           // <3>
	double tol;                  // <4>
	findroot_status status;      // <5>
	std::string message;         // <6>

	operator SEXP() const;       // <7>
};
```
1. The final approximation for the root.
2. The value of $f$ at the root.
3. The number of iterations.
4. An estimate of the error in approximation.
5. A code describing the status of the operation.
6. A message describing the status of the operation.
7. Conversion operator to `Rcpp::List`.

The `SEXP` conversion operator produces the following representation of
`findroot_result` as an `Rcpp::List`. The fields here directly correspond to
those in `findroot_result`.

|Name       | Type                  | Description |
|:----------|:----------------------|:------------|
| `root`    | `Rcpp::NumericVector` | Length 1    |
| `f_root`  | `Rcpp::NumericVector` | Length 1    |
| `iter`    | `Rcpp::IntegerVector` | Length 1    |
| `tol`     | `Rcpp::NumericVector` | Length 1    |
| `status`  | `Rcpp::IntegerVector` | Length 1    |
| `message` | `Rcpp::StringVector`  | Length 1    |

: {tbl-colwidths="[15,30,55]"}

**Status Codes**

```{cpp}
enum class findroot_status : unsigned int {
	OK = 0L,                  // <1>
	NUMERICAL_OVERFLOW = 1L,  // <2>
	NOT_CONVERGED = 2L        // <3>
};
```
1. OK.
2. Numerical overflow: tol may be too small.
3. Not converged within `maxiter` iterations.


## Bisection {#sec-bisection}

This algorithm successively shrinks the starting interval and terminates when
the absolute difference is smaller than a tolerance $\epsilon$.

**Function**

Primary location of source code is the file [inst/include/findroot-bisect.h][findroot-bisect-h].

[findroot-bisect-h]: https://github.com/andrewraim/fntl/blob/main/inst/include/findroot-bisect.h

```{cpp}
findroot_result findroot_bisect(
	const dfd& f,              // <1>
	double lower,              // <2>
	double upper,              // <3>
	const findroot_args& args  // <4>
)

findroot_result findroot_bisect(
	const dfd& f,              // <1>
	double lower,              // <2>
	double upper               // <3>
)
```
1. Function $f$ for which a root is desired.
2. Lower limit $a$ of search interval. Must be finite.
3. Upper limit $b$ of search interval. Must be finite.
4. Additional arguments.


**Example**

Find the root of the function $f(x) = x^2 - 1$ on $[0,10]$. A C++ function with
Rcpp interface is defined in the file `examples/findroot-bisect.cpp`.

```{.cpp include="examples/findroot-bisect.cpp"}
```

Call the function from R.

```{r}
Rcpp::sourceCpp("examples/findroot-bisect.cpp")
out = findroot_bisect_ex(0, 10)
print(out)
```

## Brent's Algorithm

The algorithm successively shrinks the starting interval and terminates when
the absolute difference is smaller than a tolerance $\epsilon$.

**Function**

Primary location of source code is the file [inst/include/findroot-brent.h][findroot-brent-h].

[findroot-brent-h]: https://github.com/andrewraim/fntl/blob/main/inst/include/findroot-brent.h

```{cpp}
findroot_result findroot_brent(
	const dfd& f,              // <1>
	double lower,              // <2>
	double upper,              // <3>
	const findroot_args& args  // <4>
)

findroot_result findroot_brent(
	const dfd& f,              // <1>
	double lower,              // <2>
	double upper               // <3>
)
```
1. Function $f$ for which a root is desired.
2. Lower limit $a$ of search space.
3. Upper limit $b$ of search space.
4. Additional arguments.

**Example**

Find the root of the function $f(x) = x^2 - 1$ on $[0,10]$. A C++ function with
Rcpp interface is defined in the file `examples/findroot-brent.cpp`.

```{.cpp include="examples/findroot-brent.cpp"}
```

Call the function from R.

```{r}
Rcpp::sourceCpp("examples/findroot-brent.cpp")
out = findroot_brent_ex(0, 10)
print(out)
```

# Univariate Optimization {#sec-optimize}

Minimize the function $f : \mathbb{R} \rightarrow \mathbb{R}$ on a given
interval $[a, b]$. The R [optimize][Roptimize-src] function is based on an
underlying [Brent_fmin][optimize-src] C implementation of Brent's algorithm
[@Brent1973, Section 5]. However, this function appears not to be exported
from R for external use.

We provide several alternatives in C++: The golden section search method
[e.g., @PressEtAl2007, Section 10.2] is simple, guaranteed to converge, and
does not require information about derivatives. Brent's algorithm is more
involved but converges faster. The implementation of Brent's algorithm in C++
based is on the ALGOL code in Section 5.8 of @Brent1973. The golden section and
Brent functions use a common arguments struct, results struct, and status codes,
which are given below.

[optimize-src]: https://github.com/r-devel/r-svn/blob/5521fa62273d38f50e7e963337d74ec2f223c532/src/library/stats/src/optimize.c#L94
[Roptimize-src]: https://github.com/r-devel/r-svn/blob/5521fa62273d38f50e7e963337d74ec2f223c532/src/library/stats/R/nlm.R#L35

**Optional Arguments**

```{cpp}
struct optimize_args
{
	bool fnscale = 1;                          // <1>
	double tol = mach_eps_2r;                  // <2>
	unsigned int maxiter = 1000;               // <3>
	unsigned int report_period = uint_max;     // <4>
	error_action action = error_action::STOP;  // <5>

	optimize_args() { };                       // <6>
	optimize_args(SEXP obj);                   // <7>
	operator SEXP() const;                     // <8>
};


```
1. Scaling factor to be applied to the value of $f(x)$ during optimization. Use
   `-1` to implement maximization rather than minimization.
2. Tolerance $\epsilon$ for convergence.
3. The maximum number of iterations.
4. The frequency of reports.
5. Action to take if operation returns a status code other than `OK`.
6. Default constructor.
7. Constructor from an `Rcpp::List`.
8. Conversion operator to `Rcpp::List`.

**Result**

```{cpp}
struct optimize_result
{
	double par;                   // <1>
	double value;                 // <2>
	unsigned int iter;            // <3>
	double tol;                   // <4>
	optimize_status status;       // <5>
	std::string message;          // <6>

	operator SEXP() const;        // <7>
};
```
1. The final value of the optimization variable.
2. The value of the function corresponding to `par`.
3. The number of iterations taken.
4. The achieved tolerance $\delta$.
5. Status code from the optimizer.
6. A message describing the status of the operation.
7. Conversion operator to `Rcpp::List`.

The `SEXP` conversion operator produces the following representation of
`optimize_result` as an `Rcpp::List`. The fields here directly correspond to
those in `optimize_result`.

|Name       | Type                  | Description |
|:----------|:----------------------|:------------|
| `par`     | `Rcpp::NumericVector` | Length 1    |
| `value`   | `Rcpp::NumericVector` | Length 1    |
| `iter`    | `Rcpp::IntegerVector` | Length 1    |
| `tol`     | `Rcpp::NumericVector` | Length 1    |
| `status`  | `Rcpp::IntegerVector` | Length 1    |
| `message` | `Rcpp::StringVector`  | Length 1    |

: {tbl-colwidths="[15,30,55]"}

**Status Codes**

```{cpp}
enum class optimize_status : unsigned int {
	OK = 0L,                  // <1>
	NUMERICAL_OVERFLOW = 1L,  // <2>
	NOT_CONVERGED = 2L        // <3>
};
```
1. OK.
2. Numerical overflow: tol may be too small.
3. iteration limit had been reached.

## Golden Section Search {#sec-golden}

The algorithm successively shrinks the starting interval and terminates when
$\delta = b - a$ is smaller than a given tolerance $\epsilon$.

**Function**

Primary location of source code is the file [inst/include/goldensection.h][goldensection-h].

[goldensection-h]: https://github.com/andrewraim/fntl/blob/main/inst/include/goldensection.h

```{cpp}
optimize_result goldensection(
	const dfd& f,              // <1>
	double lower,              // <2>
	double upper,              // <3>
	const optimize_args& args  // <4>
)

optimize_result goldensection(
	const dfd& f,              // <1>
	double lower,              // <2>
	double upper               // <3>
)
```
1. Function $f$ to minimize.
2. Lower limit $a$ of search space.
2. Upper limit $b$ of search space.
4. Additional arguments.

**Example**

Maximize the function $f(x) = \exp(-x)$ on $[0,1]$. A C++ function with Rcpp
interface is defined in the file `examples/goldensection.cpp`.

```{.cpp include="examples/goldensection.cpp"}
```

Call the function from R.

```{r}
Rcpp::sourceCpp("examples/goldensection.cpp")
out = goldensection_ex(0, 1)
print(out)
```

## Brent's Algorithm {#sec-brent-optim}

This algorithm successively shrinks the starting interval and terminates when
$\delta = |x - m|$ is smaller than a tolerance $\epsilon$, where
$m = (a + b) / 2$ is the midpoint of the current $[a, b]$.

**Function**

Primary location of source code is the file [inst/include/optimize-brent.h][optimize-brent-h].

[optimize-brent-h]: https://github.com/andrewraim/fntl/blob/main/inst/include/optimize-brent.h

```{cpp}
optimize_result optimize_brent(
	const dfd& f,              // <1>
	double lower,              // <2>
	double upper,              // <3>
	const optimize_args& args  // <4>
)

optimize_result optimize_brent(
	const dfd& f,              // <1>
	double lower,              // <2>
	double upper               // <3>
)
```
1. Function $f$ to minimize.
2. Lower limit $a$ of search space.
2. Upper limit $b$ of search space.
4. Additional arguments.

**Example**

Maximize the function $f(x) = \exp(-x)$ on $[0,1]$. A C++ function with Rcpp
interface is defined in the file `examples/optimize-brent.cpp`.

```{.cpp include="examples/optimize-brent.cpp"}
```

Call the function from R.

```{r}
Rcpp::sourceCpp("examples/optimize-brent.cpp")
out = optimize_brent_ex(0, 1)
print(out)
```

# Multivariate Optimization {#sec-optim}

This section presents methods to minimize a function
$f : \mathbb{R}^n \rightarrow \mathbb{R}$ from a given starting value $x_0$.

## Nelder-Mead

Relies only on evaluation of $f$ and does not use the
gradient or Hessian [@NelderMead1965]. This function directly calls
[nmmin][nmmin-src], the C function that gets invoked when using the
[optim][Roptim-src] R function with `method = "Nelder-Mead"`.

[nmmin-src]: https://github.com/r-devel/r-svn/blob/62dd8b09461d5d1313f4722d9d63abb463a9e42d/src/include/R_ext/Applic.h#L71
[Roptim-src]: https://github.com/r-devel/r-svn/blob/main/src/library/stats/R/optim.R

**Function**

Primary location of source code is the file [inst/include/neldermead.h][neldermead-h].

[neldermead-h]: https://github.com/andrewraim/fntl/blob/main/inst/include/neldermead.h

```{cpp}
neldermead_result neldermead(
	const Rcpp::NumericVector& init,  // <1>
	const dfv& f,                     // <2>
	const neldermead_args& args       // <3>
)

neldermead_result neldermead(
	const Rcpp::NumericVector& init,  // <1>
	const dfv& f                      // <2>
)
```
1. Initial value for optimization variable.
2. Function $f$ to minimize.
3. Additional arguments.

**Optional Arguments**

```{cpp}
struct neldermead_args
{
	double alpha = 1.0;           // <1>
	double beta = 0.5;            // <2>
	double gamma = 2.0;           // <3>
	unsigned int trace = 0;       // <4>
	double abstol = R_NegInf;     // <5>
	double reltol = mach_eps_2r;  // <6>
	unsigned int maxit = 500;     // <7>
	double fnscale = 1.0;         // <8>

	neldermead_args() { };        // <9>
	neldermead_args(SEXP obj);    // <10>
	operator SEXP() const;        // <11>
};
```
1. Reflection factor.
2. Contraction and reduction factor.
3. Extension factor.
4. If positive, print progress info.
5. Absolute tolerance.
6. User-initialized conversion tolerance.
7. Maximum number of iterations.
8. Scaling factor to be applied to the value of $f(x)$ during optimization. Use
   `-1` to implement maximization rather than minimization.
9. Default constructor.
10. Constructor from an `Rcpp::List`.
11. Conversion operator to `Rcpp::List`.

**Result**

```{cpp}
struct neldermead_result {
	std::vector<double> par;   // <1>
	double value;              // <2>
	neldermead_status status;  // <3>
	int fncount;               // <4>

	operator SEXP() const;     // <5>
};
```
1. The final approximation of the optimizer.
2. The value of $f$ at the optimizer.
3. A code describing the status of the operation.
4. The number of function evaluations.
5. Conversion operator to `Rcpp::List`.

The `SEXP` conversion operator produces the following representation of
`neldermead_result` as an `Rcpp::List`. The fields here directly correspond to
those in `neldermead_result`.

|Name       | Type                  | Description |
|:----------|:----------------------|:------------|
| `par`     | `Rcpp::NumericVector` | Length $n$  |
| `value`   | `Rcpp::NumericVector` | Length 1    |
| `status`  | `Rcpp::IntegerVector` | Length 1    |
| `fncount` | `Rcpp::IntegerVector` | Length 1    |

: {tbl-colwidths="[15,30,55]"}

**Status Codes**

```{cpp}
enum class neldermead_status : unsigned int {
	OK = 0L,                 // <1>
	NOT_CONVERGED = 1L,      // <2>
	SIMLEX_DEGENERACY = 10L  // <3>
};
```
1. OK.
2. Not converged within `maxiter` iterations.
3. Degeneracy of the Nelder–Mead simplex.

**Example**

Minimize the function $f(x) = \sum_{i=1}^n x_i^2 - 1$ for $n = 2$. Take
$x_0 = (1,-1)$ as the initial value. A C++ function with Rcpp interface is
defined in the file `examples/neldermead.cpp`.

```{.cpp include="examples/neldermead.cpp"}
```

Call the function from R.

```{r}
Rcpp::sourceCpp("examples/neldermead.cpp")
out = neldermead_ex(x0 = c(1, -1))
print(out)
```

## BFGS

Minimization using the BFGS (Broyden-Fletcher-Goldfarb-Shanno)
algorithm [@Nash1990]. Relies on evaluation of $f$ and its gradient
$g(x) = \frac{\partial f(x)}{\partial x}$ but not the Hessian. This function
directly calls [vmmin][vmmin-src], the C function that gets invoked when
using the [optim][Roptim-src] R function with `method = "BFGS"`.

[vmmin-src]: https://github.com/r-devel/r-svn/blob/62dd8b09461d5d1313f4722d9d63abb463a9e42d/src/include/R_ext/Applic.h#L67
[Roptim-src]: https://github.com/r-devel/r-svn/blob/main/src/library/stats/R/optim.R

**Function**

Primary location of source code is the file [inst/include/bfgs.h][bfgs-h].

[bfgs-h]: https://github.com/andrewraim/fntl/blob/main/inst/include/bfgs.h

```{cpp}
bfgs_result bfgs(
	const Rcpp::NumericVector& init,  // <1>
	const dfv& f,                     // <2>
	const vfv& g,                     // <3>
	const bfgs_args& args             // <4>
)

bfgs_result bfgs(
	const Rcpp::NumericVector& init,  // <1>
	const dfv& f,                     // <2>
	const bfgs_args& args             // <4>
)

bfgs_result bfgs(
	const Rcpp::NumericVector& init,  // <1>
	const dfv& f,                     // <2>
	const vfv& g                      // <3>
)

bfgs_result bfgs(
	const Rcpp::NumericVector& init,  // <1>
	const dfv& f                      // <2>
)
```
1. Initial value for optimization variable.
2. Function $f$ to minimize.
3. Gradient function $g(x) = \frac{\partial f(x)}{\partial x}$.
4. Additional arguments.

Forms with the $g$ argument omitted compute the gradient using finite
differences, via the `gradient` method in [@sec-gradient].

**Optional Arguments**

```{cpp}
struct bfgs_args {
	richardson_args deriv_args;    // <1>
	double parscale = 1;           // <2>
	int trace = 0;                 // <3>
	double fnscale = 1;            // <4>
	int maxit = 100;               // <5>
	int report = 10;               // <6>
	double abstol = R_NegInf;      // <7>
	double reltol = mach_eps_2r;   // <8>

	bfgs_args() { };               // <9>
	bfgs_args(SEXP obj);           // <10>
	operator SEXP() const;         // <11>
};
```

1. Arguments for Richardson extrapolated numerical derivatives to compute
   the gradient if $g$ is omitted in the call to `bfgs`. See @sec-deriv.
2. A vector of scaling values for the parameters. (Currently not used).
3. If positive, tracing information on the progress of the optimization is
   produced. There are six levels which give progressively more detail.
4. Scaling factor applied to the value of `f` and `g` during optimization.
5. The maximum number of iterations.
6. The frequency of reports.
7. Absolute tolerance.
8. Relative tolerance.
9. Default constructor.
10. Constructor from an `Rcpp::List`.
11. Conversion operator to `Rcpp::List`.

**Result**

```{cpp}
struct bfgs_result {
	std::vector<double> par;  // <1>
	double value;             // <2>
	bfgs_status status;       // <3>
	int fncount;              // <4>
	int grcount;              // <5>

	operator SEXP() const;    // <6>
};
```
1. The final value of the optimization variable.
2. The value of the function corresponding to `par`.
3. Status code from the optimizer.
4. Number of times the objective function was called.
5. Number of times the gradient function was called.
6. Conversion operator to `Rcpp::List`.

The `SEXP` conversion operator produces the following representation of
`bfgs_result` as an `Rcpp::List`. The fields here directly correspond to
those in `bfgs_result`.

|Name       | Type                  | Description |
|:----------|:----------------------|:------------|
| `par`     | `Rcpp::NumericVector` | Length $n$  |
| `value`   | `Rcpp::NumericVector` | Length 1    |
| `status`  | `Rcpp::IntegerVector` | Length 1    |
| `fncount` | `Rcpp::IntegerVector` | Length 1    |
| `grcount` | `Rcpp::IntegerVector` | Length 1    |

: {tbl-colwidths="[15,30,55]"}

**Status Codes**

```{cpp}
enum class bfgs_status : unsigned int {
	OK = 0L,             // <1>
	NOT_CONVERGED = 1L   // <2>
};
```
1. OK.
2. iteration limit maxit had been reached.

**Example**

Maximize the function $f(x) = \exp(-x^\top x)$. First use the default
numerical gradient, then use an explicitly coded the gradient function
$g(x) = -2 x \exp(-x^\top x)$. A C++ function with Rcpp interface is defined in
the file `examples/bfgs.cpp`.

```{.cpp include="examples/bfgs.cpp"}
```

Call the function from R.

```{r}
Rcpp::sourceCpp("examples/bfgs.cpp")
out = bfgs_ex(x0 = rep(1, 4))
print(out$numerical)
print(out$analytical)
```

## L-BFGS-B {#sec-lbfgs}

Minimization using the L-BFGS-B (Limited memory
Broyden-Fletcher-Goldfarb-Shanno) algorithm [@ByrdEtAl1995]. Relies on
evaluation of $f$ and its gradient $g(x) = \frac{\partial f(x)}{\partial x}$
but not the Hessian. This function directly calls [lbfgsb][lbfgsb-src], the C
function that gets invoked when using the [optim][Roptim-src] R function with
`method = "L-BFGS-B"`.

[lbfgsb-src]: https://github.com/r-devel/r-svn/blob/62dd8b09461d5d1313f4722d9d63abb463a9e42d/src/include/R_ext/Applic.h#L79
[Roptim-src]: https://github.com/r-devel/r-svn/blob/main/src/library/stats/R/optim.R

**Function**

Primary location of source code is the file [inst/include/lbfgsb.h][lbfgsb-h].

[lbfgsb-h]: https://github.com/andrewraim/fntl/blob/main/inst/include/lbfgsb.h

```{cpp}
lbfgsb_result lbfgsb(
	const Rcpp::NumericVector& init,  // <1>
	const dfv& f,                     // <2>
	const vfv& g,                     // <3>
	const lbfgsb_args& args           // <4>
)

lbfgsb_result lbfgsb(
	const Rcpp::NumericVector& init,  // <1>
	const dfv& f,                     // <2>
	const lbfgsb_args& args           // <4>
)

lbfgsb_result lbfgsb(
	const Rcpp::NumericVector& init,  // <1>
	const dfv& f,                     // <2>
	const vfv& g                      // <3>
)

lbfgsb_result lbfgsb(
	const Rcpp::NumericVector& init,  // <1>
	const dfv& f                      // <2>
)
```
1. Initial value for optimization variable.
2. Function $f$ to minimize.
3. Gradient function $g(x) = \frac{\partial f(x)}{\partial x}$.
4. Additional arguments.

Forms with the $g$ argument omitted compute the gradient using finite
differences, via the `gradient` method in [@sec-gradient].

**Optional Arguments**

```{cpp}
struct lbfgsb_args {
	std::vector<double> lower;     // <1>
	std::vector<double> upper;     // <2>
	richardson_args deriv_args;    // <3>
	double parscale = 1;           // <4>
	int trace = 0;                 // <5>
	double fnscale = 1;            // <6>
	int lmm = 5;                   // <7>
	int maxit = 100;               // <8>
	int report = 10;               // <9>
	double factr = 1e7;            // <10>
	double pgtol = 0;              // <11>

	lbfgsb_args() { };             // <12>
	lbfgsb_args(SEXP obj);         // <13>
	operator SEXP() const;         // <14>
};
```
1. A vector of lower bounds. If left unspecified, will be taken to be a vector
   of `-Inf` values.
2. A vector of upper bounds. If left unspecified, will be taken to be a vector
   of `Inf` values.
3. Arguments for Richardson extrapolated numerical derivatives to compute
   the gradient if $g$ is omitted in the call to `lbfgsb`. See @sec-deriv.
4. A vector of scaling values for the parameters. (Currently not used).
5. If positive, tracing information on the progress of the optimization is
   produced. There are six levels which give progressively more detail.
6. Scaling factor applied to the value of `f` and `g` during optimization.
7. Number of BFGS updates retained.
8. The maximum number of iterations.
9. The frequency of reports.
10. Convergence occurs when the reduction in the objective is within this
    factor of the machine tolerance.
11. A tolerance on the projected gradient in the current search direction. The
    check is suppressed when the value is zero.
12. Default constructor.
13. Constructor from an `Rcpp::List`.
14. Conversion operator to `Rcpp::List`.

**Result**

```{cpp}
struct lbfgsb_result {
	std::vector<double> par;  // <1>
	double value;             // <2>
	lbfgsb_status status;     // <3>
	int fncount;              // <4>
	int grcount;              // <5>
	std::string msg;          // <6>

	operator SEXP() const;    // <7>
};
```
1. The final value of the optimization variable.
2. The value of the function corresponding to `par`.
3. Status code from the optimizer.
4. Number of times the objective function was called.
5. Number of times the gradient function was called.
6. String with additional information from the optimizer.
7. Conversion operator to `Rcpp::List`.

The `SEXP` conversion operator produces the following representation of
`lbfgsb_result` as an `Rcpp::List`. The fields here directly correspond to
those in `lbfgsb_result`.

|Name       | Type                  | Description |
|:----------|:----------------------|:------------|
| `par`     | `Rcpp::NumericVector` | Length $n$  |
| `value`   | `Rcpp::NumericVector` | Length 1    |
| `status`  | `Rcpp::IntegerVector` | Length 1    |
| `fncount` | `Rcpp::IntegerVector` | Length 1    |
| `grcount` | `Rcpp::IntegerVector` | Length 1    |
| `message` | `Rcpp::StringVector`  | Length 1    |

: {tbl-colwidths="[15,30,55]"}

**Status Codes**

```{cpp}
enum class lbfgsb_status : unsigned int {
	OK = 0L,             // <1>
	NOT_CONVERGED = 1L,  // <2>
	WARN = 51L,          // <3>
	ERROR = 52L,         // <4>
};
```
1. OK.
2. iteration limit maxit had been reached.
3. algorithm reported a warning.
4. algorithm reported an error.

**Example**

Maximize the function $f(x) = \exp(-x^\top x)$. First use the default
numerical gradient, then use explicitly coded gradient function
$g(x) = -2 x \exp(-x^\top x)$. A C++ function with Rcpp interface is defined in
the file `examples/lbfgsb.cpp`.

```{.cpp include="examples/lbfgsb.cpp"}
```

Call the function from R.

```{r}
Rcpp::sourceCpp("examples/lbfgsb.cpp")
out = lbfgsb_ex(x0 = rep(1, 4))
print(out$numerical)
print(out$analytical)
```

## Conjugate Gradient {#sec-cg}

Minimization using the conjugate gradient algorithm
[@FletcherReeves1964; @Nash1990]. Relies on evaluation of $f$ and its gradient
$g(x) = \frac{\partial f(x)}{\partial x}$ but not the Hessian. This function
directly calls [cgmin][cgmin-src], the C function that gets invoked when
using the [optim][Roptim-src] R function with `method = "CG"`.

[cgmin-src]: https://github.com/r-devel/r-svn/blob/62dd8b09461d5d1313f4722d9d63abb463a9e42d/src/include/R_ext/Applic.h#L75
[Roptim-src]: https://github.com/r-devel/r-svn/blob/main/src/library/stats/R/optim.R

**Function**

Primary location of source code is the file [inst/include/cg.h][cg-h].

[cg-h]: https://github.com/andrewraim/fntl/blob/main/inst/include/cg.h

```{cpp}
cg_result bfgs(
	const Rcpp::NumericVector& init,  // <1>
	const dfv& f,                     // <2>
	const vfv& g,                     // <3>
	const cg_args& args               // <4>
)

cg_result cg(
	const Rcpp::NumericVector& init,  // <1>
	const dfv& f,                     // <2>
	const cg_args& args               // <4>
)

cg_result cg(
	const Rcpp::NumericVector& init,  // <1>
	const dfv& f,                     // <2>
	const vfv& g                      // <3>
)

cg_result cg(
	const Rcpp::NumericVector& init,  // <1>
	const dfv& f                      // <2>
)
```
1. Initial value for optimization variable.
2. Function $f$ to minimize.
3. Gradient function $g(x) = \frac{\partial f(x)}{\partial x}$.
4. Additional arguments.

Forms with the $g$ argument omitted compute the gradient using finite
differences, via the `gradient` method in [@sec-gradient].

**Optional Arguments**

```{cpp}

struct cg_args
{
	richardson_args deriv_args;    // <1>
	double parscale = 1;           // <2>
	double fnscale = 1;            // <3>
	double abstol = R_NegInf;      // <4>
	double reltol = mach_eps_2r;   // <5>
	int type = 1;                  // <6>
	int trace = 0;                 // <7>
	int maxit = 100;               // <8>

	cg_args() { };                 // <9>
	cg_args(SEXP obj);             // <10>
	operator SEXP() const;         // <11>
};
```

1. Arguments for Richardson extrapolated numerical derivatives to compute
   the gradient if $g$ is omitted in the call to `cg`. See @sec-deriv.
2. A vector of scaling values for the parameters. (Currently not used).
3. Scaling factor applied to the value of `f` and `g` during optimization.
4. Absolute tolerance.
5. Relative tolerance.
6.  Type of update: 1 for Fletcher-Reeves, 2 for Polak-Ribiere, and 3 for
    Beale-Sorenson.
7. If positive, tracing information on the progress of the optimization is
   produced. There are six levels which give progressively more detail.
8. The maximum number of iterations.
9. Default constructor.
10. Constructor from an `Rcpp::List`.
11. Conversion operator to `Rcpp::List`.

**Result**

```{cpp}
struct cg_result {
	std::vector<double> par;  // <1>
	double value;             // <2>
	cg_status status;         // <3>
	int fncount;              // <4>
	int grcount;              // <5>

	operator SEXP() const;    // <6>
};
```
1. The final value of the optimization variable.
2. The value of the function corresponding to `par`.
3. Status code from the optimizer.
4. Number of times the objective function was called.
5. Number of times the gradient function was called.
6. Conversion operator to `Rcpp::List`.

The `SEXP` conversion operator produces the following representation of
`cg_result` as an `Rcpp::List`. The fields here directly correspond to
those in `cg_result`.

|Name       | Type                  | Description |
|:----------|:----------------------|:------------|
| `par`     | `Rcpp::NumericVector` | Length $n$  |
| `value`   | `Rcpp::NumericVector` | Length 1    |
| `status`  | `Rcpp::IntegerVector` | Length 1    |
| `fncount` | `Rcpp::IntegerVector` | Length 1    |
| `grcount` | `Rcpp::IntegerVector` | Length 1    |

: {tbl-colwidths="[15,30,55]"}

**Status Codes**

```{cpp}
enum class cg_status : unsigned int {
	OK = 0L,             // <1>
	NOT_CONVERGED = 1L   // <2>
};
```
1. OK.
2. iteration limit maxit had been reached.

**Example**

Maximize the function $f(x) = \exp(-x^\top x)$. First use the default
numerical gradient, then use an explicitly coded the gradient function
$g(x) = -2 x \exp(-x^\top x)$. A C++ function with Rcpp interface is defined in
the file `examples/cg.cpp`.

```{.cpp include="examples/cg.cpp"}
```

Call the function from R.

```{r}
Rcpp::sourceCpp("examples/cg.cpp")
out = cg_ex(x0 = rep(1, 4))
print(out$numerical)
print(out$analytical)
```

## Newton-Type Algorithm for Nonlinear Optimization

Minimization using the Newton-type algorithm underlying the
[nlm][nlm-src] R function. The amounts to calling the C function
[optif9][optif9-src] within the R API. The implementation of `optif9` is based
on @DennisSchnabel1983.

The gradient $g(x) = \frac{\partial f(x)}{\partial x}$ and Hessian 
$h(x) = \frac{\partial^2 f(x)}{\partial x \partial x^\top}$ may be provided
explicitly if available. When the gradient and/or Hessian are not
specified, numerical approximations are used by `optif9`.

[optif9-src]: https://github.com/r-devel/r-svn/blob/62dd8b09461d5d1313f4722d9d63abb463a9e42d/src/include/R_ext/Applic.h#L136
[nlm-src]: https://github.com/r-devel/r-svn/blob/main/src/library/stats/R/nlm.R

**Function**

Primary location of source code is the file [inst/include/nlm.h][nlm-h].

[nlm-h]: https://github.com/andrewraim/fntl/blob/main/inst/include/nlm.h

```{cpp}

nlm_result nlm(
	const Rcpp::NumericVector& init,  // <1>
	const dfv& f,                     // <2>
	const vfv& g,                     // <3>
	const mfv& h,                     // <4>
	const nlm_args& args              // <5>
)

nlm_result nlm(
	const Rcpp::NumericVector& init,  // <1>
	const dfv& f,                     // <2>
	const vfv& g,                     // <3>
	const nlm_args& args              // <5>
)

nlm_result nlm(
	const Rcpp::NumericVector& init,  // <1>
	const dfv& f,                     // <2>
	const nlm_args& args              // <5>
)

nlm_result nlm(
	const Rcpp::NumericVector& init,  // <1>
	const dfv& f,                     // <2>
	const vfv& g,                     // <3>
	const mfv& h                      // <4>
)

nlm_result nlm(
	const Rcpp::NumericVector& init,  // <1>
	const dfv& f,                     // <2>
	const vfv& g                      // <3>
)

nlm_result nlm(
	const Rcpp::NumericVector& init,  // <1>
	const dfv& f                      // <2>
)
```
1. Initial value for optimization variable.
2. Function $f$ to minimize.
3. Gradient of $f$.
4. Hessian of $f$.
5. Additional arguments.

**Optional Arguments**

```{cpp}
struct nlm_args
{
	std::vector<double> typsize;   // <1>
	unsigned int print_level = 0;  // <2>
	double fscale = 1;             // <3>
	double fnscale = 1;            // <4>
	unsigned int ndigit = 12;      // <5>
	double gradtol = 1e-6;         // <6>
	double stepmax = R_PosInf;     // <7>
	double steptol = 1e-6;         // <8>
	int iterlim = 100;             // <9>
	unsigned int method = 1;       // <10>
	double trust_radius = 1.0;     // <11>

	nlm_args() { };                // <12>
	nlm_args(SEXP obj);            // <13>
	operator SEXP() const;         // <14>
};
```
1. An estimate of the size of each parameter at the minimum.
2. Verbosity of messages during optimization:
	- 0: no printing;
	- 1: final details are printed;
	- 2: full tracing is provided.
3. An estimate of the size of f at the minimum.
4. Scaling factor applied to the value of `f`, `g`, and `h` during
   optimization. Taking this to be `-1` changes the optimization to
   maximization.
5. Number of significant digits in the function $f$.
6. Tolerance to terminate algorithm based on the distance of gradient to zero.
7. Maximum allowable scaled step length
8. Tolerance to terminate algorithm based on relative step size of successive
   iterates.
9. The maximum number of iterations.
10. Algorithm used in optimization:
	- 1: line search;
    - 2: double dogleg;
    - 3: more-hebdon.
11. Radius of trust region.
12. Default constructor.
13. Constructor from an `Rcpp::List`.
14. Conversion operator to `Rcpp::List`.

Arguments correspond to those in `nlm` with the following exceptions.

- The arguments `method` and `trust_radius` are provided from within `optif9`
  but not exposed from `nlm`.
  
- An argument `hessian `is provided in `nlm` to compute the value of the
  Hessian via finite differences using the final value of the optimization
  variable. That is not provided here, but may be requested using the Hessian
  function in @sec-hessian.

- The `nlm` function provides an `check.analyticals` argument to check the
  correctness of provided expressions for the gradient and Hessian. This
  argument is not provided here, but a similar check can be done using the
  numerical gradient and Hessian functions in @sec-gradient and @sec-hessian,
  respectively.

See the `nlm` manual page for additional details about other arguments.

If `typsize` is given as the default empty value, it is transformed internally
to a vector of $n$ ones to match the default in `nlm`. Similarly, if `stepmax`
is given as the default infinity value, it is transformed internally to match
the default value in `nlm`.

**Result**

```{cpp}
struct nlm_result
{
	std::vector<double> par;      // <1>
	std::vector<double> grad;     // <2>
	double estimate;              // <3>
	int iterations;               // <4>
	nlm_status status;            // <5>

	operator SEXP() const;        // <6>
};
```
1. The final value of the optimization variable.
2. The final value of the gradient.
3. The value of the function corresponding to `par`.
4. Number of iterations carried out.
5. Status code from the optimizer.
6. Conversion operator to `Rcpp::List`.

The `SEXP` conversion operator produces the following representation of
`nlm_result` as an `Rcpp::List`. The fields here directly correspond to
those in `nlm_result`.

|Name          | Type                  | Description  |
|:-------------|:----------------------|:-------------|
| `par`        | `Rcpp::NumericVector` | Length $n$   |
| `grad`       | `Rcpp::NumericVector` | Length $n$   |
| `estimate`   | `Rcpp::NumericVector` | Length 1     |
| `iterations` | `Rcpp::IntegerVector` | Length 1     |
| `status`     | `Rcpp::IntegerVector` | Length 1     |
| `hessian`    | `Rcpp::IntegerVector` | Length $n^2$ |

: {tbl-colwidths="[15,30,55]"}

**Status Codes**

```{cpp}
enum class nlm_status : unsigned int {
	OK = 0L,                   // <1>
	GRADIENT_WITHIN_TOL = 1L,  // <2>
	ITERATES_WITH_TOL = 2L,    // <3>
	NO_LOWER_STEP = 3L,        // <4>
	ITERATION_MAX = 4L,        // <5>
	STEP_SIZE_EXCEEDED = 5L    // <6>
};

```
1. OK.
2. Relative gradient is within given tolerance.
3. Relative step size of successive iterates is within given tolerance.
4. Last global step failed to locate a point lower than `estimate`.
5. Reached maximum number of iterations.
6. Maximum step size `stepmax` exceeded five consecutive times. 

See the manual page for `nlm` for more detail about these statuses.

**Example**

Maximize the function $f(x) = \exp(-x^\top x)$. The associated gradient and
Hessian functions are $g(x) = -2 f(x) x$ and 
$h(x) = (4 x x^\top - 2 I) f(x)$, respectively. Consider three calls: first
with a numerical gradient and Hessian, then with explicitly coded gradient, and
finally with both explicitly coded gradient and Hessian. A C++ function with
Rcpp interface is defined in the file `examples/nlm.cpp`.

```{.cpp include="examples/nlm.cpp"}
```

Call the function from R.

```{r}
Rcpp::sourceCpp("examples/nlm.cpp")
out = nlm_ex(x0 = rep(1, 4))
nn = c("par", "grad")
print(out$res1[nn])
print(out$res2[nn])
print(out$res3[nn])
```

# Matrix Operations {#sec-matrix}

This section presents several matrix operations based on a lambda function.

## Apply

Apply a function to the elements, rows, or columns of an $m \times n$ matrix.
Suppose $X \in \mathbb{R}^{m \times n}$ is a matrix with rows
$x_{1 \bullet}, \ldots, x_{m \bullet}$ and columns
$x_{\bullet 1}, \ldots, x_{\bullet n}$.

The function `mat_apply` is an elementwise application of
$f : \mathbb{R} \rightarrow \mathbb{R}$ which computes
<!-- -->
$$
\text{\texttt{mat\_apply}}(X, f)
= \begin{bmatrix}
f(x_{11}) & \cdots & f(x_{1n}) \\
\vdots & \ddots & \vdots \\
f(x_{m1}) & \cdots & f(x_{mn}) \\
\end{bmatrix}.
$$
<!-- -->
The function `row_apply` is a rowwise application of
$g : \mathbb{R}^n \rightarrow \mathbb{R}$ which computes
<!-- -->
$$
\text{\texttt{row\_apply}}(X, g)
= \big( g(x_{1 \bullet}), \ldots, g(x_{m \bullet}) \big).
$$
<!-- -->
The function `col_apply` is a columnwise application of
$h : \mathbb{R}^m \rightarrow \mathbb{R}$ which computes
<!-- -->
$$
\text{\texttt{col\_apply}}(X, h)
= \big( h(x_{\bullet 1}), \ldots, h(x_{\bullet n}) \big).
$$
<!-- -->
The above replicate the behavior of following R calls, respectively.

```{r}
#| eval: false
apply(X, c(1,2), f)
apply(X, 1, f)
apply(X, 2, f)
```

**Functions**

Primary location of source code is the file [inst/include/apply.h][apply-h].

[apply-h]: https://github.com/andrewraim/fntl/blob/main/inst/include/apply.h

```{cpp}
template <typename T, int RTYPE>
Rcpp::Vector<RTYPE> row_apply(
	const Rcpp::Matrix<RTYPE>& X,							// <1>
	const std::function<T(const Rcpp::Vector<RTYPE>&)>& f   // <2>
)

template <typename T, int RTYPE>
Rcpp::Vector<RTYPE> col_apply(
	const Rcpp::Matrix<RTYPE>& X,							// <1>
	const std::function<T(const Rcpp::Vector<RTYPE>&)>& f   // <2>
)

template <typename T, int RTYPE>
Rcpp::Matrix<RTYPE> mat_apply(
	const Rcpp::Matrix<RTYPE>& X,							// <1>
	const std::function<T(T)>& f                            // <2>
)
```
1. An Rcpp matrix object.
2. Function $f$ to apply.

The functions `mat_apply`, `row_apply`, and `col_apply` correspond to
elementwise, rowwise, and columnwise apply.

Rcpp matrix objects may be of type `NumericMatrix`, `IntegerMatrix`,
or one of the others defined in the [Rcpp API][rcpp-matrices-src]. The template
argument `RTYPE` represents the type of data stored in in the matrix, taking
on value `REALSXP` for `NumericMatrix`, `INTSXP` for `IntegerMatrix`, etc. The
template argument `T` represents an underlying C++ variable type such as
`double`, `int`, etc.

The domain and range of function `f` should match the class of `x` and type of
apply operation. As an example, suppose `X` is an object of type `NumericMatrix`.

- The domain of `row_apply` should be of type `const NumericVector&` and
  the range should be of type `double`.
- The domain of `col_apply` should be of type `const NumericVector&` and
  the range should be of type `double`.
- The domain and range of `mat_apply` should be of type `double`.

[rcpp-matrices-src]: https://dirk.eddelbuettel.com/code/rcpp/html/instantiation_8h.html

**Example**

Compute the square of each element of a matrix, then its rowwise sums, then its
columnwise sums. A C++ function with Rcpp interface is defined in the file
`examples/apply.cpp`.

```{.cpp include="examples/apply.cpp"}
```

Call the function from R.

```{r}
Rcpp::sourceCpp("examples/apply.cpp")
X = matrix(1:12, 4, 3)
out = apply_ex(X)
print(out)
```

## Outer

Construct a matrix from a real-valued function of two arguments. Or carry out a
matrix multiplication without explicitly constructing the matrix.

Suppose $f(x,x') : \mathbb{R}^d \times \mathbb{R}^d \rightarrow \mathbb{R}$ and
$X \in \mathbb{R}^{n \times d}$ is a matrix with rows $x_1, \ldots, x_n$. Also
let $a = [a_1 \cdots a_n]^\top$ be a fixed vector. The `outer` operation
computes the $n \times n$ symmetric matrix
<!-- -->
$$
\text{\texttt outer}(X, f) =
\begin{bmatrix}
f(x_1, x_1) & \cdots & f(x_1, x_n) \\
\vdots & \ddots & \vdots \\
f(x_n, x_1) & \cdots & f(x_n, x_n) \\
\end{bmatrix}
$$
<!-- -->
and the `outer_matvec` operation computes the $n$-dimensional vector
<!-- -->
$$
\text{\texttt outer\_matvec}(X, f, a) =
\begin{bmatrix}
f(x_1, x_1) & \cdots & f(x_1, x_n) \\
\vdots & \ddots & \vdots \\
f(x_n, x_1) & \cdots & f(x_n, x_n) \\
\end{bmatrix}
\begin{bmatrix}
a_1 \\
\vdots \\
a_n \\
\end{bmatrix}.
$$

Now suppose $f(x,y) : \mathbb{R}^{d_1} \times \mathbb{R}^{d_2} \rightarrow \mathbb{R}$,
$X \in \mathbb{R}^{m \times d_1}$ is a matrix with rows $x_1, \ldots, x_m$,
$Y \in \mathbb{R}^{n \times d_2}$ is a matrix with rows $y_1, \ldots, y_n$,
and $a = [a_1 \cdots a_n]^\top$ is a fixed vector. The `outer` operation
computes the $m \times n$ matrix
<!-- -->
$$
\text{\texttt outer}(X, Y, f) =
\begin{bmatrix}
f(x_1, y_1) & \cdots & f(x_1, y_n) \\
\vdots & \ddots & \vdots \\
f(x_m, y_1) & \cdots & f(x_m, y_n) \\
\end{bmatrix}
$$
<!-- -->
and the `outer_matvec` operation computes the $m$-dimensional vector
<!-- -->
$$
\text{\texttt outer\_matvec}(X, Y, f, a) =
\begin{bmatrix}
f(x_1, y_1) & \cdots & f(x_1, y_n) \\
\vdots & \ddots & \vdots \\
f(x_m, y_1) & \cdots & f(x_m, y_n) \\
\end{bmatrix}
\begin{bmatrix}
a_1 \\
\vdots \\
a_n \\
\end{bmatrix}.
$$

The `outer` functions above replicate the behavior of the `outer` function in R.

**Functions**

Primary location of source code is the file [inst/include/outer.h][outer-h].

[outer-h]: https://github.com/andrewraim/fntl/blob/main/inst/include/outer.h

```{cpp}
Rcpp::NumericMatrix outer(
	const Rcpp::NumericMatrix& X,  // <1>
	const dfvv& f                  // <3>
)

Rcpp::NumericMatrix outer(
	const Rcpp::NumericMatrix& X,  // <1>
	const Rcpp::NumericMatrix& Y,  // <2>
	const dfvv& f                  // <3>
)

Rcpp::NumericVector outer_matvec(
	const Rcpp::NumericMatrix& X,  // <1>
	const dfvv& f,                 // <3>
	const Rcpp::NumericVector& a   // <4>
)

Rcpp::NumericVector outer_matvec(
	const Rcpp::NumericMatrix& X,  // <1>
	const Rcpp::NumericMatrix& Y,  // <2>
	const dfvv& f,                 // <3>
	const Rcpp::NumericVector& a   // <4>
)
```
1. An Rcpp matrix object of dimension $m \times d$.
2. An Rcpp matrix object of dimension $n \times d$.
3. Function $f$ to apply.
4. An Rcpp vector object of dimension $n$.

**Example**

Compute the distance between pairs of rows of $X$, then compute the distance
between each pairs of rows taken from $X$ and $Y$. Then multiply the respective
matrices by $a = [1, \ldots, 1]$. A C++ function with Rcpp interface is defined
in the file `examples/outer.cpp`.

```{.cpp include="examples/outer.cpp"}
```

Call the function from R.

```{r}
Rcpp::sourceCpp("examples/outer.cpp")
m = 5; n = 3; d = 2
X = matrix(rnorm(10), m, d)
Y = matrix(rnorm(6), n, d)
a = rep(1, m)
b = rep(1, n)
out = outer_ex(X, Y, a, b)
print(out)
```

## Matrix-Free Linear Solve

Solve a linear system $A x = b$ for symmetric positive definite
$A \in \mathbb{R}^{n \times n}$ [@NocedalWright2006]. Here the operation $Ax$
is specified through a function $\ell(x) = Ax$. This can be used to avoid
explicit storage of large matrices when $A$ is sparse. The solution $x^*$ is
found by minimizing the quadratic function
<!-- -->
$$
f(x) = \frac{1}{2} x^\top \ell(x) - b^\top x,
$$
<!-- -->
so that $f'(x^*) = 0 \iff A x^* = b$. Minimization is carried out using the
conjugate gradient method in @sec-cg.

**Functions**

Primary location of source code is the file [inst/include/solve_cg.h][solve_cg-h].

[solve_cg-h]: https://github.com/andrewraim/fntl/blob/main/inst/include/solve_cg.h

```{cpp}
cg_result solve_cg(
	const vfv& l,                     // <1>
	const Rcpp::NumericVector& b,     // <2>
	const Rcpp::NumericVector& init,  // <3>
	const cg_args& args               // <4>
)

cg_result solve_cg(
	const vfv& l,                     // <1>
	const Rcpp::NumericVector& b,     // <2>
	const Rcpp::NumericVector& init   // <3>
)

cg_result solve_cg(
	const vfv& l,                     // <1>
	const Rcpp::NumericVector& b      // <2>
)
```
1. The function $\ell : \mathbb{R}^n \rightarrow \mathbb{R}^n$.
2. The vector $b \in \mathbb{R}^{n}$.
3. Initial value for $x$.
4. Optional arguments to CG method.

The routine checks that `l(init)` is an $n$-dimensional vector, but there is no
check that the operation $\ell$ represents a symmetric positive definite
matrix.

See @sec-cg for details on `cg_args` and `cg_result`.

**Example**

Solve the equation $Ax = b$ with $n \times n$ matrix $A$ having value 2 
on the main diagonal and value 1 on the upper and lower diagonals; let
$b = [1, \ldots, 1]$. The initial value for the solution is taken to be the
default $x = 0$. A C++ function with Rcpp interface is defined in the file
`examples/solve-cg.cpp`.

```{.cpp include="examples/solve-cg.cpp"}
```

Call the function from R.

```{r}
Rcpp::sourceCpp("examples/solve-cg.cpp")
b = rep(1, 10)
out = solve_cg_ex(b)
print(out)
```

Compare the above to solving dense the system as follows.

```{r}
A = matrix(0, 10, 10)
diag(A) = 2
A[cbind(1:9, 2:10)] = 1
A[cbind(2:10, 1:9)] = 1
solve(A, b)
```


## Which

Identify the indices of a matrix which satisfy a given indicator function.
Specifically, let $X \in \mathbb{S}^{m \times n}$ be a matrix whose elements
are in the domain $\mathbb{S}$ which may be doubles, integers, or another
Rcpp `Matrix` type. Let $f : \mathbb{S} \rightarrow \{0, 1\}$ be an indicator
function and suppose $k$ of the $mn$ elements in $X$ satisfy the indicator.
The `which` operation produces a $k \times 2$ matrix
<!-- -->
$$
\text{which}(X, f) =
\begin{bmatrix}
i_1 & j_1 \\
\vdots & \vdots \\
i_k & j_k \\
\end{bmatrix},
$$
<!-- -->
where each pair $(i_\ell, j_\ell)$ are coordinates of an element in $X$ that
satisfies $f$. Indices $i_\ell$ and $j_\ell$ represent the row and column
index, respectively. Indices are *zero-based* by default as they are primarily
intended to be used in C++ code.

An equivalent R operation is the following. Note that indices in R are
*one-based*.

```{r}
#| eval: false
which(f(X), arr.ind = TRUE)
```

**Functions**

Primary location of source code is the file [inst/include/which.h][which-h].

[which-h]: https://github.com/andrewraim/fntl/blob/main/inst/include/which.h

```{cpp}
template <typename T, int RTYPE>
Rcpp::IntegerMatrix which(
	const Rcpp::Matrix<RTYPE>& X,      // <1>
	const std::function<bool(T)>& f),  // <2>
	bool one_based = false             // <3>
)
```
1. An Rcpp matrix object.
2. Function $f$ to apply.
3. If `one_based = false`, zero-based indices are produced which are suitable
   for use with C++ code. If `true`, one-based indices are produced.

**Example**

Construct a matrix and identify the elements between $0$ and $0.5$. A C++
function with Rcpp interface is defined in the file `examples/which.cpp`.

```{.cpp include="examples/which.cpp"}
```

Call the function from R.

```{r}
Rcpp::sourceCpp("examples/which.cpp")
x = runif(10, -1, 1)
X = matrix(x, 2, 5)
out = which_ex(X)
print(X)
print(out)
```

Here is the result in R for comparison.

```{r}
f = function(x) { x > 0 & x < 0.5 }
which(f(X), arr.ind = TRUE) - 1
```

# Truncated Distributions

Functions to support a univariate distribution truncated to the interval
$(a , b]$. Operations include variate generation, the density function, the
quantile function, and the cumulative distribution function (CDF). Suppose
$f$, $F$, and $F^{-}$ are the density, CDF, and quantile
function, respectively, of an untruncated distribution.

The density $g$ of the truncated distribution is
<!-- -->
$$
g(x) = \frac{f(x)}{F(b) - F(a)} \textrm{I}(a < x \leq b).
$$
<!-- -->
The CDF $G$ of the truncated distribution is
<!-- -->
$$
G(x) = \frac{F(x) - F(a)}{F(b) - F(a)} \textrm{I}(a < x \leq b).
$$
<!-- -->
The quantile function $G^{-}$ of the truncated distribution is
<!-- -->
$$
G^{-}(\phi) = F^{-}\Big( \{ F(b) - F(a) \} \phi + F(a) \Big).
$$
<!-- -->
Draws are generated from the truncated distribution via the inverse CDF method
as $X = G^{-}(U)$ where $U \sim \text{Uniform}(0, 1)$. Some precautions are
taken computationally to avoid loss of precision. Computations are kept on the
log-scale internally accommodate very small probabilities. Probabilities based
on the CDF are considered using both the form $\Prob(X \leq x)$ and its
complement to aid with small probabilities far into the tails.

Special attention is needed for lower limits with a discrete distribution.
For example, consider the $\text{Poisson}(\lambda)$ distribution truncated to
$[1,10]$. Taking $(a, b]$ with $a = 1$ and $b = 10$ excludes the lower limit 1,
which is not desired. Instead, the correct truncation is achieved with
$a = a_0 - \epsilon$, where $a_0$ is the lower limit of interest (e.g., 1) and
$\epsilon$ is a small positive number (e.g., $10^{-5}$).

This functions in this section were originally adapted from the `rtruncated`
function in the `LearnBayes` package [@LearnBayes].

**Typedefs**

The following typedefs are utilized specifically in this section.

```{cpp}
typedef std::function<double(double,bool)> density;       // <1>
typedef std::function<double(double,bool,bool)> cdf;      // <2>
typedef std::function<double(double,bool,bool)> quantile; // <3>
```
1. A density whose arguments are: (1) the argument `x` for which to evaluate
   the density and (2) a boolean `log` that determines whether the value is
   returned on the log-scale.
2. A CDF whose arguments are: (1) the argument `x` for which to evaluate
   the CDF, (2) a boolean `lower` that determines whether lower-tail
   probability $\Prob(X \leq x)$ is to be computed, and (3) a boolean `log`
   that determines whether the value is returned on the log-scale.
3. A quantile function whose arguments are: (1) the argument `p` for which to
   compute quantiles, (2) a boolean `lower` that determines whether `p` is
   interpreted as `p` (if `true`) or `1 - p` (if `false`), and (3) a boolean
   `log` that determines `p` is assumed to be on the log-scale (if `true`).

## Density

**Functions**

Primary location of source code is the file [inst/include/trunc.h][trunc-h].
The following functions operate on scalar arguments.

[trunc-h]: https://github.com/andrewraim/fntl/blob/main/inst/include/trunc.h

```{cpp}
double d_trunc(
	double x,                      // <1>
	double lo,                     // <2>
	double hi,                     // <3>
	const density& f,              // <4>
	const cdf& F,                  // <5>
	bool log = false               // <6>
)

Rcpp::NumericVector d_trunc(
	const Rcpp::NumericVector& x,  // <1>
	const Rcpp::NumericVector& lo, // <2>
	const Rcpp::NumericVector& hi, // <3>
	const density& f,              // <4>
	const cdf& F,                  // <5>
	bool log = false               // <6>
)
```
1. Argument of density and CDF.
2. Lower limit.
3. Upper limit.
4. Function that computes density of untruncated distribution.
5. Function that computes CDF of untruncated distribution.
6. Boolean; if `true`, probabilities $p$ are given as $\log(p)$.

When `x`, `lo` and `hi` are specified as vectors, they must be the same length.

**Example**

Compute density of the $\text{Beta}(2,5)$ distribution truncated to the
interval $(0.5, 0.95]$. A C++ function with Rcpp interface is defined in the
file `examples/d_trunc.cpp`.

```{.cpp include="examples/d_trunc.cpp"}
```

Call the function from R.

```{r}
Rcpp::sourceCpp("examples/d_trunc.cpp")
x = seq(0, 1, length.out = 30)
d_trunc_ex(x, shape1 = 2, shape2 = 5, lo = 0.5, hi = 0.95)
```

## CDF

**Functions**

Primary location of source code is the file [inst/include/trunc.h][trunc-h].
The following functions operate on scalar arguments.

[trunc-h]: https://github.com/andrewraim/fntl/blob/main/inst/include/trunc.h

```{cpp}
double p_trunc(
	double x,              // <1>
	double lo,             // <2>
	double hi,             // <3>
	const cdf& F,          // <4>
	bool lower = true,     // <5>
	bool log = false       // <6>
)

Rcpp::NumericVector p_trunc(
	const Rcpp::NumericVector& x,  // <1>
	const Rcpp::NumericVector& lo, // <2>
	const Rcpp::NumericVector& hi, // <3>
	const cdf& F,                  // <4>
	bool lower = true,             // <5>
	bool log = false               // <6>
)
```
1. Argument of CDF.
2. Lower limit.
3. Upper limit.
4. Function that specifies CDF of untruncated distribution.
5. Boolean; if `true`, probabilities are $\Prob(X \leq x)$, otherwise
   $\Prob(X > x)$.
6. Boolean; if `true`, probabilities $p$ are given as $\log(p)$.

When `x`, `lo` and `hi` are specified as vectors, they must be the same length.

**Example**

Compute CDF of the $\text{Beta}(2,5)$ distribution truncated to the
interval $(0.5, 0.95]$. A C++ function with Rcpp interface is defined in the
file `examples/p_trunc.cpp`.

```{.cpp include="examples/p_trunc.cpp"}
```

Call the function from R.

```{r}
Rcpp::sourceCpp("examples/p_trunc.cpp")
x = seq(0, 1, length.out = 30)
p_trunc_ex(x, shape1 = 2, shape2 = 5, lo = 0.5, hi = 0.95)
```


## Quantile

**Functions**

Primary location of source code is the file [inst/include/trunc.h][trunc-h].
The following functions operate on scalar arguments.

[trunc-h]: https://github.com/andrewraim/fntl/blob/main/inst/include/trunc.h

```{cpp}
double q_trunc(
	double p,              // <1>
	double lo,             // <2>
	double hi,             // <3>
	const cdf& F,          // <4>
	const quantile& Finv,  // <5>
	bool lower = true,     // <6>
	bool log = false       // <7>
)

Rcpp::NumericVector q_trunc(
	const Rcpp::NumericVector& p,  // <1>
	const Rcpp::NumericVector& lo, // <2>
	const Rcpp::NumericVector& hi, // <3>
	const cdf& F,                  // <4>
	const quantile& Finv,          // <5>
	bool lower = true,             // <6>
	bool log = false               // <7>
)
```
1. Probability argument.
2. Lower limit.
3. Upper limit.
4. Function that specifies CDF of untruncated distribution.
5. Function that computes quantiles of untruncated distribution.
6. Boolean; if `true`, probabilities are $\Prob(X \leq x)$, otherwise
   $\Prob(X > x)$.
7. Boolean; if `true`, probabilities $p$ are given as $\log(p)$.

When `x`, `lo` and `hi` are specified as vectors, they must be the same length.

**Example**

Compute CDF of the $\text{Beta}(2,5)$ distribution truncated to the
interval $(0.5, 0.95]$. A C++ function with Rcpp interface is defined in the
file `examples/q_trunc.cpp`.

```{.cpp include="examples/q_trunc.cpp"}
```

Call the function from R.

```{r}
Rcpp::sourceCpp("examples/q_trunc.cpp")
p = seq(0, 1, length.out = 30)
q_trunc_ex(p, shape1 = 2, shape2 = 5, lo = 0.5, hi = 0.95)
```

## Variate Generation

**Functions**

Primary location of source code is the file [inst/include/trunc.h][trunc-h].
The following functions operate on scalar arguments.

[trunc-h]: https://github.com/andrewraim/fntl/blob/main/inst/include/trunc.h

```{cpp}
double r_trunc(
	double lo,             // <2>
	double hi,             // <3>
	const cdf& F,          // <4>
	const quantile& Finv   // <5>
)

Rcpp::NumericVector r_trunc(
	unsigned int n,                // <1>
	const Rcpp::NumericVector& lo, // <2>
	const Rcpp::NumericVector& hi, // <3>
	const cdf& F,                  // <4>
	const quantile& Finv           // <5>
)

```
1. Number of desired draws.
2. Lower limit.
3. Upper limit.
4. Function that specifies CDF of untruncated distribution.
5. Function that computes quantiles of untruncated distribution.

When `lo` and `hi` are specified as vectors, they must be the same length.

The following are analogous functions for some of the arguments are vectors,
for convenience.

**Example**

Draw from the $\text{Beta}(2,5)$ distribution truncated to the interval
$(0.5, 0.95]$. A C++ function with Rcpp interface is defined in the file
`examples/rtrunc.cpp`.

```{.cpp include="examples/r_trunc.cpp"}
```

Call the function from R.

```{r}
Rcpp::sourceCpp("examples/r_trunc.cpp")
r_trunc_ex(n = 20, shape1 = 2, shape2 = 5, lo = 0.5, hi = 0.95)
```

# References {-}

::: {#refs}
:::

---
title: "`fntl`: Numerical Tools for Rcpp and Lambda Functions"
author:
  - name: Andrew M. Raim
    email: andrew.raim@gmail.com
affiliations:
  - name: U.S. Census Bureau
    department: Center for Statistical Research and Methodology
    address: 4600 Silver Hill Road
    city: Washington DC
    country: U.S.A.
format:
  pdf:
    fontsize: 12pt
    indent: false
    toc: true
    number-sections: true
    colorlinks: true
    link-citations: true
    prompt: false
    keep-tex: true
    latex-clean: false
    include-in-header:
      text: |
        \usepackage{common}
vignette: >
  %\VignetteIndexEntry{fntl}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{quarto::pdf}
bibliography: references.bib
editor_options: 
  chunk_output_type: console
abstract: |
  R programmers can combine R and C++ to effectively navigate
  a variety of computing tasks: R excels as a language for interactive tasks
  such as data wrangling, analysis, and plotting; on the other hand, C++ can be
  used to efficiently carry out intensive computations. Rcpp and related
  tools have greatly simplified interoperability between the two languages.
  However, numerical computing tasks that involve functions as arguments, such
  as integration, root-finding, and optimization, which are routinely carried
  out in R, are not as straightforward in C++ within the Rcpp framework.
  The `fntl` package seeks to improve this by providing a straightforward
  API for numerical tools where functional arguments are specified as C++
  lambda functions. Like functions in R, lambda functions can be defined in
  the course of a C++ program, "capturing" variables in the surrounding
  environment if desired, and be passed as arguments to other functions. This
  enables the development of R-like programs in C++, which may be appealing to
  Rcpp users compared to existing alternatives in the extended Rcpp family of
  packages. Because the overhead to evaluate a lambda function is low compared
  to that of evaluating an R function from C++, good performance is also
  possible in this paradigm.
thanks: |
  Document was compiled 2024\-11\-22 10:39:10 EST and
  corresponds to `fntl` version 0.1.2.
  **Contact**`:` <andrew.raim@gmail.com>, Center for Statistical Research &
  Methodology, U.S. Census Bureau, Washington, DC, 20233, U.S.A.
geometry:
  - left=0.75in
  - right=0.75in
  - top=0.75in
  - bottom=1.00in
code-block-bg: "#FAFAFA"
code-block-border-left: "#008CFF"
callout-icon: false
filters:
  - include-code-files
execute:
  eval: false
---







::: {.callout-important title="$\dagger$Important"}
Users should read @sec-inferring-return carefully. It describes a
seemingly innocuous way of coding lambdas that can lead to crashing R. This can
be difficult to troubleshoot but is easily avoided once we are aware.
:::

# Disclaimer and Acknowledgments {-}

This document is released to inform interested parties of ongoing research and
to encourage discussion of work in progress. Any views expressed are those of
the author and not those of the U.S. Census Bureau.

Thanks to Drs. Joseph Kang and Tommy Wright (U.S. Census Bureau) for reviewing
a draft of this document.

Although there are no guarantees of correctness of the `fntl` package,
reasonable efforts will be made to address shortcomings. Comments, questions,
corrections, and possible improvements can be communicated through the
project's Github repository (<https://github.com/andrewraim/fntl>).

# Introduction {#sec-intro}

Users of R [@Rcore2024] can implement intensive computations in compiled C++
and engage in interactive computation through the interpreted R language. The
Rcpp package [@Rcpp2024] simplifies the process by automating interoperation
between the two languages and providing an intuitive API in C++. However,
numerical tools such as integration, root-finding, and optimization, which are
routinely used in R, are not as readily available or easy to use when carrying
out lower-level programming in C++. Numerical tools in R may be invoked
from C++, but this incurs an overhead which can offset gains in performance
which motivate the use of C++. The purpose of the `fntl` package is to
facilitate access to such numerical tools in C++ using lambda functions. Here,
R-like code can be achieved in C++ where functions are defined on the fly and
passed as arguments to other routines. The name "fntl" is a portmanteau of
"**f**unctions", "**n**umerical **t**ools", and "**l**ambdas".

@IhakaGentleman1996 describe programming with functions as one of the main
motivations in developing R. Namely, functions may be defined dynamically in
the course of a program and can make use of variables from the surrounding
environment. As an example, consider the following R snippet to compute the
maximum likelihood estimate (MLE) of $X_1, \ldots, X_n \sim \text{N}(\mu, 1)$.




::: {.cell}

```{.r .cell-code}
mu_true = 0
x = rnorm(n = 200, mean = mu_true, sd = 1)
loglik = function(mu) { sum(dnorm(x, mu, 1, log = TRUE)) }
optimize(loglik, lower = -100, upper = 100, maximum = TRUE)
```
:::




Here the generated sample `x` is "baked in" to the definition of `loglik` so
that `loglik` can be regarded as a univariate function of `mu` to be used with
`optimize`. The ability to construct functions such as `loglik` in the course
of a program and pass them to other functions, as any other variable, can be
tremendously convenient. @Wickham2019 discusses some design patterns which are
possible using functions in R. However, such conveniences come at a cost, as
performance can be inefficient in both speed and memory management.

Performance limitations in R can sometimes be worked around; for example, loops
and apply statements are typically slow because they are executed by an interpreter,
but matrix operations are typically fast because they make use of calls
to BLAS, LAPACK, and other compiled matrix libraries. Refactoring loops into
matrix operations can yield a significant performance improvement when it is
feasible. Compiled languages such as FORTRAN and C/C++ are often used for high
performance implementation of numerical methods; however, they are not
well-suited for interactive usages such as data wrangling, data analysis,
and graphics like R, MATLAB, or Python. Julia [@BezansonEtAl2017] has emerged
over the past decade as a language for scientific computing which is both
suitable to be used interactively and compiles into efficient executable code.

R supports integration with FORTRAN and C/C++ so that high performance code
can be used interactively from the R interpreter; therefore, it remains a
viable alternative to languages like Julia. The `Rcpp` package largely
automates integration with C/C++ code so that users can focus their efforts on
solving the problem at hand. For a function call from R to C++, this automation
includes unpacking the arguments from `SEXP` objects into C++ objects and later
packing the result into an `SEXP` object to be returned. Overhead from repeated
packing and unpacking can hinder performance; on the other hand,
performance while submerged in C++ is free of this overhead. The Rcpp API
provides general C++ classes for vectors, matrices, lists, and other routinely
used objects from R. Additional Rcpp extension packages have been developed to
provide APIs with more application-specific classes; for example, RcppArmadillo
[@EddelbuettelSanderson2014] exposes the API from Armadillo for numerical
matrix operations.

A performance penalty is also suffered when defining a function in R and using
it from C++ with Rcpp. Calling the function from C++ requires going
back through the R runtime environment in addition to packing arguments into
`SEXP` objects and unpacking return values from `SEXP` objects. Doing so
repeatedly accumulates the penalty, and can negate performance benefits of
using C++. Instead, we will consider utilizing lambda functions in C++, which
were introduced in the C++11 specification. There are alternative methods of
defining and passing functions as objects - such as through function pointers
and functor classes - but lambda functions seem best aligned with the R style
discussed by @IhakaGentleman1996. Traditional functions and classes used in the
function pointer and functor approaches, respectively, are declared in their
own blocks prior to use. Another major difference lies in auxiliary data - such
as sample data in a loglikelihood function - which are not considered to be
primary arguments. With function pointers, auxiliary data are passed as
additional arguments that each caller must furnish. Functors are classes which
expose the function of interest and encapsulate auxiliary data using member
variables.

To demonstrate lambdas, consider the following C++ code snippet.




::: {.cell}

```{.cpp .cell-code}
auto f = [](double x, double y) -> double { return x*y; };
```
:::




The function `f` may be invoked as usual with an expression such as `f(x, y)`.
Here, `f` is a lambda with `double` arguments `x` and `y` which returns the
product `x*y` as another `double`. The `auto` keyword instructs the compiler to
infer the type of `f` from the  return type of the right-hand side of the
equality operator. In the previous example, the return type of the lambda can
also be inferred, and we may rewrite it as follows.




::: {.cell}

```{.cpp .cell-code}
auto f = [](double x, double y) { return x*y; };
```
:::




Like an R function, variables in scope of the lambda may be "baked in" to it.
Here is an example from our MLE setting.




::: {.cell}

```{.cpp .cell-code}
Rcpp::NumericVector x = Rcpp::rnorm(200);
auto loglik = [&](double mu) {
    double out = Rcpp::sum(Rcpp::dnorm(x, mu, 1, true));
	return out;
};
```
:::




The function `loglik` takes a `double` argument `mu` and returns a `double`.
The randomly generated data `x` is included in the function definition; C++
documentation refers to `x` as a *capture* of the lambda `loglik`. The
bracketed expression `[&]` specifies that captures of `loglik` should be by
reference; alternatively, `[=]` would specify that captures should be done by
value so that copies of the original variable are used. It is also possible to
specify by-reference or by-value for each captured variable.

To be able to pass a lambda with captures as an argument to other
functions, we wrap it in a [Standard Template Library][stl-url] (STL) function
as follows.




::: {.cell}

```{.cpp .cell-code}
std::function<double(double)> loglik = [&](double mu) {
    double out = Rcpp::sum(Rcpp::dnorm(x, mu, 1, true));
	return out;
};
```
:::




Note that the template argument of `std::function<double(double)>` describes
the domain and range of the function: the `double` in parentheses is the
argument and the `double` outside indicates the return type.

Lambda functions can be used directly with the Rcpp API in some cases. For
example, an Rcpp Gallery [post][RcppLambdaPost] demonstrates the use
of the STL `transform` function to apply a lambda to each element of a vector.
The `fntl` package avoids duplicating this existing functionality.

Numerical tools implemented in `fntl` are covered in other Rcpp-related
packages to some extent. The `RcppNumerical` package [@RcppNumerical2024]
implements numerical integration - both univariate and multivariate - and
optimization using a limited memory Broyden, Fletcher, Goldfarb and Shanno
method. This is accomplished by providing an Rcpp interface to  several open
source libraries. The `roptim` package [@Roptim2022] provides an Rcpp interface
to the R API to call individual optimization methods underlying the `optim`
function. Function arguments in both `RcppNumerical` and `roptim` are specified
via C++ functors. This vignette of `roptim` provides a discussion on calling
the R API which was helpful in the development of `fntl`. A number of numerical
utilities in the GSL [@GalassiEtAl2009] and [Boost](https://www.boost.org)
libraries are provided by the `RcppGSL` [@RcppGSL2024] and `BH` [@BH2024]
packages, respectively. The `RcppGSL` package requires installation of the
underlying GSL library to build the source package. The author of the present
document finds the interfaces of both GSL and Boost to be somewhat daunting,
which has motivated a search for alternatives.

The `fntl` package is guided by several design principles. The interface is
intended to be simple and familiar to R users. External dependencies beyond
the R platform itself and the Rcpp package are avoided; this is to support use
in locked-down computing environments where adding or upgrading system libraries
may be nontrivial. For numerical methods, we first prefer to make use of
functions exposed as entry points in the R API [@WritingRExtensions, Section 6].
In cases where a desired R method is not exposed in the API, we roll our own
implementation. Such methods in `fntl` may not be exactly the same as those in
R, but are intended to be comparable. 

@sec-overview presents an overview of the `fntl` API. Subsequent sections
present the API in detail by topic: numerical integration in @sec-integrate,
numerical differentiation in @sec-diff, root-finding in @sec-findroot,
univariate optimization in @sec-optimize, multivariate optimization in
@sec-optim, and matrix operations in @sec-matrix. The C++ examples in each
section can be obtained as standalone files in the
[vignettes/examples][examples-url] folder of the `fntl` source repository.

[stl-url]: https://www.cppreference.com/Cpp_STL_ReferenceManual.pdf
[RcppLambdaPost]: https://gallery.rcpp.org/articles/simple-lambda-func-c++11
[examples-url]: https://github.com/andrewraim/fntl/tree/main/vignettes/examples


# Overview {#sec-overview}

## A First Example {#sec-first-example}

We will first consider a brief example to illustrate use of the `fntl` package.
If attempting to follow along, ensure that you have successfully installed the
`fntl` package on your system. The following C++ code, given in the file
`examples/first.cpp`, computes the integral
$B(a,b) = \int_0^1 x^{a-1} (1-x)^{b-1} dx$.

```{.cpp include="examples/first.cpp"}
```

There are a number of points to note in this example.

1. The first two lines - the `depends` attribute and the include of `fntl.h` -
   are needed to access the `fntl` library from C++.

2. Functions and other objects in `fntl` are accessed through the `fntl`
   namespace (e.g., `fntl::integrate`).

3. The call to `integrate` follows a similar pattern as many functions in
   `fntl`. Primary arguments include the function `f` and the bounds of the
   integral, while optional arguments such as the number of subdivisions are
   passed in a struct of type `integrate_args`. The result of `integrate` is
   an `integrate_result` struct containing the integral approximation `value`,
   a return code `status`, and several other outputs from the operation.

4. The `integrate_args` struct uses default values for any unspecified
   arguments. Users often will not want to specify values such as tolerances
   or verbosities. Similar argument names and default values as the
   corresponding R function are used when possible.

5. The integrand `f` is defined as a `dfd`; this is a typedef in `fntl`
   which is a shorthand for `std::function<double(double)>`.

6. The last line shows the `integrate_result` being wrapped into an Rcpp `List`
   so that it becomes an R list when returned to R.

7. The `status` code is a value from the `integrate_status` enum class. Here it
   is converted to an integer using the function `to_underlying`.
   
8. An R function `first_ex` is generated by specifying an `Rcpp::export`
   annotation. This is done for most functions in this document to facilitate
   demonstrations.

Let us invoke the `first_ex` function through R. 




::: {.cell}

```{.r .cell-code}
Rcpp::sourceCpp("examples/first.cpp")
out = first_ex(2, 3)
print(out$value)
```
:::




## Arguments

There are a number of `args` structs which represent optional arguments to
functions. `integrate_args` is one example; other `args` types have a prefix
matching their corresponding function. Each of these may
be instantiated from an Rcpp `List` or exported to an Rcpp `List` using the
`as` and `wrap` mechanisms, respectively.^[Details on implementing these
mechanisms may be found in the [Rcpp Extending][rcpp-extend] vignette.]

[rcpp-extend]: https://cran.r-project.org/web/packages/Rcpp/vignettes/Rcpp-extending.pdf




::: {.cell}

```{.cpp .cell-code}
// Create args and export to a List
fntl::integrate_args args0;
Rcpp::List x = Rcpp::wrap(args0);

// Instantiate a second args struct from the list x
x["stop_on_error"] = false;
fntl::integrate_args args1 = Rcpp::as<fntl::integrate_args>(x);
```
:::




When instantiating an `args` struct from a `List`, we throw an error if the
list contains any elements with names which are not expected by the struct;
this is to protect against mistakes which could occur when a field is named
incorrectly where the effects may be subtle and difficult to track down.




::: {.cell}

```{.cpp .cell-code}
// This will cause an exception
x["abcdefg"] = 0;
fntl::integrate_args args1 = Rcpp::as<fntl::integrate_args>(x);
```
:::




In most cases, a function that takes optional `args` has an alternative form
where the `args` may be omitted; this form assumes all default values for the
`args`. For example, our call to `integrate` in @sec-first-example could omit
the `args` as follows.




::: {.cell}

```{.cpp .cell-code}
fntl::integrate_result out = fntl::integrate(f, 0, 1);
```
:::




## Results

Similar to `args`, there are a number of `result` structs that represent the
output of a function. `integrate_result` is an example; other `result` types
have a prefix matching their corresponding function. A `result` may be
exported to an Rcpp `List` using the `wrap` mechanism. This was seen in
@sec-first-example.




::: {.cell}

```{.cpp .cell-code}
fntl::integrate_result out = fntl::integrate(f, 0, 1, args);
Rcpp::List x = Rcpp::wrap(out);
```
:::




## Status Codes {#sec-status}

Error conditions may result in an exception so that no return value is
produced. Some conditions - such as failure to converge within a given number
of iterations - may not result in an exception. Here, a `status` code is
returned with the result to indicate a possible issue that may warrant further
investigation. The example in @sec-first-example illustrates `integrate_status`
returned by `integrate`; other `status` types have a prefix matching their
corresponding function. Each `status` type is an enum class derived from either
`int` or `unsigned int`: an enumeration that can be constructed from an integer
or converted to an integer using the included function `to_underlying`.^[This
function is based on a post from [StackOverflow][to-underlying-url].]




::: {.cell}

```{.cpp .cell-code}
fntl::integrate_status status = fntl::integrate_status::OK;  // Define a status
int err = to_underlying(status);                             // status to int
status1 = fntl::integrate_status(err);                       // int to status
```
:::




[to-underlying-url]: https://stackoverflow.com/questions/8357240/how-to-automatically-convert-strongly-typed-enum-into-int

## Function Typedefs

`fntl` defines shorthands for several commonly used function types, given as
follows, which are used throughout the present document.




::: {.cell}

```{.cpp .cell-code}
typedef function<double(double)> dfd;
typedef function<double(const NumericVector&)> dfv;
typedef function<double(const NumericVector&, const NumericVector&)> dfvv;
typedef function<NumericVector(const NumericVector&)> vfv;
typedef function<NumericMatrix(const NumericVector&)> mfv;
```
:::




Type names are intended to convey argument and return types as briefly as
possible. Symbols to the right of `f` represent arguments while those to the
left represent the return type; in particular, `d` is **d**ouble, `v` is
numeric **v**ector, and `m` is numeric **m**atrix. For example, a `dfvv`
takes two vectors and returns a double.

The namespace `std` for `function` and `Rcpp` for `NumericVector` and
`NumericMatrix` have been omitted in the display so that statements each fit on
a single line.

## Constants

The following constants are defined in `fntl` and utilized in the API.




::: {.cell}

```{.cpp .cell-code}
double mach_eps = std::numeric_limits<double>::epsilon();
double mach_eps_2r = sqrt(mach_eps);
double mach_eps_4r = std::pow(mach_eps, 0.25);
unsigned int uint_max = std::numeric_limits<unsigned int>::max();
```
:::




These correspond to machine epsilon $\epsilon$, $\epsilon^{1/2}$,
$\epsilon^{1/4}$, and the maximum value of an unsigned integer.

## Error Actions

Several functions in `fntl` take an `error_action` as an input to determine how
to react in an error state. Here is the definition of `error_action`.




::: {.cell}

```{.cpp .cell-code}
enum class error_action : unsigned int {
	STOP = 3L,     // <1>
	WARNING = 2L,  // <2>
	MESSAGE = 1L,  // <3>
	NONE = 0L      // <4>
};
```
:::



1. Throw an exception.
2. Emit a warning and proceed.
3. Print a message and proceed.
4. Do not take any of the above actions and proceed.

Functions making use of an `error_action` typically also return a status code
as in @sec-status, which can be inspected by the caller if an exception is not
thrown.

## Inferring Return Types$^\dagger$ {#sec-inferring-return}

@sec-intro mentioned that the return type of a lambda does not necessarily need
to be specified in its interface. However, we must be vigilant when allowing
the type to be inferred, especially when working with `Rcpp` objects which are
converted seamlessly behind the scenes.

The following example appears harmless but is likely to crash R.




::: {.cell}

```{.cpp .cell-code}
// [[Rcpp::depends(fntl)]]
#include "fntl.h"

// [[Rcpp::export]]
Rcpp::List crash_ex(Rcpp::NumericVector x0)
{
	fntl::dfv f = [](Rcpp::NumericVector x) { return Rcpp::sum(x*x); };
	auto out = fntl::gradient(f, x0);
	return Rcpp::wrap(out);
}
```
:::




The issue appears to be that `Rcpp::sum` does not necessarily return a
`double` - the expected result of a `fntl::dfv` - but something else that
normally produces a `double` behind the scenes when needed. Some contextual
information may be missing so that the conversion does not occur.

One way to address this is to specify that `double` is the return type of the
lambda.




::: {.cell}

```{.cpp .cell-code}
fntl::dfv f = [](Rcpp::NumericVector x) -> double { return Rcpp::sum(x*x); };
```
:::




A second way to address the issue is to explicitly convert the result to a
`double` before returning.




::: {.cell}

```{.cpp .cell-code}
fntl::dfv f = [](Rcpp::NumericVector x) {
	double out = Rcpp::sum(x*x);
	return out;
};
```
:::




## R Functions as Lambdas

Although not a primary intended use of the `fntl` package, it is possible to
use R functions as inputs. This is accomplished by wrapping an `Rcpp::Function`
in a lambda. This will incur the usual overhead of calling R code from C++,
but may be useful for testing or in situations where the overhead is a small
proportion of the run time.

Here is an example based on the first example in @sec-first-example.

```{.cpp include="examples/callr.cpp"}
```

We create function `f` in R and pass it as an argument to `callr_ex`. The
lambda `ff` calls the function `f`, implicitly converting the input `x` into an
`Rcpp::NumericVector`, and converts the output from an `Rcpp::NumericVector` to
a `double`. Let us demonstrate a call to the function `callr_ex` from R.




::: {.cell}

```{.r .cell-code}
Rcpp::sourceCpp("examples/callr.cpp")
a = 2
b = 3
f = function(x) { x^(a - 1) * (1 - x)^(b - 1) }
out = callr_ex(f)
print(out$value)
```
:::




## R Interface

The `fntl` package includes an R interface which may be used to invoke much of
the underlying C++ API. This is intended for demonstration and testing purposes
only; performance will generally suffer here because of the overhead in moving
between C++ and R. In real applications, R code should make use of mainstream R
functions rather than this R interface.

For example, the `integrate0` R function is included to call the underlying
`integrate` C++ function shown in @sec-first-example. (The suffix "0" is added
to avoid a naming clash with R's `integrate` function). The R function
`integrate_args` can be used to construct a list which is suitable to pass to
`integrate0`.




::: {.cell}

```{.r .cell-code}
args = integrate_args()
print(args)
a = 2; b = 3
f = function(x) { x^(a-1) * (1-x)^(b-1) }
out = integrate0(f, 0, 1, args)
print(out$value)
```
:::




## Performance Illustration

To illustrate performance characteristics, let us consider a brief simulation
of the relationship between mean-squared error (MSE) and sample size
in a logistic regression, suppose $Y_i \sim \text{Ber}(p_i)$ are independent
with $p_i = \text{logit}^{-1}(\beta_0 + \beta_1 x_i)$. Data-generating values of
the coefficients are taken to be $\beta_0 = 0$ and $\beta_1 = 1$, respectively.
We take $R$ draws of $(y_1, \ldots, y_n)$ and compute the MLE
$\hat{\beta}^{(r)} = (\hat{\beta}_0^{(r)}, \hat{\beta}_1^{(r)})$
for the $r$th draw using the L-BFGS-B optimization method. The MSE associated
with sample size $n$ is computed as
$\text{MSE}_n = \frac{1}{R} \sum_{r=1}^R \lVert \hat{\beta}^{(r)} - \beta \rVert^2$;
this is computed for sample sizes $n \in \{ 100, 200, 500, 1000, 10000 \}$.
We describe four versions of the code; to see the implementations, refer to the
corresponding source files in the `examples` folder.




::: {.cell}

```{.r .cell-code}
source("examples/timing1.R")
Rcpp::sourceCpp("examples/timing2.cpp")
Rcpp::sourceCpp("examples/timing3.cpp")
Rcpp::sourceCpp("examples/timing4.cpp")
n_levels = c(100, 200, 500, 1000, 10000)
```
:::




The first version of the program `timing1_ex` is written in pure R. This
version uses a naively coded version of the loglikelihood based on a loop over
$y_1, \ldots, y_n$. Experienced R users will recognize that vectorization will
dramatically improve the performance. However, we will proceed with the slow
loglikelihood for illustration. A particular run of this code took 1.31 minutes.




::: {.cell}

```{.r .cell-code}
set.seed(1234)
start = Sys.time()
timing1_ex(R = 200, n_levels)
print(Sys.time() - start)
```
:::




The second version `timing2_ex` ports `timing1_ex` from R to C++, where loops
generally need not be avoided to achieve good performance. Here we define the
loglikelihood as a lambda function and use the `lbfgsb` function described in
@sec-lbfgs to carry out optimization. A run of this code took 22.17 seconds.




::: {.cell}

```{.r .cell-code}
set.seed(1234)
start = Sys.time()
timing2_ex(R = 200, n_levels)
print(Sys.time() - start)
```
:::




A third version of the code `timing3_ex` demonstrates that overhead of calling
an R function from C++ does not necessarily result in poor performance. Here we
make a vectorized call to `dbinom` for each evaluation of the loglikelihood.
In this case, the overhead does not contribute significantly to the run time:
a run took 25.94 seconds.




::: {.cell}

```{.r .cell-code}
set.seed(1234)
start = Sys.time()
timing3_ex(R = 200, n_levels)
print(Sys.time() - start)
```
:::




Finally, a fourth version of the code `timing4_ex` demonstrates a case where
the overhead of repeatedly calling an R function from C++ results in abysmal
performance. Here we revert back to the loop in `timing2_ex`, but call the
`plogis` function in R rather than use one provided in Rcpp. A run of this code
took 9.89 minutes.




::: {.cell}

```{.r .cell-code}
set.seed(1234)
start = Sys.time()
timing4_ex(R = 200, n_levels)
print(Sys.time() - start)
```
:::




## Pass by Value and Reference

References and const references can be used in C++ code to avoid making
unnecessary copies of variables which waste time and memory. This additional
level of control (and responsibility) is typically not considered in R
programming, where pass-by-value is the standard. Consider the following
function, which adds one to each element of a matrix and returns the sum of
the result.




::: {.cell}

```{.cpp .cell-code}
double sum1p(Rcpp::NumericMatrix x) { return Rcpp::sum(x + 1); }
```
:::




Here a copy of the matrix `x` is passed to `sum1p`. This pass-by-value can be
changed to pass-by-reference which avoids copying `x`.




::: {.cell}

```{.cpp .cell-code}
double sum1p_1(Rcpp::NumericMatrix& x) { return Rcpp::sum(x + 1); }
```
:::




The body of `sum1p_1` does not alter `x`, but there are no safeguards in place to
enforce that it does not. This might raise anxiety for a user of `sum1p_1`
about whether their `x` has been modified. To alleviate their anxiety, we can
provide a safeguard using a const reference.




::: {.cell}

```{.cpp .cell-code}
double sum1p_2(const Rcpp::NumericMatrix& x) { return Rcpp::sum(x + 1); }
```
:::




The compiler will emit an error if `sum1p_2` attempts to modify `x`.

Examples given in this document tend to use pass-by-value to avoid extra visual
clutter of const references. However, the `fntl` package makes frequent use of
const references in both the API and internal implementation. It is recommended
that users consider consider which of the three - by-value, by-reference, or
by-const-reference - is most appropriate for their application in actual usage.

# Integration {#sec-integrate}

Compute the integral
<!-- -->
$$
\int_a^b f(x) dx,
$$
<!-- -->
where limit $a$ may be finite or $-\infty$ and limit $b$ may be finite or
$\infty$. Directly uses the C functions [Rdqags][Rdqags-src] and
[Rdqagi][Rdqagi-src] underlying the R function [integrate][Rintegrate-src].
These functions are based on two respective QUADPACK routines:
[dqags][dqags-src] for the case when both limits are finite and
[dqagi][dqagi-src] for the case when one or both limits are infinite
[@PiessensEtAl1983]. 

[Rintegrate-src]: https://github.com/r-devel/r-svn/blob/main/src/library/stats/R/integrate.R
[Rdqags-src]: https://github.com/r-devel/r-svn/blob/b3133c253e72c7ae069e0d46964c9458a7219eaa/src/include/R_ext/Applic.h#L52
[Rdqagi-src]: https://github.com/r-devel/r-svn/blob/b3133c253e72c7ae069e0d46964c9458a7219eaa/src/include/R_ext/Applic.h#L57
[dqags-src]: https://www.netlib.org/quadpack/dqags.f
[dqagi-src]: https://www.netlib.org/quadpack/dqagi.f

**Function**

Source code is in the file [inst/include/integrate.h][integrate-h].

[integrate-h]: https://github.com/andrewraim/fntl/blob/main/inst/include/integrate.h




::: {.cell}

```{.cpp .cell-code}
integrate_result integrate(
	const dfd& f,        // <1>
	double lower,                // <2>
	double upper,                // <3>
    const integrate_args& args   // <4>
)

integrate_result integrate(
	const dfd& f,        // <1>
	double lower,                // <2>
	double upper                 // <3>
)
```
:::



1. Function to use as the integrand.
2. Lower limit $a$ of integral; may be `R_NegInf`.
3. Upper limit $b$ of integral; may be `R_PosInf`.
4. Additional arguments.

**Optional Arguments**




::: {.cell}

```{.cpp .cell-code}
struct integrate_args {
	unsigned int subdivisions = 100L;  // <1>
	double rel_tol = mach_eps_4r;      // <2>
	double abs_tol = mach_eps_4r;      // <3>
	bool stop_on_error = true;         // <4>
};
```
:::



1. The maximum number of subintervals.
2. Relative accuracy requested.
3. Absolute accuracy requested.
4. If `true`, errors in `integrate` raise exceptions.

**Result**




::: {.cell}

```{.cpp .cell-code}
struct integrate_result {
	double value;             // <1>
	double abs_error;         // <2>
	int subdivisions;         // <3>
	integrate_status status;  // <4>
	int n_eval;               // <5>
	std::string message;      // <6>

	operator SEXP() const;    // <7>
};
```
:::



1. The final approximation of the integral.
2. Estimate of the modulus of the absolute error.
3. The number of subintervals produced in the subdivision process.
4. A code describing the status of the operation.
5. Number of function evaluations.
6. A message describing the status of the operation.
7. Conversion operator to `Rcpp::List`.

The `SEXP` conversion operator produces the following representation of
`integrate_result` as an `Rcpp::List`. The fields here directly correspond to
those in `integrate_result`.

|Name            | Type                  | Description |
|:---------------|:----------------------|:------------|
| `value`        | `Rcpp::NumericVector` | Length 1    |
| `abs_error`    | `Rcpp::NumericVector` | Length 1    |
| `subdivisions` | `Rcpp::IntegerVector` | Length 1    |
| `status`       | `Rcpp::IntegerVector` | Length 1    |
| `n_eval`       | `Rcpp::IntegerVector` | Length 1    |
| `message`      | `Rcpp::StringVector`  | Length 1    |

: {tbl-colwidths="[15,30,55]"}


**Status Codes**




::: {.cell}

```{.cpp .cell-code}
enum class integrate_status : int {
	OK = 0L,                                  // <1>
	MAX_SUBDIVISIONS = 1L,                    // <2>
	ROUNDOFF_ERROR = 2L,                      // <3>
	BAD_INTEGRAND_BEHAVIOR = 3L,              // <4>
	ROUNDOFF_ERROR_EXTRAPOLATION_TABLE = 4L,  // <5>
	PROBABLY_DIVERGENT_INTEGRAL = 5L,         // <6>
	INVALID_INPUT = 6L                        // <7>
};
```
:::



1. OK.
2. maximum number of subdivisions reached.
3. roundoff error was detected.
4. extremely bad integrand behaviour.
5. roundoff error is detected in the extrapolation table.
6. the integral is probably divergent.
7. the input is invalid.

**Example**

Compute the integral $\int_0^\infty e^{-x^2/2} dx$. A C++ function with Rcpp
interface is defined in the file `examples/integrate.cpp`.

```{.cpp include="examples/integrate.cpp"}
```

Call the function from R.




::: {.cell}

```{.r .cell-code}
Rcpp::sourceCpp("examples/integrate.cpp")
out = integrate_ex(0 , Inf)
print(out)
```
:::




# Differentiation {#sec-diff}

This section presents methods for numerical differentiation. First we present
simple finite differences, then Richardson extrapolation to automatically
select a step size, then the functions to compute the gradient, Jacobian, and
Hessian. The latter three make use of Richardson extrapolation.

## Finite Differences {#sec-fdderiv}

Compute the first and second derivatives of
$f : \mathbb{R}^n \rightarrow \mathbb{R}$ numerically at point $x$.
Denote $e_i$ as the $i$th column of an $n \times n$ identity matrix.
First derivatives in the $i$th coordinate are computed as
<!-- -->
\begin{align*}
&\frac{\partial f(x)}{\partial x_i} \approx \frac{
f(x + h e_i) - f(x - h e_i)
}{2h}, \\
%
&\frac{\partial f(x)}{\partial x_i} \approx 
\frac{ f(x + h e_i) - f(x) }{h}, \\
%
&\frac{\partial f(x)}{\partial x_i} \approx 
\frac{ f(x) - f(x - h e_i) }{h},
\end{align*}
<!-- -->
for given $h > 0$ using symmetric, forward, or backward differences
respectively. Second derivatives in the $i$th and $j$th coordinate, with
$i, j \in \{ 1 \ldots, n\}$, are computed as
<!-- -->
\begin{align*}
&\frac{\partial^2 f(x)}{\partial x_i x_j} \approx
\frac{
f(x + h_i e_i + h_j e_j) - f(x + h_i e_i - h_j e_j) - f(x - h_i e_i + h_j e_j) +
f(x - h_i e_i - h_j e_j)
}{4 h_i h_j}, \\
%
&\frac{\partial^2 f(x)}{\partial x_i x_j} \approx
\frac{
f(x + h_i e_i + 2 h_j e_j) - f(x + h e_i) - f(x + h_j e_j) + f(x)
}{ h_i h_j}, \\
%
&\frac{\partial^2 f(x)}{\partial x_i x_j} \approx
\frac{
f(x) - f(x - h_i e_i) - f(x - h_j e_j) + f(x - h_i e_i - h_j e_j)
}{h_i h_j},
\end{align*}
<!-- -->
for given $h_1 > 0$ and $h_2 > 0$
using symmetric, forward, or backward differences respectively.
The accuracy of these derivatives depends on the nature of the function $f$ at
$x$ and the choice of $h$. See Section 5.7 of @PressEtAl2007 for discussion.

**Function**

Primary location of source code is the file [inst/include/fd-deriv.h][fd-deriv-h].

[fd-deriv-h]: https://github.com/andrewraim/fntl/blob/main/inst/include/fd-deriv.h




::: {.cell}

```{.cpp .cell-code}
double fd_deriv(
	const dfv& f,                                  // <1>
	const Rcpp::NumericVector& x,                  // <2>
	unsigned int i,                                // <3>
	double h,                                      // <5>
	const fd_types& fd_type = fd_types::SYMMETRIC  // <7>
)

double fd_deriv2(
	const dfv& f,                                  // <1>
	const Rcpp::NumericVector& x,                  // <2>
	unsigned int i,                                // <3>
	unsigned int j,                                // <4>
	double h_i,                                    // <5>
	double h_j,                                    // <6>
	const fd_types& fd_type = fd_types::SYMMETRIC  // <7>
)
```
:::



1. Function $d$ to differentiate.
2. Point $x$ at which derivative is taken.
3. First coordinate to differentiate.
4. Second coordinate to differentiate.
5. Step size in the first coordinate.
6. Step size in the second coordinate
7. Type of finite difference.

The functions `fd_deriv` and `fd_deriv2` compute first and second derivatives,
respectively. The options for `fd_type` are given in the following enumeration
class.




::: {.cell}

```{.cpp .cell-code}
enum class fd_types : unsigned int {
	SYMMETRIC = 0L,
	FORWARD = 1L,
	BACKWARD = 2L
};
```
:::




**Example**

Compute the first and second derivatives of $f(x) = \sin(b^\top x)$ at
$x_0 = (1/2, \ldots, 1/2)$ where $b = (1, \ldots, n)$. The gradient and Hessian
are given in closed-form by
<!-- -->
\begin{align*}
&\frac{\partial f(x)}{\partial x} = b \cos(b^\top x), \\
&\frac{\partial^2 f(x)}{\partial x \partial x^\top} = -b b^\top \sin(b^\top x).
\end{align*}
<!-- -->
Let us first prepare the problem in R.




::: {.cell}

```{.r .cell-code}
n = 3
b = seq_len(n)
x0 = rep(0.5, n)
eps = 0.001  ## Fix a step size for finite differences
g = function(x) { b * cos(sum(b * x)) }
h = function(x) { -tcrossprod(b) * sin(sum(b * x)) }
```
:::




To demonstrate the numerical first derivative, a C++ function with Rcpp
interface is defined in the file `examples/fd-deriv.cpp`.

```{.cpp include="examples/fd-deriv.cpp"}
```

Call the function from R.




::: {.cell}

```{.r .cell-code}
Rcpp::sourceCpp("examples/fd-deriv.cpp")
out3 = out2 = out1 = numeric(n)
for (i in 1:n) {
	out1[i] = fd_deriv_ex(x0, i-1, eps, type = 0)  ## Symmetric
	out2[i] = fd_deriv_ex(x0, i-1, eps, type = 1)  ## Forward
	out3[i] = fd_deriv_ex(x0, i-1, eps, type = 2)  ## Backward
}
print(out1)
print(out2)
print(out3)
print(g(x0))
```
:::




To demonstrate the numerical second derivative, a C++ function with Rcpp
interface is defined in the file `examples/fd-deriv2.cpp`.

```{.cpp include="examples/fd-deriv2.cpp"}
```

Call the function from R.




::: {.cell}

```{.r .cell-code}
Rcpp::sourceCpp("examples/fd-deriv2.cpp")
out3 = out2 = out1 = matrix(NA, n, n)
for (i in 1:n) {
for (j in 1:n) {
	out1[i,j] = fd_deriv2_ex(x0, i-1, j-1, eps, eps, type = 0)  ## Symmetric
	out2[i,j] = fd_deriv2_ex(x0, i-1, j-1, eps, eps, type = 1)  ## Forward
	out3[i,j] = fd_deriv2_ex(x0, i-1, j-1, eps, eps, type = 2)  ## Backward
}
}
print(out1)
print(out2)
print(out3)
h(x0)
```
:::




## Richardson Extrapolated Finite Differences {#sec-deriv}

Compute the first and second derivatives of
$f : \mathbb{R}^n \rightarrow \mathbb{R}$ numerically at point $x$ using
Richardson extrapolation. This is typically more accurate than the simple
finite differences in @sec-fdderiv @Sec-fdderiv and does not require an informed choice of
the step size $h$; however, it requires more evaluations of $f$. A general
discussion of Richardson extrapolation is given in Section 9.6 of
@QuarteroniSaccoSaleri2007.

Taking the function $g(h)$ to be one of finite difference approximations
(symmetric, forward, or backward), $h > 0$ to be an initial step size, and
$\delta \in (0,1)$ to be a reduction factor, the method computes a table of
values
<!-- -->
\begin{align*}
A_{mq} = 
\begin{cases}
g(\delta^m h), & \text{for $m = 0, \ldots, n$ and $q = 0$}, \\
\displaystyle \frac{A_{m,q-1} - \delta^q A_{m-1,q-1}}{1 - \delta^q}, & 
\text{for $m = 0, \ldots, n$ and $q = 1, \ldots, m$},
\end{cases}
\end{align*}
<!-- -->
given a predetermined $n$. Note that $n+1$ evaluations of $f$ are required. The
result is taken to be the $A_{mq}$ such that
<!-- -->
$$
e_{mq} = \max\{ |A_{mq} - A_{m,q-1}|, |A_{mq} - A_{m-1,q-1}| \}
$$
<!-- -->
is the smallest, with the corresponding $e = e_{mq}$ reported as the achieved
tolerance. Furthermore, because numerical error may begin to worsen as the
table is iteratively computed, the method halts if it encounters the condition
<!-- -->
$$
|A_{mq} - A_{m-1,q-1}| > \tau e,
$$
<!-- -->
with $e$ as the smallest $e_{mq}$ computed so far and $\tau > 1$ is a given
multiplier. These criteria for convergence and stopping early due to reduced
numerical precision are suggested in Section 5.7 of @PressEtAl2007; similar
considerations for halting criteria are considered in the Julia package
[Richardson.jl][richardson-jl].

Taking $n = 0$ produces finite differences described in @sec-fdderiv using the
initial step size $h$ as the perturbation. Here the achieved tolerance is
reported as infinite.

[richardson-jl]: https://github.com/JuliaMath/Richardson.jl

**Function**

Primary location of source code are the files
[inst/include/deriv.h][deriv-h],
[inst/include/deriv2.h][deriv2-h], and
[inst/include/richardson.h][richardson-h].

[deriv-h]: https://github.com/andrewraim/fntl/blob/main/inst/include/deriv.h
[deriv2-h]: https://github.com/andrewraim/fntl/blob/main/inst/include/deriv2.h
[richardson-h]: https://github.com/andrewraim/fntl/blob/main/inst/include/richardson.h





::: {.cell}

```{.cpp .cell-code}
richardson_result deriv(
	const dfv& f,                                  // <1>
	const Rcpp::NumericVector& x,                  // <2>
	unsigned int i,                                // <3>
	const richardson_args& args,                   // <5>
	const fd_types& fd_type = fd_types::SYMMETRIC  // <6>
)

richardson_result deriv(
	const dfv& f,                                  // <1>
	const Rcpp::NumericVector& x,                  // <2>
	unsigned int i,                                // <3>
	const fd_types& fd_type = fd_types::SYMMETRIC  // <6>
)

richardson_result deriv2(
	const dfv& f,                                  // <1>
	const Rcpp::NumericVector& x,                  // <2>
	unsigned int i,                                // <3>
	unsigned int j,                                // <4>
	const richardson_args& args,                   // <5>
	const fd_types& fd_type = fd_types::SYMMETRIC  // <6>
)

richardson_result deriv2(
	const dfv& f,                                  // <1>
	const Rcpp::NumericVector& x,                  // <2>
	unsigned int i,                                // <3>
	unsigned int j,                                // <4>
	const fd_types& fd_type = fd_types::SYMMETRIC  // <6>
)
```
:::



1. Function $d$ to differentiate.
2. Point $x$ at which derivative is taken.
3. Optional arguments.
4. First coordinate to differentiate.
5. Second coordinate to differentiate.
6. Type of finite difference.

The functions `deriv` and `deriv2` compute first and second derivatives,
respectively. The enum class `fd_type` is described in @sec-fdderiv.


**Optional Arguments**




::: {.cell}

```{.cpp .cell-code}
struct richardson_args
{
	double delta = 0.5;                 // <1>
	unsigned int maxiter = 10;          // <2>
	double h = 1;                       // <3>
	double tol = mach_eps_4r;           // <4>
	double accuracy_factor = R_PosInf;  // <5>

	richardson_args() { };              // <6>
	richardson_args(SEXP obj);          // <7>
	operator SEXP() const;              // <8>
};

```
:::



1. The factor $\delta$ used to reduce $h$.
2. Maximum number of iterations.
3. The initial value of $h$.
4. Tolerance for convergence.
5. The factor $\tau$ used to check for loss of precision. The infinite default
   value disables the check.
6. Default constructor.
7. Constructor from an `Rcpp::List`.
8. Conversion operator to `Rcpp::List`.

**Result**




::: {.cell}

```{.cpp .cell-code}
struct richardson_result
{
	double value;              // <1>
	double err;                // <2>
	unsigned int iter;         // <3>
	richardson_status status;  // <4>

	operator SEXP() const;     // <5>
};
```
:::



1. The final approximation of the derivative.
2. An estimate of the error in approximation.
3. Number of iterations $m$ used to produce the approximation.
4. A code describing the status of the operation. 
5. Conversion operator to `Rcpp::List`.

The `SEXP` conversion operator produces the following representation of
`fd_deriv_result` as an `Rcpp::List`.

|Name      | Type                  | Description                       |
|:---------|:----------------------|:----------------------------------|
| `value`  | `Rcpp::NumericVector` | A scalar based on field `value`.  |
| `err`    | `Rcpp::NumericVector` | A scalar based on field `err`.    |
| `iter`   | `Rcpp::IntegerVector` | A scalar based on field `iter`.   |
| `status` | `Rcpp::IntegerVector` | A scalar based on field `status`.   |

**Status Codes**




::: {.cell}

```{.cpp .cell-code}
enum class richardson_status : unsigned int {
	OK = 0L,                  // <1>
	NOT_CONVERGED = 1L,       // <2>
	NUMERICAL_PRECISION = 2L  // <3>
};
```
:::



1. OK.
2. Not converged within `maxiter` iterations.
3. Early termination due to numerical precision.

**Example**

Compute first and second derivatives of $f(x) = \sin(b^\top x)$ at
$x_0 = (1/2, \ldots, 1/2)$ with $b = (1, \ldots, n)$. This is a continuation of
the example in @sec-fdderiv. Let us again define the point $x_0$.




::: {.cell}

```{.r .cell-code}
n = 3
x0 = rep(0.5, n)
```
:::




To demonstrate the numerical first derivative, a C++ function with Rcpp
interface is defined in the file `examples/deriv.cpp`.

```{.cpp include="examples/deriv.cpp"}
```

Call the function from R.




::: {.cell}

```{.r .cell-code}
Rcpp::sourceCpp("examples/deriv.cpp")
out3 = out2 = out1 = numeric(n)
for (i in 1:n) {
	out1[i] = deriv_ex(x0, i-1, type = 0)$value  ## Symmetric
	out2[i] = deriv_ex(x0, i-1, type = 1)$value  ## Forward
	out3[i] = deriv_ex(x0, i-1, type = 2)$value  ## Backward
}
print(out1)
print(out2)
print(out3)
```
:::




To demonstrate the numerical second derivative, a C++ function with Rcpp
interface is defined in the file `examples/deriv2.cpp`.

```{.cpp include="examples/deriv2.cpp"}
```

Call the function from R.




::: {.cell}

```{.r .cell-code}
Rcpp::sourceCpp("examples/deriv2.cpp")
out3 = out2 = out1 = matrix(NA, n, n)
for (i in 1:n) {
for (j in 1:n) {
	out1[i,j] = deriv2_ex(x0, i-1, j-1, type = 0)$value  ## Symmetric
	out2[i,j] = deriv2_ex(x0, i-1, j-1, type = 1)$value  ## Forward
	out3[i,j] = deriv2_ex(x0, i-1, j-1, type = 2)$value  ## Backward
}
}
print(out1)
print(out2)
print(out3)
```
:::




## Gradient {#sec-gradient}

The gradient of $f : \mathbb{R}^n \rightarrow \mathbb{R}$:
<!-- -->
$$
\frac{\partial f(x)}{\partial x}
=
\left[
\frac{\partial f(x)}{\partial x_1},
\ldots,
\frac{\partial f(x)}{\partial x_n}
\right].
$$
<!-- -->
Each coordinate is computed via `deriv` in @sec-deriv.

**Function**

Primary location of source code is the file [inst/include/gradient.h][gradient-h].

[gradient-h]: https://github.com/andrewraim/fntl/blob/main/inst/include/gradient.h




::: {.cell}

```{.cpp .cell-code}
gradient_result gradient(
	const dfv& f,                                  // <1>
	const Rcpp::NumericVector& x,                  // <2>
	const richardson_args& args,                   // <3>
	const fd_types& fd_type = fd_types::SYMMETRIC  // <4>
)

gradient_result gradient(
	const dfv& f,                                  // <1>
	const Rcpp::NumericVector& x,                  // <2>
	const fd_types& fd_type = fd_types::SYMMETRIC  // <4>
)
```
:::



1. Function to take the gradient of.
2. Point at which to compute the gradient.
3. Optional arguments.
4. Type of finite difference to use. See the definition of `fd_types` in
   @sec-fdderiv.

The arguments in `args` are applied to each coordinate of the gradient.

**Result**




::: {.cell}

```{.cpp .cell-code}
struct gradient_result {
	std::vector<double> value;       // <1>
	std::vector<double> err;         // <2>
	std::vector<unsigned int> iter;  // <3>

	operator SEXP() const;           // <4>
};
```
:::



1. The final approximation of the gradient.
2. The respective approximation errors from `deriv` for each coordinate.
2. The respective iterations taken in `deriv` for each coordinate.
4. Conversion operator to `Rcpp::List`.

The `SEXP` conversion operator produces the following representation of
`gradient_result` as an `Rcpp::List`. The fields here directly correspond to
those in `gradient_result`.

|Name     | Type                  | Description |
|:--------|:----------------------|:------------|
| `value` | `Rcpp::NumericVector` | Length $n$  |
| `err`   | `Rcpp::NumericVector` | Length $n$  |
| `iter`  | `Rcpp::IntegerVector` | Length $n$  |

: {tbl-colwidths="[15,30,55]"}

**Example**

Compute the gradient of $f(x) = x^\top x$. A C++ function with Rcpp interface
is defined in the file `examples/gradient.cpp`.

```{.cpp include="examples/gradient.cpp"}
```

Call the function from R.




::: {.cell}

```{.r .cell-code}
Rcpp::sourceCpp("examples/gradient.cpp")
gradient_ex(x0 = 1:4)
```
:::




Compare to `grad` from the `numDeriv` package [@numDeriv].




::: {.cell}

```{.r .cell-code}
f = function(x) { sum(x^2) }
numDeriv::grad(f, x = 1:4)
```
:::




## Jacobian

The Jacobian of $f : \mathbb{R}^n \rightarrow \mathbb{R}^m$:
<!-- -->
$$
\frac{\partial f(x)}{\partial x}
=
\begin{bmatrix}
\frac{\partial f_1(x)}{\partial x_1} & \cdots & \frac{\partial f_1(x)}{\partial x_n} \\
\vdots & \ddots & \vdots \\
\frac{\partial f_m(x)}{\partial x_1} & \cdots & \frac{\partial f_m(x)}{\partial x_n}
\end{bmatrix}.
$$
<!-- -->
Each coordinate is computed via `deriv` in @sec-deriv.

**Function**

Source code is in the file [inst/include/jacobian.h][jacobian-h].

[jacobian-h]: https://github.com/andrewraim/fntl/blob/main/inst/include/jacobian.h




::: {.cell}

```{.cpp .cell-code}
jacobian_result jacobian(
	const vfv& f,                                 // <1>
	const Rcpp::NumericVector& x,                 // <2>
	const richardson_args& args                   // <3>
	const fd_types& fd_type = fd_types::SYMMETRIC // <4>
)

jacobian_result jacobian(
	const vfv& f,                                 // <1>
	const Rcpp::NumericVector& x,                 // <2>
	const fd_types& fd_type = fd_types::SYMMETRIC // <4>
)
```
:::



1. Function to obtain the Jacobian of
2. Point at which to take the Jacobian.
3. Optional arguments.
4. Type of finite difference to use. See the definition of `fd_types` in
   @sec-fdderiv.

**Result**




::: {.cell}

```{.cpp .cell-code}
struct jacobian_result
{
	std::vector<double> value;       // <1>
	std::vector<double> err;         // <2>
	std::vector<unsigned int> iter;  // <3>
	double rows;                     // <4>
	double cols;                     // <5>

	operator SEXP() const;           // <6>
};
```
:::



1. The final approximation of the Jacobian stored as a vector in row-major
   format.
2. The respective approximation errors from `deriv` for each coordinate of
   `value`.
3. The respective iterations taken in `deriv` for each coordinate of `value`.
4. The row dimension $m$ of the Jacobian.
5. The column dimension $n$ of the Jacobian.
6. Conversion operator to `Rcpp::List`.

The `SEXP` conversion operator produces the following representation of
`jacobian_result` as an `Rcpp::List`.

|Name     | Type                  | Description                                    |
|:--------|:----------------------|:-----------------------------------------------|
| `value` | `Rcpp::NumericMatrix` | An $m \times n$ matrix based on field `value`. |
| `err`   | `Rcpp::NumericMatrix` | An $m \times n$ matrix based on field `err`.   |
| `iter`  | `Rcpp::IntegerMatrix` | An $m \times n$ matrix based on field `iter`.  |

: {tbl-colwidths="[15,30,55]"}

**Example**

Compute the Jacobian of $f(x) = [f_1(x), \ldots, f_5(x)]$,
$f_i(x) = \sum_{j=1}^i \sin(x_j)$, at the point $x = (1,2,3,4,5)$. To obtain
the resulting value as an $m \times n$ matrix, we apply the List conversion
operator to the result. A C++ function with Rcpp interface is defined in the
file `examples/jacobian.cpp`.

```{.cpp include="examples/jacobian.cpp"}
```

Call the function from R.




::: {.cell}

```{.r .cell-code}
Rcpp::sourceCpp("examples/jacobian.cpp")
out = jacobian_ex(x0 = 1:4)
names(out)
print(out$value)
```
:::




Compare to `jacobian` from the `numDeriv` package [@numDeriv].




::: {.cell}

```{.r .cell-code}
f = function(x) { cumsum(sin(x)) }
numDeriv::jacobian(f, x = 1:4)
```
:::




## Hessian {#sec-hessian}

Compute the Hessian of $f : \mathbb{R}^n \rightarrow \mathbb{R}$ numerically
at point $x$:
<!-- -->
$$
\frac{\partial^2 f(x)}{\partial x \partial x^\top} =
\begin{bmatrix}
\frac{\partial^2 f(x)}{\partial x_1 \partial x_1} &
\cdots &
\frac{\partial^2 f(x)}{\partial x_1 \partial x_n} \\
\vdots & \ddots & \vdots \\
\frac{\partial^2 f(x)}{\partial x_n \partial x_1} &
\cdots &
\frac{\partial^2 f(x)}{\partial x_n \partial x_n}
\end{bmatrix}.
$$
<!-- -->

Each coordinate is computed via `deriv2` in @sec-deriv.

Note that there is a C function [fdhess][fdhess-src] within R which is based on
Algorithm A5.6.2 of @DennisSchnabel1983. However, it is not included in the API
for external use [@WritingRExtensions, Section 6].

[fdhess-src]: https://github.com/r-devel/r-svn/blob/5c0e367225ec0ed9e1a95864302a6311708533ee/src/include/R_ext/Applic.h#L131


**Function**

Primary location of source code is the file [inst/include/hessian.h][hessian-h].

[hessian-h]: https://github.com/andrewraim/fntl/blob/main/inst/include/hessian.h




::: {.cell}

```{.cpp .cell-code}
hessian_result hessian(
	const dfv& f,                                  // <1>
	const Rcpp::NumericVector& x,                  // <2>
	const richardson_args& args,                   // <3>
	const fd_types& fd_type = fd_types::SYMMETRIC  // <4>
)

hessian_result hessian(
	const dfv& f,                                  // <1>
	const Rcpp::NumericVector& x,                  // <2>
	const fd_types& fd_type = fd_types::SYMMETRIC  // <4>
)
```
:::



1. Function $f$ for which Hessian is to be computed.
2. Point $x$ at which Hessian is taken.
3. Optional arguments.
4. Type of finite difference to use. See the definition of `fd_types` in
   @sec-fdderiv.

**Result**

The result is an $n \times n$ Hessian matrix.




::: {.cell}

```{.cpp .cell-code}
struct hessian_result
{
	std::vector<double> value;       // <1>
	std::vector<double> err;         // <2>
	std::vector<unsigned int> iter;  // <3>
	double dim;                      // <4>

	operator SEXP() const;           // <5>
};
```
:::



1. The value of the Hessian: a vector containing the lower-triangular in
   column-major order.
2. The respective approximation errors from `deriv2` for each coordinate.
3. The respective iterations taken in `deriv2` for each coordinate.
4. The row and column dimension.
5. Conversion operator to `Rcpp::List`.

**Example**

Compute the Hessian of $f(x) = \sum_{i=1}^n \sin(x_i)$ at $x = (1, 2)$. To
obtain the resulting value as an $n \times n$ matrix, we apply the List
conversion operator to the result. A C++ function with Rcpp interface is
defined in the file `examples/hessian.cpp`.

```{.cpp include="examples/hessian.cpp"}
```

Call the function from R.




::: {.cell}

```{.r .cell-code}
Rcpp::sourceCpp("examples/hessian.cpp")
out = hessian_ex(x0 = c(1,2))
print(out)
```
:::




Compare to `hessian` from the `numDeriv` package [@numDeriv].




::: {.cell}

```{.r .cell-code}
f = function(x) { sum(sin(x)) }
numDeriv::hessian(f, x = c(1,2))
```
:::




# Root-Finding {#sec-findroot}

Find a root of $f : \mathbb{R} \rightarrow \mathbb{R}$, if present, on the
interval $[a,b]$; i.e., find $x \in [a,b]$ such that $f(x) \approx 0$.
The R function [uniroot][Runiroot-src] has its implementation in an underlying
C function [zeroin2][zeroin2-src] which appears not to be readily exported
for external use. We therefore provide several alternatives in C++.

[Runiroot-src]: https://github.com/r-devel/r-svn/blob/685baa84532ee09727f04725d5128f39d564bea5/src/library/stats/R/nlm.R#L55
[zeroin2-src]: https://github.com/r-devel/r-svn/blob/685baa84532ee09727f04725d5128f39d564bea5/src/library/stats/src/stats.h#L61C8-L61C17

There are currently two implementations. The bisection method
[e.g., @PressEtAl2007, Section 9.1] is simpler while Brent's method is
faster and is used in `uniroot`. This implementation of Brent's method is based
on the ALGOL code in Section 4.6 of @Brent1973. The two functions use a common
arguments struct, results struct, and status codes, which are given below.

**Optional Arguments**




::: {.cell}

```{.cpp .cell-code}
struct findroot_args {
	double tol = mach_eps_4r;                      // <1>
	unsigned int maxiter = 1000;                   // <2>
	error_action action = error_action::STOP;      // <3>
	
	findroot_args() { };                           // <4>
	findroot_args(SEXP obj);                       // <5>
	operator SEXP() const;                         // <6>
};
```
:::



1. Tolerance for convergence.
2. Maximum number of iterations.
3. Action to take if operation returns a status code other than `OK`.
4. Default constructor.
5. Constructor from an `Rcpp::List`.
6. Conversion operator to `Rcpp::List`.

**Result**




::: {.cell}

```{.cpp .cell-code}
struct findroot_result {
	double root;                 // <1>
	double f_root;               // <2>
	unsigned int iter;           // <3>
	double tol;                  // <4>
	findroot_status status;      // <5>
	std::string message;         // <6>

	operator SEXP() const;       // <7>
};
```
:::



1. The final approximation for the root.
2. The value of $f$ at the root.
3. The number of iterations.
4. An estimate of the error in approximation.
5. A code describing the status of the operation.
6. A message describing the status of the operation.
7. Conversion operator to `Rcpp::List`.

The `SEXP` conversion operator produces the following representation of
`findroot_result` as an `Rcpp::List`. The fields here directly correspond to
those in `findroot_result`.

|Name       | Type                  | Description |
|:----------|:----------------------|:------------|
| `root`    | `Rcpp::NumericVector` | Length 1    |
| `f_root`  | `Rcpp::NumericVector` | Length 1    |
| `iter`    | `Rcpp::IntegerVector` | Length 1    |
| `tol`     | `Rcpp::NumericVector` | Length 1    |
| `status`  | `Rcpp::IntegerVector` | Length 1    |
| `message` | `Rcpp::StringVector`  | Length 1    |

: {tbl-colwidths="[15,30,55]"}

**Status Codes**




::: {.cell}

```{.cpp .cell-code}
enum class findroot_status : unsigned int {
	OK = 0L,                  // <1>
	NUMERICAL_OVERFLOW = 1L,  // <2>
	NOT_CONVERGED = 2L        // <3>
};
```
:::



1. OK.
2. Numerical overflow: tol may be too small.
3. Not converged within `maxiter` iterations.


## Bisection {#sec-bisection}

This algorithm successively shrinks the starting interval and terminates when
the absolute difference is smaller than a tolerance $\epsilon$.

**Function**

Primary location of source code is the file [inst/include/findroot-bisect.h][findroot-bisect-h].

[findroot-bisect-h]: https://github.com/andrewraim/fntl/blob/main/inst/include/findroot-bisect.h




::: {.cell}

```{.cpp .cell-code}
findroot_result findroot_bisect(
	const dfd& f,              // <1>
	double lower,              // <2>
	double upper,              // <3>
	const findroot_args& args  // <4>
)

findroot_result findroot_bisect(
	const dfd& f,              // <1>
	double lower,              // <2>
	double upper               // <3>
)
```
:::



1. Function $f$ for which a root is desired.
2. Lower limit $a$ of search interval. Must be finite.
3. Upper limit $b$ of search interval. Must be finite.
4. Additional arguments.


**Example**

Find the root of the function $f(x) = x^2 - 1$ on $[0,10]$. A C++ function with
Rcpp interface is defined in the file `examples/findroot-bisect.cpp`.

```{.cpp include="examples/findroot-bisect.cpp"}
```

Call the function from R.




::: {.cell}

```{.r .cell-code}
Rcpp::sourceCpp("examples/findroot-bisect.cpp")
out = findroot_bisect_ex(0, 10)
print(out)
```
:::




## Brent's Algorithm

The algorithm successively shrinks the starting interval and terminates when
the absolute difference is smaller than a tolerance $\epsilon$.

**Function**

Primary location of source code is the file [inst/include/findroot-brent.h][findroot-brent-h].

[findroot-brent-h]: https://github.com/andrewraim/fntl/blob/main/inst/include/findroot-brent.h




::: {.cell}

```{.cpp .cell-code}
findroot_result findroot_brent(
	const dfd& f,              // <1>
	double lower,              // <2>
	double upper,              // <3>
	const findroot_args& args  // <4>
)

findroot_result findroot_brent(
	const dfd& f,              // <1>
	double lower,              // <2>
	double upper               // <3>
)
```
:::



1. Function $f$ for which a root is desired.
2. Lower limit $a$ of search space.
3. Upper limit $b$ of search space.
4. Additional arguments.

**Example**

Find the root of the function $f(x) = x^2 - 1$ on $[0,10]$. A C++ function with
Rcpp interface is defined in the file `examples/findroot-brent.cpp`.

```{.cpp include="examples/findroot-brent.cpp"}
```

Call the function from R.




::: {.cell}

```{.r .cell-code}
Rcpp::sourceCpp("examples/findroot-brent.cpp")
out = findroot_brent_ex(0, 10)
print(out)
```
:::




# Univariate Optimization {#sec-optimize}

Minimize the function $f : \mathbb{R} \rightarrow \mathbb{R}$ on a given
interval $[a, b]$. The R [optimize][Roptimize-src] function is based on an
underlying [Brent_fmin][optimize-src] C implementation of Brent's algorithm
[@Brent1973, Section 5]. However, this function appears not to be exported
from R for external use.

We provide several alternatives in C++: The golden section search method
[e.g., @PressEtAl2007, Section 10.2] is simple, guaranteed to converge, and
does not require information about derivatives. Brent's algorithm is more
involved but converges faster. The implementation of Brent's algorithm in C++
based is on the ALGOL code in Section 5.8 of @Brent1973. The golden section and
Brent functions use a common arguments struct, results struct, and status codes,
which are given below.

[optimize-src]: https://github.com/r-devel/r-svn/blob/5521fa62273d38f50e7e963337d74ec2f223c532/src/library/stats/src/optimize.c#L94
[Roptimize-src]: https://github.com/r-devel/r-svn/blob/5521fa62273d38f50e7e963337d74ec2f223c532/src/library/stats/R/nlm.R#L35

**Optional Arguments**




::: {.cell}

```{.cpp .cell-code}
struct optimize_args
{
	bool fnscale = 1;                          // <1>
	double tol = mach_eps_2r;                  // <2>
	unsigned int maxiter = 1000;               // <3>
	unsigned int report_period = uint_max;     // <4>
	error_action action = error_action::STOP;  // <5>

	optimize_args() { };                       // <6>
	optimize_args(SEXP obj);                   // <7>
	operator SEXP() const;                     // <8>
};

```
:::



1. Scaling factor to be applied to the value of $f(x)$ during optimization. Use
   `-1` to implement maximization rather than minimization.
2. Tolerance $\epsilon$ for convergence.
3. The maximum number of iterations.
4. The frequency of reports.
5. Action to take if operation returns a status code other than `OK`.
6. Default constructor.
7. Constructor from an `Rcpp::List`.
8. Conversion operator to `Rcpp::List`.

**Result**




::: {.cell}

```{.cpp .cell-code}
struct optimize_result
{
	double par;                   // <1>
	double value;                 // <2>
	unsigned int iter;            // <3>
	double tol;                   // <4>
	optimize_status status;       // <5>
	std::string message;          // <6>

	operator SEXP() const;        // <7>
};
```
:::



1. The final value of the optimization variable.
2. The value of the function corresponding to `par`.
3. The number of iterations taken.
4. The achieved tolerance $\delta$.
5. Status code from the optimizer.
6. A message describing the status of the operation.
7. Conversion operator to `Rcpp::List`.

The `SEXP` conversion operator produces the following representation of
`optimize_result` as an `Rcpp::List`. The fields here directly correspond to
those in `optimize_result`.

|Name       | Type                  | Description |
|:----------|:----------------------|:------------|
| `par`     | `Rcpp::NumericVector` | Length 1    |
| `value`   | `Rcpp::NumericVector` | Length 1    |
| `iter`    | `Rcpp::IntegerVector` | Length 1    |
| `tol`     | `Rcpp::NumericVector` | Length 1    |
| `status`  | `Rcpp::IntegerVector` | Length 1    |
| `message` | `Rcpp::StringVector`  | Length 1    |

: {tbl-colwidths="[15,30,55]"}

**Status Codes**




::: {.cell}

```{.cpp .cell-code}
enum class optimize_status : unsigned int {
	OK = 0L,                  // <1>
	NUMERICAL_OVERFLOW = 1L,  // <2>
	NOT_CONVERGED = 2L        // <3>
};
```
:::



1. OK.
2. Numerical overflow: tol may be too small.
3. iteration limit had been reached.

## Golden Section Search {#sec-golden}

The algorithm successively shrinks the starting interval and terminates when
$\delta = b - a$ is smaller than a given tolerance $\epsilon$.

**Function**

Primary location of source code is the file [inst/include/goldensection.h][goldensection-h].

[goldensection-h]: https://github.com/andrewraim/fntl/blob/main/inst/include/goldensection.h




::: {.cell}

```{.cpp .cell-code}
optimize_result goldensection(
	const dfd& f,              // <1>
	double lower,              // <2>
	double upper,              // <3>
	const optimize_args& args  // <4>
)

optimize_result goldensection(
	const dfd& f,              // <1>
	double lower,              // <2>
	double upper               // <3>
)
```
:::



1. Function $f$ to minimize.
2. Lower limit $a$ of search space.
2. Upper limit $b$ of search space.
4. Additional arguments.

**Example**

Maximize the function $f(x) = \exp(-x)$ on $[0,1]$. A C++ function with Rcpp
interface is defined in the file `examples/goldensection.cpp`.

```{.cpp include="examples/goldensection.cpp"}
```

Call the function from R.




::: {.cell}

```{.r .cell-code}
Rcpp::sourceCpp("examples/goldensection.cpp")
out = goldensection_ex(0, 1)
print(out)
```
:::




## Brent's Algorithm {#sec-brent-optim}

This algorithm successively shrinks the starting interval and terminates when
$\delta = |x - m|$ is smaller than a tolerance $\epsilon$, where
$m = (a + b) / 2$ is the midpoint of the current $[a, b]$.

**Function**

Primary location of source code is the file [inst/include/optimize-brent.h][optimize-brent-h].

[optimize-brent-h]: https://github.com/andrewraim/fntl/blob/main/inst/include/optimize-brent.h




::: {.cell}

```{.cpp .cell-code}
optimize_result optimize_brent(
	const dfd& f,              // <1>
	double lower,              // <2>
	double upper,              // <3>
	const optimize_args& args  // <4>
)

optimize_result optimize_brent(
	const dfd& f,              // <1>
	double lower,              // <2>
	double upper               // <3>
)
```
:::



1. Function $f$ to minimize.
2. Lower limit $a$ of search space.
2. Upper limit $b$ of search space.
4. Additional arguments.

**Example**

Maximize the function $f(x) = \exp(-x)$ on $[0,1]$. A C++ function with Rcpp
interface is defined in the file `examples/optimize-brent.cpp`.

```{.cpp include="examples/optimize-brent.cpp"}
```

Call the function from R.




::: {.cell}

```{.r .cell-code}
Rcpp::sourceCpp("examples/optimize-brent.cpp")
out = optimize_brent_ex(0, 1)
print(out)
```
:::




# Multivariate Optimization {#sec-optim}

This section presents methods to minimize a function
$f : \mathbb{R}^n \rightarrow \mathbb{R}$ from a given starting value $x_0$.

## Nelder-Mead

Relies only on evaluation of $f$ and does not use the
gradient or Hessian [@NelderMead1965]. This function directly calls
[nmmin][nmmin-src], the C function that gets invoked when using the
[optim][Roptim-src] R function with `method = "Nelder-Mead"`.

[nmmin-src]: https://github.com/r-devel/r-svn/blob/62dd8b09461d5d1313f4722d9d63abb463a9e42d/src/include/R_ext/Applic.h#L71
[Roptim-src]: https://github.com/r-devel/r-svn/blob/main/src/library/stats/R/optim.R

**Function**

Primary location of source code is the file [inst/include/neldermead.h][neldermead-h].

[neldermead-h]: https://github.com/andrewraim/fntl/blob/main/inst/include/neldermead.h




::: {.cell}

```{.cpp .cell-code}
neldermead_result neldermead(
	const Rcpp::NumericVector& init,  // <1>
	const dfv& f,                     // <2>
	const neldermead_args& args       // <3>
)

neldermead_result neldermead(
	const Rcpp::NumericVector& init,  // <1>
	const dfv& f                      // <2>
)
```
:::



1. Initial value for optimization variable.
2. Function $f$ to minimize.
3. Additional arguments.

**Optional Arguments**




::: {.cell}

```{.cpp .cell-code}
struct neldermead_args
{
	double alpha = 1.0;           // <1>
	double beta = 0.5;            // <2>
	double gamma = 2.0;           // <3>
	unsigned int trace = 0;       // <4>
	double abstol = R_NegInf;     // <5>
	double reltol = mach_eps_2r;  // <6>
	unsigned int maxit = 500;     // <7>
	double fnscale = 1.0;         // <8>

	neldermead_args() { };        // <9>
	neldermead_args(SEXP obj);    // <10>
	operator SEXP() const;        // <11>
};
```
:::



1. Reflection factor.
2. Contraction and reduction factor.
3. Extension factor.
4. If positive, print progress info.
5. Absolute tolerance.
6. User-initialized conversion tolerance.
7. Maximum number of iterations.
8. Scaling factor to be applied to the value of $f(x)$ during optimization. Use
   `-1` to implement maximization rather than minimization.
9. Default constructor.
10. Constructor from an `Rcpp::List`.
11. Conversion operator to `Rcpp::List`.

**Result**




::: {.cell}

```{.cpp .cell-code}
struct neldermead_result {
	std::vector<double> par;   // <1>
	double value;              // <2>
	neldermead_status status;  // <3>
	int fncount;               // <4>

	operator SEXP() const;     // <5>
};
```
:::



1. The final approximation of the optimizer.
2. The value of $f$ at the optimizer.
3. A code describing the status of the operation.
4. The number of function evaluations.
5. Conversion operator to `Rcpp::List`.

The `SEXP` conversion operator produces the following representation of
`neldermead_result` as an `Rcpp::List`. The fields here directly correspond to
those in `neldermead_result`.

|Name       | Type                  | Description |
|:----------|:----------------------|:------------|
| `par`     | `Rcpp::NumericVector` | Length $n$  |
| `value`   | `Rcpp::NumericVector` | Length 1    |
| `status`  | `Rcpp::IntegerVector` | Length 1    |
| `fncount` | `Rcpp::IntegerVector` | Length 1    |

: {tbl-colwidths="[15,30,55]"}

**Status Codes**




::: {.cell}

```{.cpp .cell-code}
enum class neldermead_status : unsigned int {
	OK = 0L,                 // <1>
	NOT_CONVERGED = 1L,      // <2>
	SIMLEX_DEGENERACY = 10L  // <3>
};
```
:::



1. OK.
2. Not converged within `maxiter` iterations.
3. Degeneracy of the Nelder–Mead simplex.

**Example**

Minimize the function $f(x) = \sum_{i=1}^n x_i^2 - 1$ for $n = 2$. Take
$x_0 = (1,-1)$ as the initial value. A C++ function with Rcpp interface is
defined in the file `examples/neldermead.cpp`.

```{.cpp include="examples/neldermead.cpp"}
```

Call the function from R.




::: {.cell}

```{.r .cell-code}
Rcpp::sourceCpp("examples/neldermead.cpp")
out = neldermead_ex(x0 = c(1, -1))
print(out)
```
:::




## BFGS

Minimization using the BFGS (Broyden-Fletcher-Goldfarb-Shanno)
algorithm [@Nash1990]. Relies on evaluation of $f$ and its gradient
$g(x) = \frac{\partial f(x)}{\partial x}$ but not the Hessian. This function
directly calls [vmmin][vmmin-src], the C function that gets invoked when
using the [optim][Roptim-src] R function with `method = "BFGS"`.

[vmmin-src]: https://github.com/r-devel/r-svn/blob/62dd8b09461d5d1313f4722d9d63abb463a9e42d/src/include/R_ext/Applic.h#L67
[Roptim-src]: https://github.com/r-devel/r-svn/blob/main/src/library/stats/R/optim.R

**Function**

Primary location of source code is the file [inst/include/bfgs.h][bfgs-h].

[bfgs-h]: https://github.com/andrewraim/fntl/blob/main/inst/include/bfgs.h




::: {.cell}

```{.cpp .cell-code}
bfgs_result bfgs(
	const Rcpp::NumericVector& init,  // <1>
	const dfv& f,                     // <2>
	const vfv& g,                     // <3>
	const bfgs_args& args             // <4>
)

bfgs_result bfgs(
	const Rcpp::NumericVector& init,  // <1>
	const dfv& f,                     // <2>
	const bfgs_args& args             // <4>
)

bfgs_result bfgs(
	const Rcpp::NumericVector& init,  // <1>
	const dfv& f,                     // <2>
	const vfv& g                      // <3>
)

bfgs_result bfgs(
	const Rcpp::NumericVector& init,  // <1>
	const dfv& f                      // <2>
)
```
:::



1. Initial value for optimization variable.
2. Function $f$ to minimize.
3. Gradient function $g(x) = \frac{\partial f(x)}{\partial x}$.
4. Additional arguments.

Forms with the $g$ argument omitted compute the gradient using finite
differences, via the `gradient` method in [@sec-gradient].

**Optional Arguments**




::: {.cell}

```{.cpp .cell-code}
struct bfgs_args {
	richardson_args deriv_args;    // <1>
	double parscale = 1;           // <2>
	int trace = 0;                 // <3>
	double fnscale = 1;            // <4>
	int maxit = 100;               // <5>
	int report = 10;               // <6>
	double abstol = R_NegInf;      // <7>
	double reltol = mach_eps_2r;   // <8>

	bfgs_args() { };               // <9>
	bfgs_args(SEXP obj);           // <10>
	operator SEXP() const;         // <11>
};
```
:::




1. Arguments for Richardson extrapolated numerical derivatives to compute
   the gradient if $g$ is omitted in the call to `bfgs`. See @sec-deriv.
2. A vector of scaling values for the parameters. (Currently not used).
3. If positive, tracing information on the progress of the optimization is
   produced. There are six levels which give progressively more detail.
4. Scaling factor applied to the value of `f` and `g` during optimization.
5. The maximum number of iterations.
6. The frequency of reports.
7. Absolute tolerance.
8. Relative tolerance.
9. Default constructor.
10. Constructor from an `Rcpp::List`.
11. Conversion operator to `Rcpp::List`.

**Result**




::: {.cell}

```{.cpp .cell-code}
struct bfgs_result {
	std::vector<double> par;  // <1>
	double value;             // <2>
	bfgs_status status;       // <3>
	int fncount;              // <4>
	int grcount;              // <5>

	operator SEXP() const;    // <6>
};
```
:::



1. The final value of the optimization variable.
2. The value of the function corresponding to `par`.
3. Status code from the optimizer.
4. Number of times the objective function was called.
5. Number of times the gradient function was called.
6. Conversion operator to `Rcpp::List`.

The `SEXP` conversion operator produces the following representation of
`bfgs_result` as an `Rcpp::List`. The fields here directly correspond to
those in `bfgs_result`.

|Name       | Type                  | Description |
|:----------|:----------------------|:------------|
| `par`     | `Rcpp::NumericVector` | Length $n$  |
| `value`   | `Rcpp::NumericVector` | Length 1    |
| `status`  | `Rcpp::IntegerVector` | Length 1    |
| `fncount` | `Rcpp::IntegerVector` | Length 1    |
| `grcount` | `Rcpp::IntegerVector` | Length 1    |

: {tbl-colwidths="[15,30,55]"}

**Status Codes**




::: {.cell}

```{.cpp .cell-code}
enum class bfgs_status : unsigned int {
	OK = 0L,             // <1>
	NOT_CONVERGED = 1L   // <2>
};
```
:::



1. OK.
2. iteration limit maxit had been reached.

**Example**

Maximize the function $f(x) = \exp(-x^\top x)$. First use the default
numerical gradient, then use an explicitly coded the gradient function
$g(x) = -2 x \exp(-x^\top x)$. A C++ function with Rcpp interface is defined in
the file `examples/bfgs.cpp`.

```{.cpp include="examples/bfgs.cpp"}
```

Call the function from R.




::: {.cell}

```{.r .cell-code}
Rcpp::sourceCpp("examples/bfgs.cpp")
out = bfgs_ex(x0 = rep(1, 4))
print(out$numerical)
print(out$analytical)
```
:::




## L-BFGS-B {#sec-lbfgs}

Minimization using the L-BFGS-B (Limited memory
Broyden-Fletcher-Goldfarb-Shanno) algorithm [@ByrdEtAl1995]. Relies on
evaluation of $f$ and its gradient $g(x) = \frac{\partial f(x)}{\partial x}$
but not the Hessian. This function directly calls [lbfgsb][lbfgsb-src], the C
function that gets invoked when using the [optim][Roptim-src] R function with
`method = "L-BFGS-B"`.

[lbfgsb-src]: https://github.com/r-devel/r-svn/blob/62dd8b09461d5d1313f4722d9d63abb463a9e42d/src/include/R_ext/Applic.h#L79
[Roptim-src]: https://github.com/r-devel/r-svn/blob/main/src/library/stats/R/optim.R

**Function**

Primary location of source code is the file [inst/include/lbfgsb.h][lbfgsb-h].

[lbfgsb-h]: https://github.com/andrewraim/fntl/blob/main/inst/include/lbfgsb.h




::: {.cell}

```{.cpp .cell-code}
lbfgsb_result lbfgsb(
	const Rcpp::NumericVector& init,  // <1>
	const dfv& f,                     // <2>
	const vfv& g,                     // <3>
	const lbfgsb_args& args           // <4>
)

lbfgsb_result lbfgsb(
	const Rcpp::NumericVector& init,  // <1>
	const dfv& f,                     // <2>
	const lbfgsb_args& args           // <4>
)

lbfgsb_result lbfgsb(
	const Rcpp::NumericVector& init,  // <1>
	const dfv& f,                     // <2>
	const vfv& g                      // <3>
)

lbfgsb_result lbfgsb(
	const Rcpp::NumericVector& init,  // <1>
	const dfv& f                      // <2>
)
```
:::



1. Initial value for optimization variable.
2. Function $f$ to minimize.
3. Gradient function $g(x) = \frac{\partial f(x)}{\partial x}$.
4. Additional arguments.

Forms with the $g$ argument omitted compute the gradient using finite
differences, via the `gradient` method in [@sec-gradient].

**Optional Arguments**




::: {.cell}

```{.cpp .cell-code}
struct lbfgsb_args {
	std::vector<double> lower;     // <1>
	std::vector<double> upper;     // <2>
	richardson_args deriv_args;    // <3>
	double parscale = 1;           // <4>
	int trace = 0;                 // <5>
	double fnscale = 1;            // <6>
	int lmm = 5;                   // <7>
	int maxit = 100;               // <8>
	int report = 10;               // <9>
	double factr = 1e7;            // <10>
	double pgtol = 0;              // <11>

	lbfgsb_args() { };             // <12>
	lbfgsb_args(SEXP obj);         // <13>
	operator SEXP() const;         // <14>
};
```
:::



1. A vector of lower bounds. If left unspecified, will be taken to be a vector
   of `-Inf` values.
2. A vector of upper bounds. If left unspecified, will be taken to be a vector
   of `Inf` values.
3. Arguments for Richardson extrapolated numerical derivatives to compute
   the gradient if $g$ is omitted in the call to `lbfgsb`. See @sec-deriv.
4. A vector of scaling values for the parameters. (Currently not used).
5. If positive, tracing information on the progress of the optimization is
   produced. There are six levels which give progressively more detail.
6. Scaling factor applied to the value of `f` and `g` during optimization.
7. Number of BFGS updates retained.
8. The maximum number of iterations.
9. The frequency of reports.
10. Convergence occurs when the reduction in the objective is within this
    factor of the machine tolerance.
11. A tolerance on the projected gradient in the current search direction. The
    check is suppressed when the value is zero.
12. Default constructor.
13. Constructor from an `Rcpp::List`.
14. Conversion operator to `Rcpp::List`.

**Result**




::: {.cell}

```{.cpp .cell-code}
struct lbfgsb_result {
	std::vector<double> par;  // <1>
	double value;             // <2>
	lbfgsb_status status;     // <3>
	int fncount;              // <4>
	int grcount;              // <5>
	std::string msg;          // <6>

	operator SEXP() const;    // <7>
};
```
:::



1. The final value of the optimization variable.
2. The value of the function corresponding to `par`.
3. Status code from the optimizer.
4. Number of times the objective function was called.
5. Number of times the gradient function was called.
6. String with additional information from the optimizer.
7. Conversion operator to `Rcpp::List`.

The `SEXP` conversion operator produces the following representation of
`lbfgsb_result` as an `Rcpp::List`. The fields here directly correspond to
those in `lbfgsb_result`.

|Name       | Type                  | Description |
|:----------|:----------------------|:------------|
| `par`     | `Rcpp::NumericVector` | Length $n$  |
| `value`   | `Rcpp::NumericVector` | Length 1    |
| `status`  | `Rcpp::IntegerVector` | Length 1    |
| `fncount` | `Rcpp::IntegerVector` | Length 1    |
| `grcount` | `Rcpp::IntegerVector` | Length 1    |
| `message` | `Rcpp::StringVector`  | Length 1    |

: {tbl-colwidths="[15,30,55]"}

**Status Codes**




::: {.cell}

```{.cpp .cell-code}
enum class lbfgsb_status : unsigned int {
	OK = 0L,             // <1>
	NOT_CONVERGED = 1L,  // <2>
	WARN = 51L,          // <3>
	ERROR = 52L,         // <4>
};
```
:::



1. OK.
2. iteration limit maxit had been reached.
3. algorithm reported a warning.
4. algorithm reported an error.

**Example**

Maximize the function $f(x) = \exp(-x^\top x)$. First use the default
numerical gradient, then use explicitly coded gradient function
$g(x) = -2 x \exp(-x^\top x)$. A C++ function with Rcpp interface is defined in
the file `examples/lbfgsb.cpp`.

```{.cpp include="examples/lbfgsb.cpp"}
```

Call the function from R.




::: {.cell}

```{.r .cell-code}
Rcpp::sourceCpp("examples/lbfgsb.cpp")
out = lbfgsb_ex(x0 = rep(1, 4))
print(out$numerical)
print(out$analytical)
```
:::




## Conjugate Gradient {#sec-cg}

Minimization using the conjugate gradient algorithm
[@FletcherReeves1964; @Nash1990]. Relies on evaluation of $f$ and its gradient
$g(x) = \frac{\partial f(x)}{\partial x}$ but not the Hessian. This function
directly calls [cgmin][cgmin-src], the C function that gets invoked when
using the [optim][Roptim-src] R function with `method = "CG"`.

[cgmin-src]: https://github.com/r-devel/r-svn/blob/62dd8b09461d5d1313f4722d9d63abb463a9e42d/src/include/R_ext/Applic.h#L75
[Roptim-src]: https://github.com/r-devel/r-svn/blob/main/src/library/stats/R/optim.R

**Function**

Primary location of source code is the file [inst/include/cg.h][cg-h].

[cg-h]: https://github.com/andrewraim/fntl/blob/main/inst/include/cg.h




::: {.cell}

```{.cpp .cell-code}
cg_result bfgs(
	const Rcpp::NumericVector& init,  // <1>
	const dfv& f,                     // <2>
	const vfv& g,                     // <3>
	const cg_args& args               // <4>
)

cg_result cg(
	const Rcpp::NumericVector& init,  // <1>
	const dfv& f,                     // <2>
	const cg_args& args               // <4>
)

cg_result cg(
	const Rcpp::NumericVector& init,  // <1>
	const dfv& f,                     // <2>
	const vfv& g                      // <3>
)

cg_result cg(
	const Rcpp::NumericVector& init,  // <1>
	const dfv& f                      // <2>
)
```
:::



1. Initial value for optimization variable.
2. Function $f$ to minimize.
3. Gradient function $g(x) = \frac{\partial f(x)}{\partial x}$.
4. Additional arguments.

Forms with the $g$ argument omitted compute the gradient using finite
differences, via the `gradient` method in [@sec-gradient].

**Optional Arguments**




::: {.cell}

```{.cpp .cell-code}

struct cg_args
{
	richardson_args deriv_args;    // <1>
	double parscale = 1;           // <2>
	double fnscale = 1;            // <3>
	double abstol = R_NegInf;      // <4>
	double reltol = mach_eps_2r;   // <5>
	int type = 1;                  // <6>
	int trace = 0;                 // <7>
	int maxit = 100;               // <8>

	cg_args() { };                 // <9>
	cg_args(SEXP obj);             // <10>
	operator SEXP() const;         // <11>
};
```
:::




1. Arguments for Richardson extrapolated numerical derivatives to compute
   the gradient if $g$ is omitted in the call to `cg`. See @sec-deriv.
2. A vector of scaling values for the parameters. (Currently not used).
3. Scaling factor applied to the value of `f` and `g` during optimization.
4. Absolute tolerance.
5. Relative tolerance.
6.  Type of update: 1 for Fletcher-Reeves, 2 for Polak-Ribiere, and 3 for
    Beale-Sorenson.
7. If positive, tracing information on the progress of the optimization is
   produced. There are six levels which give progressively more detail.
8. The maximum number of iterations.
9. Default constructor.
10. Constructor from an `Rcpp::List`.
11. Conversion operator to `Rcpp::List`.

**Result**




::: {.cell}

```{.cpp .cell-code}
struct cg_result {
	std::vector<double> par;  // <1>
	double value;             // <2>
	cg_status status;         // <3>
	int fncount;              // <4>
	int grcount;              // <5>

	operator SEXP() const;    // <6>
};
```
:::



1. The final value of the optimization variable.
2. The value of the function corresponding to `par`.
3. Status code from the optimizer.
4. Number of times the objective function was called.
5. Number of times the gradient function was called.
6. Conversion operator to `Rcpp::List`.

The `SEXP` conversion operator produces the following representation of
`cg_result` as an `Rcpp::List`. The fields here directly correspond to
those in `cg_result`.

|Name       | Type                  | Description |
|:----------|:----------------------|:------------|
| `par`     | `Rcpp::NumericVector` | Length $n$  |
| `value`   | `Rcpp::NumericVector` | Length 1    |
| `status`  | `Rcpp::IntegerVector` | Length 1    |
| `fncount` | `Rcpp::IntegerVector` | Length 1    |
| `grcount` | `Rcpp::IntegerVector` | Length 1    |

: {tbl-colwidths="[15,30,55]"}

**Status Codes**




::: {.cell}

```{.cpp .cell-code}
enum class cg_status : unsigned int {
	OK = 0L,             // <1>
	NOT_CONVERGED = 1L   // <2>
};
```
:::



1. OK.
2. iteration limit maxit had been reached.

**Example**

Maximize the function $f(x) = \exp(-x^\top x)$. First use the default
numerical gradient, then use an explicitly coded the gradient function
$g(x) = -2 x \exp(-x^\top x)$. A C++ function with Rcpp interface is defined in
the file `examples/cg.cpp`.

```{.cpp include="examples/cg.cpp"}
```

Call the function from R.




::: {.cell}

```{.r .cell-code}
Rcpp::sourceCpp("examples/cg.cpp")
out = cg_ex(x0 = rep(1, 4))
print(out$numerical)
print(out$analytical)
```
:::




## Newton-Type Algorithm for Nonlinear Optimization

Minimization using the Newton-type algorithm underlying the
[nlm][nlm-src] R function. The amounts to calling the C function
[optif9][optif9-src] within the R API. The implementation of `optif9` is based
on @DennisSchnabel1983.

The gradient $g(x) = \frac{\partial f(x)}{\partial x}$ and Hessian 
$h(x) = \frac{\partial^2 f(x)}{\partial x \partial x^\top}$ may be provided
explicitly if available. When the gradient and/or Hessian are not
specified, numerical approximations are used by `optif9`.

[optif9-src]: https://github.com/r-devel/r-svn/blob/62dd8b09461d5d1313f4722d9d63abb463a9e42d/src/include/R_ext/Applic.h#L136
[nlm-src]: https://github.com/r-devel/r-svn/blob/main/src/library/stats/R/nlm.R

**Function**

Primary location of source code is the file [inst/include/nlm.h][nlm-h].

[nlm-h]: https://github.com/andrewraim/fntl/blob/main/inst/include/nlm.h




::: {.cell}

```{.cpp .cell-code}

nlm_result nlm(
	const Rcpp::NumericVector& init,  // <1>
	const dfv& f,                     // <2>
	const vfv& g,                     // <3>
	const mfv& h,                     // <4>
	const nlm_args& args              // <5>
)

nlm_result nlm(
	const Rcpp::NumericVector& init,  // <1>
	const dfv& f,                     // <2>
	const vfv& g,                     // <3>
	const nlm_args& args              // <5>
)

nlm_result nlm(
	const Rcpp::NumericVector& init,  // <1>
	const dfv& f,                     // <2>
	const nlm_args& args              // <5>
)

nlm_result nlm(
	const Rcpp::NumericVector& init,  // <1>
	const dfv& f,                     // <2>
	const vfv& g,                     // <3>
	const mfv& h                      // <4>
)

nlm_result nlm(
	const Rcpp::NumericVector& init,  // <1>
	const dfv& f,                     // <2>
	const vfv& g                      // <3>
)

nlm_result nlm(
	const Rcpp::NumericVector& init,  // <1>
	const dfv& f                      // <2>
)
```
:::



1. Initial value for optimization variable.
2. Function $f$ to minimize.
3. Gradient of $f$.
4. Hessian of $f$.
5. Additional arguments.

**Optional Arguments**




::: {.cell}

```{.cpp .cell-code}
struct nlm_args
{
	std::vector<double> typsize;   // <1>
	unsigned int print_level = 0;  // <2>
	double fscale = 1;             // <3>
	double fnscale = 1;            // <4>
	unsigned int ndigit = 12;      // <5>
	double gradtol = 1e-6;         // <6>
	double stepmax = R_PosInf;     // <7>
	double steptol = 1e-6;         // <8>
	int iterlim = 100;             // <9>
	unsigned int method = 1;       // <10>
	double trust_radius = 1.0;     // <11>

	nlm_args() { };                // <12>
	nlm_args(SEXP obj);            // <13>
	operator SEXP() const;         // <14>
};
```
:::



1. An estimate of the size of each parameter at the minimum.
2. Verbosity of messages during optimization:
	- 0: no printing;
	- 1: final details are printed;
	- 2: full tracing is provided.
3. An estimate of the size of f at the minimum.
4. Scaling factor applied to the value of `f`, `g`, and `h` during
   optimization. Taking this to be `-1` changes the optimization to
   maximization.
5. Number of significant digits in the function $f$.
6. Tolerance to terminate algorithm based on the distance of gradient to zero.
7. Maximum allowable scaled step length
8. Tolerance to terminate algorithm based on relative step size of successive
   iterates.
9. The maximum number of iterations.
10. Algorithm used in optimization:
	- 1: line search;
    - 2: double dogleg;
    - 3: more-hebdon.
11. Radius of trust region.
12. Default constructor.
13. Constructor from an `Rcpp::List`.
14. Conversion operator to `Rcpp::List`.

Arguments correspond to those in `nlm` with the following exceptions.

- The arguments `method` and `trust_radius` are provided from within `optif9`
  but not exposed from `nlm`.
  
- An argument `hessian `is provided in `nlm` to compute the value of the
  Hessian via finite differences using the final value of the optimization
  variable. That is not provided here, but may be requested using the Hessian
  function in @sec-hessian.

- The `nlm` function provides an `check.analyticals` argument to check the
  correctness of provided expressions for the gradient and Hessian. This
  argument is not provided here, but a similar check can be done using the
  numerical gradient and Hessian functions in @sec-gradient and @sec-hessian,
  respectively.

See the `nlm` manual page for additional details about other arguments.

If `typsize` is given as the default empty value, it is transformed internally
to a vector of $n$ ones to match the default in `nlm`. Similarly, if `stepmax`
is given as the default infinity value, it is transformed internally to match
the default value in `nlm`.

**Result**




::: {.cell}

```{.cpp .cell-code}
struct nlm_result
{
	std::vector<double> par;      // <1>
	std::vector<double> grad;     // <2>
	double estimate;              // <3>
	int iterations;               // <4>
	nlm_status status;            // <5>

	operator SEXP() const;        // <6>
};
```
:::



1. The final value of the optimization variable.
2. The final value of the gradient.
3. The value of the function corresponding to `par`.
4. Number of iterations carried out.
5. Status code from the optimizer.
6. Conversion operator to `Rcpp::List`.

The `SEXP` conversion operator produces the following representation of
`nlm_result` as an `Rcpp::List`. The fields here directly correspond to
those in `nlm_result`.

|Name          | Type                  | Description  |
|:-------------|:----------------------|:-------------|
| `par`        | `Rcpp::NumericVector` | Length $n$   |
| `grad`       | `Rcpp::NumericVector` | Length $n$   |
| `estimate`   | `Rcpp::NumericVector` | Length 1     |
| `iterations` | `Rcpp::IntegerVector` | Length 1     |
| `status`     | `Rcpp::IntegerVector` | Length 1     |
| `hessian`    | `Rcpp::IntegerVector` | Length $n^2$ |

: {tbl-colwidths="[15,30,55]"}

**Status Codes**




::: {.cell}

```{.cpp .cell-code}
enum class nlm_status : unsigned int {
	OK = 0L,                   // <1>
	GRADIENT_WITHIN_TOL = 1L,  // <2>
	ITERATES_WITH_TOL = 2L,    // <3>
	NO_LOWER_STEP = 3L,        // <4>
	ITERATION_MAX = 4L,        // <5>
	STEP_SIZE_EXCEEDED = 5L    // <6>
};

```
:::



1. OK.
2. Relative gradient is within given tolerance.
3. Relative step size of successive iterates is within given tolerance.
4. Last global step failed to locate a point lower than `estimate`.
5. Reached maximum number of iterations.
6. Maximum step size `stepmax` exceeded five consecutive times. 

See the manual page for `nlm` for more detail about these statuses.

**Example**

Maximize the function $f(x) = \exp(-x^\top x)$. The associated gradient and
Hessian functions are $g(x) = -2 f(x) x$ and 
$h(x) = (4 x x^\top - 2 I) f(x)$, respectively. Consider three calls: first
with a numerical gradient and Hessian, then with explicitly coded gradient, and
finally with both explicitly coded gradient and Hessian. A C++ function with
Rcpp interface is defined in the file `examples/nlm.cpp`.

```{.cpp include="examples/nlm.cpp"}
```

Call the function from R.




::: {.cell}

```{.r .cell-code}
Rcpp::sourceCpp("examples/nlm.cpp")
out = nlm_ex(x0 = rep(1, 4))
nn = c("par", "grad")
print(out$res1[nn])
print(out$res2[nn])
print(out$res3[nn])
```
:::




# Matrix Operations {#sec-matrix}

This section presents several matrix operations based on a lambda function.

## Apply

Apply a function to the elements, rows, or columns of an $m \times n$ matrix.
Suppose $X \in \mathbb{R}^{m \times n}$ is a matrix with rows
$x_{1 \bullet}, \ldots, x_{m \bullet}$ and columns
$x_{\bullet 1}, \ldots, x_{\bullet n}$.

The function `mat_apply` is an elementwise application of
$f : \mathbb{R} \rightarrow \mathbb{R}$ which computes
<!-- -->
$$
\text{\texttt{mat\_apply}}(X, f)
= \begin{bmatrix}
f(x_{11}) & \cdots & f(x_{1n}) \\
\vdots & \ddots & \vdots \\
f(x_{m1}) & \cdots & f(x_{mn}) \\
\end{bmatrix}.
$$
<!-- -->
The function `row_apply` is a rowwise application of
$g : \mathbb{R}^n \rightarrow \mathbb{R}$ which computes
<!-- -->
$$
\text{\texttt{row\_apply}}(X, g)
= \big( g(x_{1 \bullet}), \ldots, g(x_{m \bullet}) \big).
$$
<!-- -->
The function `col_apply` is a columnwise application of
$h : \mathbb{R}^m \rightarrow \mathbb{R}$ which computes
<!-- -->
$$
\text{\texttt{col\_apply}}(X, h)
= \big( h(x_{\bullet 1}), \ldots, h(x_{\bullet n}) \big).
$$
<!-- -->
The above replicate the behavior of following R calls, respectively.




::: {.cell}

```{.r .cell-code}
apply(X, c(1,2), f)
apply(X, 1, f)
apply(X, 2, f)
```
:::




**Functions**

Primary location of source code is the file [inst/include/apply.h][apply-h].

[apply-h]: https://github.com/andrewraim/fntl/blob/main/inst/include/apply.h




::: {.cell}

```{.cpp .cell-code}
template <typename T, int RTYPE>
Rcpp::Vector<RTYPE> row_apply(
	const Rcpp::Matrix<RTYPE>& X,							// <1>
	const std::function<T(const Rcpp::Vector<RTYPE>&)>& f   // <2>
)

template <typename T, int RTYPE>
Rcpp::Vector<RTYPE> col_apply(
	const Rcpp::Matrix<RTYPE>& X,							// <1>
	const std::function<T(const Rcpp::Vector<RTYPE>&)>& f   // <2>
)

template <typename T, int RTYPE>
Rcpp::Matrix<RTYPE> mat_apply(
	const Rcpp::Matrix<RTYPE>& X,							// <1>
	const std::function<T(T)>& f                            // <2>
)
```
:::



1. An Rcpp matrix object.
2. Function $f$ to apply.

The functions `mat_apply`, `row_apply`, and `col_apply` correspond to
elementwise, rowwise, and columnwise apply.

Rcpp matrix objects may be of type `NumericMatrix`, `IntegerMatrix`,
or one of the others defined in the [Rcpp API][rcpp-matrices-src]. The template
argument `RTYPE` represents the type of data stored in in the matrix, taking
on value `REALSXP` for `NumericMatrix`, `INTSXP` for `IntegerMatrix`, etc. The
template argument `T` represents an underlying C++ variable type such as
`double`, `int`, etc.

The domain and range of function `f` should match the class of `x` and type of
apply operation. As an example, suppose `X` is an object of type `NumericMatrix`.

- The domain of `row_apply` should be of type `const NumericVector&` and
  the range should be of type `double`.
- The domain of `col_apply` should be of type `const NumericVector&` and
  the range should be of type `double`.
- The domain and range of `mat_apply` should be of type `double`.

[rcpp-matrices-src]: https://dirk.eddelbuettel.com/code/rcpp/html/instantiation_8h.html

**Example**

Compute the square of each element of a matrix, then its rowwise sums, then its
columnwise sums. A C++ function with Rcpp interface is defined in the file
`examples/apply.cpp`.

```{.cpp include="examples/apply.cpp"}
```

Call the function from R.




::: {.cell}

```{.r .cell-code}
Rcpp::sourceCpp("examples/apply.cpp")
X = matrix(1:12, 4, 3)
out = apply_ex(X)
print(out)
```
:::




## Outer

Construct a matrix from a real-valued function of two arguments. Or carry out a
matrix multiplication without explicitly constructing the matrix.

Suppose $f(x,x') : \mathbb{R}^d \times \mathbb{R}^d \rightarrow \mathbb{R}$ and
$X \in \mathbb{R}^{n \times d}$ is a matrix with rows $x_1, \ldots, x_n$. Also
let $a = [a_1 \cdots a_n]^\top$ be a fixed vector. The `outer` operation
computes the $n \times n$ symmetric matrix
<!-- -->
$$
\text{\texttt outer}(X, f) =
\begin{bmatrix}
f(x_1, x_1) & \cdots & f(x_1, x_n) \\
\vdots & \ddots & \vdots \\
f(x_n, x_1) & \cdots & f(x_n, x_n) \\
\end{bmatrix}
$$
<!-- -->
and the `outer_matvec` operation computes the $n$-dimensional vector
<!-- -->
$$
\text{\texttt outer\_matvec}(X, f, a) =
\begin{bmatrix}
f(x_1, x_1) & \cdots & f(x_1, x_n) \\
\vdots & \ddots & \vdots \\
f(x_n, x_1) & \cdots & f(x_n, x_n) \\
\end{bmatrix}
\begin{bmatrix}
a_1 \\
\vdots \\
a_n \\
\end{bmatrix}.
$$

Now suppose $f(x,y) : \mathbb{R}^{d_1} \times \mathbb{R}^{d_2} \rightarrow \mathbb{R}$,
$X \in \mathbb{R}^{m \times d_1}$ is a matrix with rows $x_1, \ldots, x_m$,
$Y \in \mathbb{R}^{n \times d_2}$ is a matrix with rows $y_1, \ldots, y_n$,
and $a = [a_1 \cdots a_n]^\top$ is a fixed vector. The `outer` operation
computes the $m \times n$ matrix
<!-- -->
$$
\text{\texttt outer}(X, Y, f) =
\begin{bmatrix}
f(x_1, y_1) & \cdots & f(x_1, y_n) \\
\vdots & \ddots & \vdots \\
f(x_m, y_1) & \cdots & f(x_m, y_n) \\
\end{bmatrix}
$$
<!-- -->
and the `outer_matvec` operation computes the $m$-dimensional vector
<!-- -->
$$
\text{\texttt outer\_matvec}(X, Y, f, a) =
\begin{bmatrix}
f(x_1, y_1) & \cdots & f(x_1, y_n) \\
\vdots & \ddots & \vdots \\
f(x_m, y_1) & \cdots & f(x_m, y_n) \\
\end{bmatrix}
\begin{bmatrix}
a_1 \\
\vdots \\
a_n \\
\end{bmatrix}.
$$

The `outer` functions above replicate the behavior of the `outer` function in R.

**Functions**

Primary location of source code is the file [inst/include/outer.h][outer-h].

[outer-h]: https://github.com/andrewraim/fntl/blob/main/inst/include/outer.h




::: {.cell}

```{.cpp .cell-code}
Rcpp::NumericMatrix outer(
	const Rcpp::NumericMatrix& X,  // <1>
	const dfvv& f                  // <3>
)

Rcpp::NumericMatrix outer(
	const Rcpp::NumericMatrix& X,  // <1>
	const Rcpp::NumericMatrix& Y,  // <2>
	const dfvv& f                  // <3>
)

Rcpp::NumericVector outer_matvec(
	const Rcpp::NumericMatrix& X,  // <1>
	const dfvv& f,                 // <3>
	const Rcpp::NumericVector& a   // <4>
)

Rcpp::NumericVector outer_matvec(
	const Rcpp::NumericMatrix& X,  // <1>
	const Rcpp::NumericMatrix& Y,  // <2>
	const dfvv& f,                 // <3>
	const Rcpp::NumericVector& a   // <4>
)
```
:::



1. An Rcpp matrix object of dimension $m \times d$.
2. An Rcpp matrix object of dimension $n \times d$.
3. Function $f$ to apply.
4. An Rcpp vector object of dimension $n$.

**Example**

Compute the distance between pairs of rows of $X$, then compute the distance
between each pairs of rows taken from $X$ and $Y$. Then multiply the respective
matrices by $a = [1, \ldots, 1]$. A C++ function with Rcpp interface is defined
in the file `examples/outer.cpp`.

```{.cpp include="examples/outer.cpp"}
```

Call the function from R.




::: {.cell}

```{.r .cell-code}
Rcpp::sourceCpp("examples/outer.cpp")
m = 5; n = 3; d = 2
X = matrix(rnorm(10), m, d)
Y = matrix(rnorm(6), n, d)
a = rep(1, m)
b = rep(1, n)
out = outer_ex(X, Y, a, b)
print(out)
```
:::




## Matrix-Free Linear Solve

Solve a linear system $A x = b$ for symmetric positive definite
$A \in \mathbb{R}^{n \times n}$ [@NocedalWright2006]. Here the operation $Ax$
is specified through a function $\ell(x) = Ax$. This can be used to avoid
explicit storage of large matrices when $A$ is sparse. The solution $x^*$ is
found by minimizing the quadratic function
<!-- -->
$$
f(x) = \frac{1}{2} x^\top \ell(x) - b^\top x,
$$
<!-- -->
so that $f'(x^*) = 0 \iff A x^* = b$. Minimization is carried out using the
conjugate gradient method in @sec-cg.

**Functions**

Primary location of source code is the file [inst/include/solve_cg.h][solve_cg-h].

[solve_cg-h]: https://github.com/andrewraim/fntl/blob/main/inst/include/solve_cg.h




::: {.cell}

```{.cpp .cell-code}
cg_result solve_cg(
	const vfv& l,                     // <1>
	const Rcpp::NumericVector& b,     // <2>
	const Rcpp::NumericVector& init,  // <3>
	const cg_args& args               // <4>
)

cg_result solve_cg(
	const vfv& l,                     // <1>
	const Rcpp::NumericVector& b,     // <2>
	const Rcpp::NumericVector& init   // <3>
)

cg_result solve_cg(
	const vfv& l,                     // <1>
	const Rcpp::NumericVector& b      // <2>
)
```
:::



1. The function $\ell : \mathbb{R}^n \rightarrow \mathbb{R}^n$.
2. The vector $b \in \mathbb{R}^{n}$.
3. Initial value for $x$.
4. Optional arguments to CG method.

The routine checks that `l(init)` is an $n$-dimensional vector, but there is no
check that the operation $\ell$ represents a symmetric positive definite
matrix.

See @sec-cg for details on `cg_args` and `cg_result`.

**Example**

Solve the equation $Ax = b$ with $n \times n$ matrix $A$ having value 2 
on the main diagonal and value 1 on the upper and lower diagonals; let
$b = [1, \ldots, 1]$. The initial value for the solution is taken to be the
default $x = 0$. A C++ function with Rcpp interface is defined in the file
`examples/solve-cg.cpp`.

```{.cpp include="examples/solve-cg.cpp"}
```

Call the function from R.




::: {.cell}

```{.r .cell-code}
Rcpp::sourceCpp("examples/solve-cg.cpp")
b = rep(1, 10)
out = solve_cg_ex(b)
print(out)
```
:::




Compare the above to solving dense the system as follows.




::: {.cell}

```{.r .cell-code}
A = matrix(0, 10, 10)
diag(A) = 2
A[cbind(1:9, 2:10)] = 1
A[cbind(2:10, 1:9)] = 1
solve(A, b)
```
:::





## Which

Identify the indices of a matrix which satisfy a given indicator function.
Specifically, let $X \in \mathbb{S}^{m \times n}$ be a matrix whose elements
are in the domain $\mathbb{S}$ which may be doubles, integers, or another
Rcpp `Matrix` type. Let $f : \mathbb{S} \rightarrow \{0, 1\}$ be an indicator
function and suppose $k$ of the $mn$ elements in $X$ satisfy the indicator.
The `which` operation produces a $k \times 2$ matrix
<!-- -->
$$
\text{which}(X, f) =
\begin{bmatrix}
i_1 & j_1 \\
\vdots & \vdots \\
i_k & j_k \\
\end{bmatrix},
$$
<!-- -->
where each pair $(i_\ell, j_\ell)$ are coordinates of an element in $X$ that
satisfies $f$. Indices $i_\ell$ and $j_\ell$ represent the row and column
index, respectively. Indices are *zero-based* by default as they are primarily
intended to be used in C++ code.

An equivalent R operation is the following. Note that indices in R are
*one-based*.




::: {.cell}

```{.r .cell-code}
which(f(X), arr.ind = TRUE)
```
:::




**Functions**

Primary location of source code is the file [inst/include/which.h][which-h].

[which-h]: https://github.com/andrewraim/fntl/blob/main/inst/include/which.h




::: {.cell}

```{.cpp .cell-code}
template <typename T, int RTYPE>
Rcpp::IntegerMatrix which(
	const Rcpp::Matrix<RTYPE>& X,      // <1>
	const std::function<bool(T)>& f),  // <2>
	bool one_based = false             // <3>
)
```
:::



1. An Rcpp matrix object.
2. Function $f$ to apply.
3. If `one_based = false`, zero-based indices are produced which are suitable
   for use with C++ code. If `true`, one-based indices are produced.

**Example**

Construct a matrix and identify the elements between $0$ and $0.5$. A C++
function with Rcpp interface is defined in the file `examples/which.cpp`.

```{.cpp include="examples/which.cpp"}
```

Call the function from R.




::: {.cell}

```{.r .cell-code}
Rcpp::sourceCpp("examples/which.cpp")
x = runif(10, -1, 1)
X = matrix(x, 2, 5)
out = which_ex(X)
print(X)
print(out)
```
:::




Here is the result in R for comparison.




::: {.cell}

```{.r .cell-code}
f = function(x) { x > 0 & x < 0.5 }
which(f(X), arr.ind = TRUE) - 1
```
:::




# Truncated Distributions

Functions to support a univariate distribution truncated to the interval
$(a , b]$. Operations include variate generation, the density function, the
quantile function, and the cumulative distribution function (CDF). Suppose
$f$, $F$, and $F^{-}$ are the density, CDF, and quantile
function, respectively, of an untruncated distribution.

The density $g$ of the truncated distribution is
<!-- -->
$$
g(x) = \frac{f(x)}{F(b) - F(a)} \textrm{I}(a < x \leq b).
$$
<!-- -->
The CDF $G$ of the truncated distribution is
<!-- -->
$$
G(x) = \frac{F(x) - F(a)}{F(b) - F(a)} \textrm{I}(a < x \leq b).
$$
<!-- -->
The quantile function $G^{-}$ of the truncated distribution is
<!-- -->
$$
G^{-}(\phi) = F^{-}\Big( \{ F(b) - F(a) \} \phi + F(a) \Big).
$$
<!-- -->
Draws are generated from the truncated distribution via the inverse CDF method
as $X = G^{-}(U)$ where $U \sim \text{Uniform}(0, 1)$. Some precautions are
taken computationally to avoid loss of precision. Computations are kept on the
log-scale internally accommodate very small probabilities. Probabilities based
on the CDF are considered using both the form $\Prob(X \leq x)$ and its
complement to aid with small probabilities far into the tails.

Special attention is needed for lower limits with a discrete distribution.
For example, consider the $\text{Poisson}(\lambda)$ distribution truncated to
$[1,10]$. Taking $(a, b]$ with $a = 1$ and $b = 10$ excludes the lower limit 1,
which is not desired. Instead, the correct truncation is achieved with
$a = a_0 - \epsilon$, where $a_0$ is the lower limit of interest (e.g., 1) and
$\epsilon$ is a small positive number (e.g., $10^{-5}$).

This functions in this section were originally adapted from the `rtruncated`
function in the `LearnBayes` package [@LearnBayes].

**Typedefs**

The following typedefs are utilized specifically in this section.




::: {.cell}

```{.cpp .cell-code}
typedef std::function<double(double,bool)> density;       // <1>
typedef std::function<double(double,bool,bool)> cdf;      // <2>
typedef std::function<double(double,bool,bool)> quantile; // <3>
```
:::



1. A density whose arguments are: (1) the argument `x` for which to evaluate
   the density and (2) a boolean `log` that determines whether the value is
   returned on the log-scale.
2. A CDF whose arguments are: (1) the argument `x` for which to evaluate
   the CDF, (2) a boolean `lower` that determines whether the lower-tail
   probability $\Prob(X \leq x)$ is to be computed, and (3) a boolean `log`
   that determines whether the value is returned on the log-scale.
3. A quantile function whose arguments are: (1) the argument `p` for which to
   compute quantiles, (2) a boolean `lower` that determines whether `p` is
   interpreted as `p` (if `true`) or `1 - p` (if `false`), and (3) a boolean
   `log` that determines whether `p` is assumed to be on the log-scale (if
   `true`).

## Density

**Functions**

Primary location of source code is the file [inst/include/trunc.h][trunc-h].
The following functions operate on scalar arguments.

[trunc-h]: https://github.com/andrewraim/fntl/blob/main/inst/include/trunc.h




::: {.cell}

```{.cpp .cell-code}
double d_trunc(
	double x,                      // <1>
	double lo,                     // <2>
	double hi,                     // <3>
	const density& f,              // <4>
	const cdf& F,                  // <5>
	bool log = false               // <6>
)

Rcpp::NumericVector d_trunc(
	const Rcpp::NumericVector& x,  // <1>
	const Rcpp::NumericVector& lo, // <2>
	const Rcpp::NumericVector& hi, // <3>
	const density& f,              // <4>
	const cdf& F,                  // <5>
	bool log = false               // <6>
)
```
:::



1. Argument of density and CDF.
2. Lower limit.
3. Upper limit.
4. Function that computes density of untruncated distribution.
5. Function that computes CDF of untruncated distribution.
6. Boolean; if `true`, probabilities $p$ are given as $\log(p)$.

When `x`, `lo` and `hi` are specified as vectors, they must be the same length.

**Example**

Compute density of the $\text{Beta}(2,5)$ distribution truncated to the
interval $(0.5, 0.95]$. A C++ function with Rcpp interface is defined in the
file `examples/d_trunc.cpp`.

```{.cpp include="examples/d_trunc.cpp"}
```

Call the function from R.




::: {.cell}

```{.r .cell-code}
Rcpp::sourceCpp("examples/d_trunc.cpp")
x = seq(0, 1, length.out = 30)
d_trunc_ex(x, shape1 = 2, shape2 = 5, lo = 0.5, hi = 0.95)
```
:::




## CDF

**Functions**

Primary location of source code is the file [inst/include/trunc.h][trunc-h].
The following functions operate on scalar arguments.

[trunc-h]: https://github.com/andrewraim/fntl/blob/main/inst/include/trunc.h




::: {.cell}

```{.cpp .cell-code}
double p_trunc(
	double x,              // <1>
	double lo,             // <2>
	double hi,             // <3>
	const cdf& F,          // <4>
	bool lower = true,     // <5>
	bool log = false       // <6>
)

Rcpp::NumericVector p_trunc(
	const Rcpp::NumericVector& x,  // <1>
	const Rcpp::NumericVector& lo, // <2>
	const Rcpp::NumericVector& hi, // <3>
	const cdf& F,                  // <4>
	bool lower = true,             // <5>
	bool log = false               // <6>
)
```
:::



1. Argument of CDF.
2. Lower limit.
3. Upper limit.
4. Function that specifies CDF of untruncated distribution.
5. Boolean; if `true`, probabilities are $\Prob(X \leq x)$, otherwise
   $\Prob(X > x)$.
6. Boolean; if `true`, probabilities $p$ are given as $\log(p)$.

When `x`, `lo` and `hi` are specified as vectors, they must be the same length.

**Example**

Compute CDF of the $\text{Beta}(2,5)$ distribution truncated to the
interval $(0.5, 0.95]$. A C++ function with Rcpp interface is defined in the
file `examples/p_trunc.cpp`.

```{.cpp include="examples/p_trunc.cpp"}
```

Call the function from R.




::: {.cell}

```{.r .cell-code}
Rcpp::sourceCpp("examples/p_trunc.cpp")
x = seq(0, 1, length.out = 30)
p_trunc_ex(x, shape1 = 2, shape2 = 5, lo = 0.5, hi = 0.95)
```
:::





## Quantile

**Functions**

Primary location of source code is the file [inst/include/trunc.h][trunc-h].
The following functions operate on scalar arguments.

[trunc-h]: https://github.com/andrewraim/fntl/blob/main/inst/include/trunc.h




::: {.cell}

```{.cpp .cell-code}
double q_trunc(
	double p,              // <1>
	double lo,             // <2>
	double hi,             // <3>
	const cdf& F,          // <4>
	const quantile& Finv,  // <5>
	bool lower = true,     // <6>
	bool log = false       // <7>
)

Rcpp::NumericVector q_trunc(
	const Rcpp::NumericVector& p,  // <1>
	const Rcpp::NumericVector& lo, // <2>
	const Rcpp::NumericVector& hi, // <3>
	const cdf& F,                  // <4>
	const quantile& Finv,          // <5>
	bool lower = true,             // <6>
	bool log = false               // <7>
)
```
:::



1. Probability argument.
2. Lower limit.
3. Upper limit.
4. Function that specifies CDF of untruncated distribution.
5. Function that computes quantiles of untruncated distribution.
6. Boolean; if `true`, probabilities are $\Prob(X \leq x)$, otherwise
   $\Prob(X > x)$.
7. Boolean; if `true`, probabilities $p$ are given as $\log(p)$.

When `x`, `lo` and `hi` are specified as vectors, they must be the same length.

**Example**

Compute CDF of the $\text{Beta}(2,5)$ distribution truncated to the
interval $(0.5, 0.95]$. A C++ function with Rcpp interface is defined in the
file `examples/q_trunc.cpp`.

```{.cpp include="examples/q_trunc.cpp"}
```

Call the function from R.




::: {.cell}

```{.r .cell-code}
Rcpp::sourceCpp("examples/q_trunc.cpp")
p = seq(0, 1, length.out = 30)
q_trunc_ex(p, shape1 = 2, shape2 = 5, lo = 0.5, hi = 0.95)
```
:::




## Variate Generation

**Functions**

Primary location of source code is the file [inst/include/trunc.h][trunc-h].
The following functions operate on scalar arguments.

[trunc-h]: https://github.com/andrewraim/fntl/blob/main/inst/include/trunc.h




::: {.cell}

```{.cpp .cell-code}
double r_trunc(
	double lo,             // <2>
	double hi,             // <3>
	const cdf& F,          // <4>
	const quantile& Finv   // <5>
)

Rcpp::NumericVector r_trunc(
	unsigned int n,                // <1>
	const Rcpp::NumericVector& lo, // <2>
	const Rcpp::NumericVector& hi, // <3>
	const cdf& F,                  // <4>
	const quantile& Finv           // <5>
)

```
:::



1. Number of desired draws.
2. Lower limit.
3. Upper limit.
4. Function that specifies CDF of untruncated distribution.
5. Function that computes quantiles of untruncated distribution.

When `lo` and `hi` are specified as vectors, they must be the same length.

The following are analogous functions for some of the arguments are vectors,
for convenience.

**Example**

Draw from the $\text{Beta}(2,5)$ distribution truncated to the interval
$(0.5, 0.95]$. A C++ function with Rcpp interface is defined in the file
`examples/rtrunc.cpp`.

```{.cpp include="examples/r_trunc.cpp"}
```

Call the function from R.




::: {.cell}

```{.r .cell-code}
Rcpp::sourceCpp("examples/r_trunc.cpp")
r_trunc_ex(n = 20, shape1 = 2, shape2 = 5, lo = 0.5, hi = 0.95)
```
:::




# References {-}

::: {#refs}
:::

When adding or modifying a function, make sure the following have been updated:

- Errors should be handled.
	a. Set status codes to report any conditions the user might need to know
	   about.
	b. Consider adding an `error_action` to control whether some statuses
	   should be thrown as errors, raised as warnings, or printed.

- Function should have appropriate args struct, if needed.
	a. Should contain reasonable defaults.
	b. Should have a conversion operator to an Rcpp List.
	c. Should have a constructor that takes an Rcpp List.

- Function should have an appropriate results struct, if needed.
	a. Should have a conversion operator to an Rcpp List.

- The header file should be included in `inst/include/fntl.h` so that
  function(s) can be called by external code.

- Rcpp interface functions should be present in `src/rcpp-interface.h` and
  `src/rcpp-interface.cpp`. Make sure Roxygen there is updated.

- Function should have appropriate sections in the vignette. This should
  include at least one example in `vignettes/examples`

- Add tests to `inst/tinytest`.

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{which0}
\alias{which0}
\title{Matrix Which Function}
\usage{
which0(X, f)
}
\arguments{
\item{X}{A matrix}

\item{f}{A predicate to apply to each element of \eqn{X}.}
}
\value{
A matrix with two columns. Each row contains a row and column index
corresponding to an element of \eqn{X} that matches the criteria of \eqn{f}.
See section "Which" of the package vignette for details.
}
\description{
Matrix Which Function
}
\details{
The \code{which} C++ functions are intended to operate like the following call
in R.

\if{html}{\out{<div class="sourceCode">}}\preformatted{which(f(X), arr.ind = TRUE) - 1
}\if{html}{\out{</div>}}

The R functions exposed here are specific to numeric-valued matrices, but
the underlying C++ functions are intended to work with any type of Rcpp
Matrix.
}
\examples{
X = matrix(1:12 / 6, nrow = 4, ncol = 3)
f = function(x) { x < 1 }
which0(X, f)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{integrate0}
\alias{integrate0}
\title{Integration}
\usage{
integrate0(f, lower, upper, args)
}
\arguments{
\item{f}{Function to integrate.}

\item{lower}{Lower limit of integral.}

\item{upper}{Upper limit of integral.}

\item{args}{List of additional arguments from the function \code{integrate_args}.}
}
\value{
A list with the form of a \code{integrate_result} described in section
"Integration" of the package vignette.
}
\description{
Compute the integral \eqn{\int_a^b f(x) dx}.
}
\examples{
f = function(x) { exp(-x^2 / 2) }
args = integrate_args()
integrate0(f, 0, 10, args)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{args}
\alias{args}
\alias{findroot_args}
\alias{optimize_args}
\alias{integrate_args}
\alias{cg_args}
\alias{bfgs_args}
\alias{lbfgsb_args}
\alias{neldermead_args}
\alias{nlm_args}
\alias{richardson_args}
\title{Arguments}
\usage{
findroot_args()

optimize_args()

integrate_args()

cg_args()

bfgs_args()

lbfgsb_args()

neldermead_args()

nlm_args()

richardson_args()
}
\value{
An argument list corresponding to the specified function. The elements of
the list are named and supplied with default values. See the package
vignette for further details.
\itemize{
\item \code{findroot_args} is documented in the section "Root-Finding".
\item \code{optimize_args} is documented in the section "Univariate Optimization".
\item \code{integrate_args} is documented in the section "Integration".
\item \code{cg_args} is documented in the section "Conjugate Gradient".
\item \code{bfgs_args} is documented in the section "BFGS".
\item \code{lbfgsb_args} is documented in the section "L-BFGS-B".
\item \code{neldermead_args}is documented in the section "Nelder-Mead".
\item \code{nlm_args} is documented in the section "Newton-Type Algorithm for
Nonlinear Optimization".
\item \code{richardson_args} is documented in the section "Richardson
Extrapolated Finite Differences".
}
}
\description{
Get an arguments list for internal methods with the default settings. This
object can be adjusted and passed to the respective function.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{outer}
\alias{outer}
\alias{outer1}
\alias{outer2}
\alias{outer1_matvec}
\alias{outer2_matvec}
\alias{outer1_triplet}
\alias{outer2_triplet}
\title{Outer Matrix}
\usage{
outer1(X, f)

outer2(X, Y, f)

outer1_matvec(X, f, a)

outer2_matvec(X, Y, f, a)

outer1_triplet(X, f, g)

outer2_triplet(X, Y, f, g)
}
\arguments{
\item{X}{A numerical matrix.}

\item{f}{Function \eqn{f(x, y)} that operates on a pair of rows and
returns an element for the result.}

\item{Y}{A numerical matrix.}

\item{a}{A scalar vector.}

\item{g}{Function \eqn{g(x, y)} that operates on a pair of rows and returns
a logical value; if \code{TRUE}, the corresponding output of \code{f} is placed into
the result. Otherwise, the corresponding output of \code{f} is omitted from the
result.}
}
\value{
\code{outer1} and \code{outer2} return a matrix. \code{outer1_matvec} and \code{outer2_matvec}
return a vector. \code{outer1_triplet} and \code{outer2_triplet} return a list with
a sparse matrix (triplet) representation. See section "Outer" of the
package vignette for details.
}
\description{
Compute "outer" matrices and matrix-vector products based on a function
that operators on pairs of rows. See details.
}
\details{
Functions \eqn{f(x,y)} and \eqn{g(x,y)} operate on pairs of rows. In the
functions with prefix \code{outer1}, \eqn{x} and \eqn{y} are pairs of rows of
\eqn{X}. In the functions with prefix \code{outer2}, \eqn{x} is a row from
\eqn{X} and \eqn{y} is a row from from \eqn{Y}.

The \code{outer1} function computes the \eqn{n \times n} symmetric matrix

\deqn{
\text{\texttt outer1}(X, f) =
\begin{bmatrix}
f(x_1, x_1) & \cdots & f(x_1, x_n) \cr
\vdots & \ddots & \vdots \cr
f(x_n, x_1) & \cdots & f(x_n, x_n) \cr
\end{bmatrix}
}

and the \code{outer1_matvec} operation computes the \eqn{n}-dimensional vector

\deqn{
\text{\texttt outer1\_matvec}(X, f, a) =
\begin{bmatrix}
f(x_1, x_1) & \cdots & f(x_1, x_n) \cr
\vdots & \ddots & \vdots \cr
f(x_n, x_1) & \cdots & f(x_n, x_n) \cr
\end{bmatrix}
\begin{bmatrix}
a_1 \cr
\vdots \cr
a_n \cr
\end{bmatrix}.
}

The \code{outer2} operation computes the \eqn{m \times n} matrix

\deqn{
\text{\texttt outer2}(X, Y, f) =
\begin{bmatrix}
f(x_1, y_1) & \cdots & f(x_1, y_n) \cr
\vdots & \ddots & \vdots \cr
f(x_m, y_1) & \cdots & f(x_m, y_n) \cr
\end{bmatrix}
}

and the \code{outer2_matvec} operation computes the \eqn{m}-dimensional vector

\deqn{
\text{\texttt outer2\_matvec}(X, Y, f, a) =
\begin{bmatrix}
f(x_1, y_1) & \cdots & f(x_1, y_n) \cr
\vdots & \ddots & \vdots \cr
f(x_m, y_1) & \cdots & f(x_m, y_n) \cr
\end{bmatrix}
\begin{bmatrix}
a_1 \cr
\vdots \cr
a_n \cr
\end{bmatrix}.
}

The \code{outer_triplet1} and \code{outer_triplet2} operations are analogous to
\code{outer1} and \code{outer2}, respectively, except return sparse matrices in
triplet form. The triplet variants take an additional argument \code{g}, a
function $g(x_i, y_j)$ that returns \code{TRUE} if the result is to be stored
in the result and \code{FALSE} otherwise. The argument \code{g} can be used for
sparseness.
}
\examples{
set.seed(1234)
f = function(x,y) { sum( (x - y)^2 ) }
X = matrix(rnorm(12), 6, 2)
Y = matrix(rnorm(10), 5, 2)
outer1(X, f)
outer2(X, Y, f)

a = rep(1, 6)
b = rep(1, 5)
outer1_matvec(X, f, a)
outer2_matvec(X, Y, f, b)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{deriv}
\alias{deriv}
\alias{fd_deriv1}
\alias{fd_deriv2}
\alias{deriv1}
\alias{deriv2}
\title{Numerical Derivatives via Finite Differences}
\usage{
fd_deriv1(f, x, i, h, fd_type)

fd_deriv2(f, x, i, j, h_i, h_j, fd_type)

deriv1(f, x, i, args, fd_type)

deriv2(f, x, i, j, args, fd_type)
}
\arguments{
\item{f}{Function to differentiate.}

\item{x}{Scalar at which to evaluate the derivative.}

\item{i}{First coordinate to differentiate.}

\item{h}{Step size in the first coordinate.}

\item{fd_type}{Type of derivative: \code{0} for symmetric difference, \code{1} for
forward difference, and \code{2} for backward difference.}

\item{j}{Second coordinate to differentiate.}

\item{h_i}{Step size in the first coordinate.}

\item{h_j}{Step size in the second coordinate.}

\item{args}{List of additional arguments from the function \code{richardson_args}.}
}
\value{
\code{fd_deriv1} and \code{fd_deriv2} return a single numeric value corresponding to
the first and second derivative via finite differences. \code{deriv1} and
\code{deriv2} return a list with the form of a \code{richardson_result} described in
section "Richardson Extrapolated Finite Differences" of the package
vignette.
}
\description{
Numerical Derivatives via Finite Differences
}
\examples{
args = richardson_args()

f = sin   # Try 2nd derivatives of a univariate function
x0 = 0.5
print(-sin(x0))  ## Exact answer for f''(x0)

fd_deriv2(f, x0, i = 0, j = 0, h_i = 0.001, h_j = 0.001, fd_type = 0)
fd_deriv2(f, x0, i = 0, j = 0, h_i = 0.001, h_j = 0.001, fd_type = 1)
fd_deriv2(f, x0, i = 0, j = 0, h_i = 0.001, h_j = 0.001, fd_type = 2)

deriv2(f, x0, i = 0, j = 0, args, fd_type = 0)

# Try 2nd derivatives of a bivariate function
f = function(x) { sin(x[1]) + cos(x[2]) }
x0 = c(0.5, 0.25)

print(-sin(x0[1]))  ## Exact answer for f_xx(x0)
print(-cos(x0[2]))  ## Exact answer for f_yy(x0)
print(0)         ## Exact answer for f_xy(x0,y0)

numDeriv::hessian(f, x0)

fd_deriv2(f, x0, i = 0, j = 0, h_i = 0.001, h_j = 0.001, fd_type = 0)
fd_deriv2(f, x0, i = 0, j = 0, h_i = 0.001, h_j = 0.001, fd_type = 1)
fd_deriv2(f, x0, i = 0, j = 0, h_i = 0.001, h_j = 0.001, fd_type = 2)

fd_deriv2(f, x0, i = 0, j = 1, h_i = 0.001, h_j = 0.001, fd_type = 0)
fd_deriv2(f, x0, i = 0, j = 1, h_i = 0.001, h_j = 0.001, fd_type = 1)
fd_deriv2(f, x0, i = 0, j = 1, h_i = 0.001, h_j = 0.001, fd_type = 2)

fd_deriv2(f, x0, i = 1, j = 1, h_i = 0.001, h_j = 0.001, fd_type = 0)
fd_deriv2(f, x0, i = 1, j = 1, h_i = 0.001, h_j = 0.001, fd_type = 1)
fd_deriv2(f, x0, i = 1, j = 1, h_i = 0.001, h_j = 0.001, fd_type = 2)

deriv2(f, x0, i = 1, j = 1, args, fd_type = 0)
deriv2(f, x0, i = 1, j = 1, args, fd_type = 1)
deriv2(f, x0, i = 1, j = 1, args, fd_type = 2)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{univariate-optimization}
\alias{univariate-optimization}
\alias{goldensection}
\alias{optimize_brent}
\title{Univariate Optimization}
\usage{
goldensection(f, lower, upper, args)

optimize_brent(f, lower, upper, args)
}
\arguments{
\item{f}{Function to optimize.}

\item{lower}{Lower limit of search interval. Must be finite.}

\item{upper}{Upper limit of search interval. Must be finite.}

\item{args}{List of additional arguments from the function \code{optimize_args}.}
}
\value{
A list with the form of a \code{optimize_result} described in section
"Univariate Optimization" of the package vignette.
}
\description{
Univariate Optimization
}
\examples{
f = function(x) { x^2 - 1 }
args = optimize_args()
goldensection(f, 0, 10, args)
optimize_brent(f, 0, 10, args)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{trunc}
\alias{trunc}
\alias{d_trunc}
\alias{p_trunc}
\alias{q_trunc}
\alias{r_trunc}
\title{Functions for Truncated Distributions}
\usage{
d_trunc(x, lo, hi, f, F, log = FALSE)

p_trunc(x, lo, hi, F, lower = TRUE, log = FALSE)

q_trunc(p, lo, hi, F, Finv, lower = TRUE, log = FALSE)

r_trunc(n, lo, hi, F, Finv)
}
\arguments{
\item{x}{Vector of quantiles.}

\item{lo}{Vector of lower limits.}

\item{hi}{Vector of upper limits.}

\item{f}{Density function with form \code{f(x, log)}.}

\item{F}{CDF function with signature \code{F(x, lower, log)}, where \code{x} is
numeric and \code{lower} and \code{log} are logical.}

\item{log}{logical; if \code{TRUE}, probabilities are given on log-scale.}

\item{lower}{logical; if \code{TRUE}, probabilities are \eqn{P(X \leq x)};
otherwise, \eqn{P(X > x)}.}

\item{p}{Vector of probabilities.}

\item{Finv}{Quantile function with signature \code{Finv(x, lower, log)}, where
\code{x} is numeric and \code{lower} and \code{log} are logical.}

\item{n}{Number of draws.}
}
\value{
Vector with results.
\code{d_trunc} computes the density, \code{r_trunc} generates random deviates,
\code{p_trunc} computes the CDF, and \code{q_trunc} computes quantiles.
}
\description{
Density, CDF, quantile, and drawing functions for a univariate distribution
with density \eqn{f}, cdf, \eqn{F}, and quantile function \eqn{F^{-}}
truncated to the interval \eqn{[a ,b]}.
}
\examples{
library(tidyverse)

m = 100  ## Length of sequence for density, CDF, etc
shape1 = 5
shape2 = 2
lo = 0.5
hi = 0.7

# Density, CDF, and quantile function for untruncated distribution
ff = function(x, log) { dbeta(x, shape1, shape2, log = log) }
FF = function(x, lower, log) { pbeta(x, shape1, shape2, lower.tail = lower, log = log) }
FFinv = function(x, lower, log) { qbeta(x, shape1, shape2, lower.tail = lower, log = log) }

# Compare truncated and untruncated densities
xseq = seq(0, 1, length.out = m)
lo_vec = rep(lo, m)
hi_vec = rep(hi, m)
f0seq = ff(xseq, log = FALSE)
fseq = d_trunc(xseq, lo_vec, hi_vec, ff, FF)
data.frame(x = xseq, f = fseq, f0 = f0seq) \%>\%
    ggplot() +
    geom_line(aes(xseq, fseq)) +
    geom_line(aes(xseq, f0seq), lty = 2) +
    xlab("x") +
    ylab("Density") +
    theme_minimal()

# Compare truncated densities and empirical density of generated draws
n = 100000
lo_vec = rep(lo, n)
hi_vec = rep(hi, n)
x = r_trunc(n = n, lo_vec, hi_vec, FF, FFinv)
hist(x, probability = TRUE, breaks = 15)
points(xseq, fseq)

# Compare empirical CDF of draws with CDF function
Femp = ecdf(x)
lo_vec = rep(lo, m)
hi_vec = rep(hi, m)
Fseq = p_trunc(xseq, lo_vec, hi_vec, FF)
data.frame(x = xseq, FF = Fseq) \%>\%
    mutate(F0 = Femp(x)) \%>\%
    ggplot() +
    geom_line(aes(xseq, FF), lwd = 1.2) +
    geom_line(aes(xseq, F0), col = "orange") +
    xlab("x") +
    ylab("Probability") +
    theme_minimal()

# Compare empirical quantiles of draws with quantile function
pseq = seq(0, 1, length.out = m)
lo_vec = rep(lo, m)
hi_vec = rep(hi, m)
Finvseq = q_trunc(pseq, lo_vec, hi_vec, FF, FFinv)
Finvemp = quantile(x, prob = pseq)
data.frame(p = pseq, Finv = Finvseq, Finvemp = Finvemp) \%>\%
    ggplot() +
    geom_line(aes(pseq, Finv), lwd = 1.2) +
    geom_line(aes(pseq, Finvemp), col = "orange") +
    xlab("p") +
    ylab("Quantile") +
    theme_minimal()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{hessian0}
\alias{hessian0}
\title{Numerical Hessian}
\usage{
hessian0(f, x, args)
}
\arguments{
\item{f}{Function to differentiate.}

\item{x}{Vector at which to evaluate the Hessian.}

\item{args}{List of additional arguments from the function \code{richardson_args}.}
}
\value{
A list with the form of a \code{hessian_result} described in section "Hessian"
of the package vignette.
}
\description{
Numerical Hessian
}
\examples{
f = function(x) { sum(x^2) }
x0 = seq(1, 10, length.out = 5)
args = richardson_args()
hessian0(f, x0, args)
numDeriv::hessian(f, x0)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{findroot}
\alias{findroot}
\alias{findroot_bisect}
\alias{findroot_brent}
\title{Find Root}
\usage{
findroot_bisect(f, lower, upper, args)

findroot_brent(f, lower, upper, args)
}
\arguments{
\item{f}{Function for which a root is desired.}

\item{lower}{Lower limit of search interval. Must be finite.}

\item{upper}{Upper limit of search interval. Must be finite.}

\item{args}{List of additional arguments from the function \code{findroot_args}.}
}
\value{
A list with the form of a \code{findroot_result} described in section
"Root-Finding" of the package vignette.
}
\description{
Find Root
}
\examples{
f = function(x) { x^2 - 1 }
args = findroot_args()
findroot_bisect(f, 0, 10, args)
findroot_brent(f, 0, 10, args)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{multivariate-optimization}
\alias{multivariate-optimization}
\alias{cg1}
\alias{cg2}
\alias{bfgs1}
\alias{bfgs2}
\alias{lbfgsb1}
\alias{lbfgsb2}
\alias{neldermead}
\alias{nlm1}
\alias{nlm2}
\alias{nlm3}
\title{Multivariate Optimization}
\usage{
cg1(init, f, g, args)

cg2(init, f, args)

bfgs1(init, f, g, args)

bfgs2(init, f, args)

lbfgsb1(init, f, g, args)

lbfgsb2(init, f, args)

neldermead(init, f, args)

nlm1(init, f, g, h, args)

nlm2(init, f, g, args)

nlm3(init, f, args)
}
\arguments{
\item{init}{Initial value}

\item{f}{Function \eqn{f} to optimize}

\item{g}{Gradient function of \eqn{f}.}

\item{args}{List of additional arguments for optimization.}

\item{h}{Hessian function of \eqn{f}.}
}
\value{
A list with results corresponding to the specified function. See the
package vignette for further details.
\itemize{
\item \code{cg1} and \code{cg2} return a \code{cg_result} which is documented in the section
"Conjugate Gradient".
\item \code{bfgs1} and \code{bfgs2} return a \code{bfgs_result} which is documented in the
section "BFGS".
\item \code{lbfgsb1} and \code{lbfgsb2} return a \code{lbfgsb_result} which is documented in
the section "L-BFGS-B".
\item \code{neldermead} returns a \code{neldermead_result} which is documented in
the section "Nelder-Mead".
\item \code{nlm1}, \code{nlm2}, and \code{nlm3} return a \code{nlm_result} which is documented in
the section "Newton-Type Algorithm for Nonlinear Optimization".
}
}
\description{
Multivariate Optimization
}
\details{
The argument \code{args} should be a list constructed from one of the following
functions:
\itemize{
\item \code{bfgs_args} for BFGS;
\item \code{lbfgsb_args} for L-BFGS-B;
\item \code{cg_args} for CG;
\item \code{neldermead_args} for Nelder-Mead;
\item \code{nlm_args} for the Newton-type algorithm used in \code{nlm}.
}

When \code{g} or \code{h} are omitted, the gradient or Hessian will be respectively
be computed via finite differences.
}
\examples{
f = function(x) { sum(x^2) }
g = function(x) { 2*x }
h = function(x) { 2*diag(length(x)) }
x0 = c(1,1)

args = cg_args()
cg1(x0, f, g, args)
cg2(x0, f, args)

args = bfgs_args()
bfgs1(x0, f, g, args)
bfgs2(x0, f, args)

args = lbfgsb_args()
lbfgsb1(x0, f, g, args)
lbfgsb2(x0, f, args)

args = neldermead_args()
neldermead(x0, f, args)

args = nlm_args()
nlm1(x0, f, g, h, args)
nlm2(x0, f, g, args)
nlm3(x0, f, args)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{matrix_apply}
\alias{matrix_apply}
\alias{mat_apply}
\alias{row_apply}
\alias{col_apply}
\title{Matrix Apply Functions}
\usage{
mat_apply(X, f)

row_apply(X, f)

col_apply(X, f)
}
\arguments{
\item{X}{A matrix}

\item{f}{The function to apply.}
}
\value{
\code{mat_apply} returns a matrix. \code{row_apply} and \code{col_apply} return a vector.
See section "Apply" of the package vignette for details.
}
\description{
Matrix Apply Functions
}
\details{
The \code{mat_apply}, \code{row_apply}, and \code{col_apply} C++ functions are intended to
operate like the following calls in R, respectively.

\if{html}{\out{<div class="sourceCode">}}\preformatted{apply(x, c(1,2), f)
apply(x, 1, f)
apply(x, 2, f)
}\if{html}{\out{</div>}}

The R functions exposed here are specific to numeric-valued matrices, but
the underlying C++ functions are intended to work with any type of Rcpp
Matrix.
}
\examples{
X = matrix(1:12, nrow = 4, ncol = 3)
mat_apply(X, f = function(x) { x^(1/3) })
row_apply(X, f = function(x) { sum(x^2) })
col_apply(X, f = function(x) { sum(x^2) })

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{solve_cg}
\alias{solve_cg}
\title{Iteratively Solve a Linear System with Conjugate Gradient}
\usage{
solve_cg(l, b, init, args)
}
\arguments{
\item{l}{A linear transformation of \eqn{x}.}

\item{b}{A vector.}

\item{init}{Initial value of solution.}

\item{args}{List of additional arguments from \code{cg_args}.}
}
\value{
A list with the form of a \code{solve_cg_result} described in section "Conjugate
Gradient" of the package vignette.
}
\description{
Solve the system \eqn{l(x) = b} where \eqn{l(x)} is a matrix-free
representation of the linear operation \eqn{Ax}.
}
\examples{
set.seed(1234)

n = 8
idx_diag = cbind(1:n, 1:n)
idx_ldiag = cbind(2:n, 1:(n-1))
idx_udiag = cbind(1:(n-1), 2:n)
b = rep(1, n)

## Solution by explicit computation of solve(A, b)
A = matrix(0, n, n)
A[idx_diag] = 2
A[idx_ldiag] = 1
A[idx_udiag] = 1
solve(A, b)

## Solve iteratively with solve_cg
f = function(x) { A \%*\% x }
args = cg_args()
init = rep(0, n)
solve_cg(f, b, init, args)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/package.R
\docType{package}
\name{fntl-package}
\alias{fntl}
\alias{fntl-package}
\title{fntl}
\description{
Package documentation
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/andrewraim/fntl}
}

}
\author{
\strong{Maintainer}: Andrew M. Raim \email{andrew.raim@gmail.com} (\href{https://orcid.org/0000-0002-4440-2330}{ORCID})

Authors:
\itemize{
  \item Andrew M. Raim \email{andrew.raim@gmail.com} (\href{https://orcid.org/0000-0002-4440-2330}{ORCID})
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{gradient0}
\alias{gradient0}
\title{Numerical Gradient Vector}
\usage{
gradient0(f, x, args)
}
\arguments{
\item{f}{Function to differentiate.}

\item{x}{Vector at which to evaluate the gradient.}

\item{args}{List of additional arguments from the function \code{richardson_args}.}
}
\value{
A list with the form of a \code{gradient_result} described in section "Gradient"
of the package vignette.
}
\description{
Numerical Gradient Vector
}
\examples{
f = function(x) { sum(sin(x)) }
args = richardson_args()
x0 = seq(0, 1, length.out = 5)
cos(x0)  ## Exact answer
gradient0(f, x0, args)
numDeriv::grad(f, x0)

}
